C***********************************************************************
C        this package contains both colcon and colpar                   
C        in the version of oct 12 1987                                  
C        by g. bader and p. kunkel                                      
C***********************************************************************
C---------------------------------------------------------------------- 
C             p a r t  1   of COLCON                                    
C        main storage allocation and program control subroutines        
C---------------------------------------------------------------------- 
C                                                                       
      SUBROUTINE COLCON (NCOMP, M, ALEFT, ARIGHT, ZETA, INFO, LTOL,     
     1                   TOL, FIXPNT, WINDOW, ISPACE, FSPACE, IFLAG,    
     2                   FSUB, DFSUB, GSUB, DGSUB, GUESS)               
C                                                                       
C                                                                       
C********************************************************************** 
C                                                                       
C     written by                                                        
C                  g. bader and p. kunkel                               
C                            institut f. angewandte mathematik          
C                            university of heidelberg                   
C                            im neuenheimer feld 294                    
C                            d-6900 heidelberg 1, frg.                  
C                                                                       
C********************************************************************** 
C                                                                       
C     purpose                                                           
C                                                                       
C     colcon is a package solving continuation for multi-               
C     point boundary value given by a mixed order system                
C     of ode-s denoted by                                               
C                                                                       
C          (m(i))                                                       
C         u       =  f  ( x; z(u(x),par) )      i = 1, ... ,ncomp       
C          i          i                                                 
C                                                                       
C                                          aleft .lt. x .lt. aright,    
C                                                                       
C                                                                       
C         g  ( zeta(j); z(u(zeta(j),par)) ) = 0   j = 1, ... ,mside     
C          j                                                            
C                                    mstar = m(1)+m(2)+...+m(ncomp),    
C                                    mside = mstar + npar - 1,          
C                                    mstate = mstar + npar,             
C                                                                       
C                                                                       
C         where                       t                                 
C            u = (u , u , ... ,u     )   is the exact solution vector   
C                  1   2        ncomp                                   
C                                                                       
C             (mi)                                                      
C            u     is the mi=m(i) th  derivative of u                   
C             i                                      i                  
C                                                                       
C                                 (1)        (m1-1)       (mncomp-1)    
C           z(u(x),par) = (u (x),u  (x),...,u    (x),...,u      (x),par)
C                           1     1          1            ncomp         
C                                                                       
C           f (x,z(u,par))   is a (generally) nonlinear function of     
C            i                                                          
C                            z(u,par)=z(u(x),par).                      
C                                                                       
C           g (zeta(j);z(u,par))  is a (generally) nonlinear function   
C            j                                                          
C                               used to represent a boundary condition. 
C                                                                       
C         the boundary points satisfy                                   
C             aleft .le. zeta(1) .le. .. .le. zeta(mside) .le. aright   
C                                                                       
C         the orders mi of the differential equations satisfy           
C                            1 .le. m(i) .le. 4.                        
C                                                                       
C         continuation is done with respect to parameter par(icon)      
C         in the range                                                  
C                 window(1) .le. par(icon) .le. window(3)               
C         starting with the value  par(icon) = window(2).               
C                                                                       
C         the initial point of continuation has to be regular           
C         with respect to the usual parameter embedding.                
C                                                                       
C********************************************************************** 
C                                                                       
C     discretization method                                             
C                                                                       
C        the method used to approximate the solution u is               
C     collocation at gaussian points, requiring m(i)-1 continuous       
C     derivatives in the i-th component, i = 1, ..., ncomp.             
C     here, k is the number of collocation points (stages) per          
C     subinterval and is chosen such that k .ge. max m(i).              
C     a runge-kutta-monomial basis is utilized.                         
C                                                                       
C     continuation method                                               
C                                                                       
C        the method used is tangent continuation for a one              
C     parameter embedding. the steplength is controled with             
C     respect to the nonlinearity of the problem.                       
C                                                                       
C     references                                                        
C                                                                       
C     [1] u. ascher, j. christiansen and r.d. russell,                  
C         collocation software for boundary-value odes,                 
C         acm trans. math software 7 (1981), 209-222.                   
C                                                                       
C     [2] g. bader and u. ascher,                                       
C         a new basis implementation for a mixed order                  
C         boundary value ode solver,                                    
C         tech. rep. 85-11, dept. computer science, univ. b.c.,         
C         vancouver, canada. 1985.                                      
C                                                                       
C     [3] u. ascher, j. christiansen and r.d. russell,                  
C         a collocation solver for mixed order                          
C         systems of boundary value problems,                           
C         math. comp. 33 (1979), 659-679.                               
C                                                                       
C     [4] u. ascher, j. christiansen and r.d. russell,                  
C         colsys - a collocation code for boundary                      
C         value problems,                                               
C         lecture notes comp.sc. 76, springer verlag,                   
C         b. childs et. al. (eds.) (1979), 164-185.                     
C                                                                       
C     [5] c. deboor and r. weiss,                                       
C                                                                       
C         solveblok: a package for solving almost block diagonal        
C         linear systems,                                               
C         acm trans. math. software 6 (1980), 80-87.                    
C                                                                       
C     [6] g. bader and p. kunkel,                                       
C                                                                       
C         continuation and collocation for parameter dependent          
C         boundary value problems,                                      
C         siam j. sci. stat. comp. 10 (1989), 72-88.        
C                                                                       
C********************************************************************** 
C                                                                       
C     ***************     input to colcon     ***************           
C                                                                       
C     variables                                                         
C                                                                       
C     ncomp - no. of differential equations   (ncomp .le. 20)           
C                                                                       
C     m(j) - order of the j-th differential equation                    
C            ( mstate = m(1) + ... + m(ncomp) + npar .le. 40 )          
C                                                                       
C     aleft - left end of interval                                      
C                                                                       
C     aright - right end of interval                                    
C                                                                       
C     zeta(j) - j-th side condition point (boundary point). must        
C               have  zeta(j) .le. zeta(j+1). all side condition        
C               points must be mesh points in all meshes used,          
C               see description of info(11) and fixpnt below.           
C                                                                       
C     info - an integer array dimensioned at least 16.                  
C            a list of the parameters in info and their meaning follows 
C            some parameters are renamed in colcon; these new names are 
C            given in parentheses.                                      
C                                                                       
C     info(1)     ( = nonlin )                                          
C             = 0 if the problem is linear                              
C             = 1 if the problem is nonlinear                           
C                                                                       
C     info(2) = no. of collocation points per subinterval  (= k )       
C               where max m(i) .le.  k .le. 7 . if info(2)=0 then       
C               colcon sets  k = max ( max m(i)+1, 5-max m(i) )         
C                                                                       
C     info(3) = no. of subintervals in the initial mesh  ( = n ).       
C               if info(3) = 0 then colcon arbitrarily sets n = 5.      
C                                                                       
C     info(4) = no. of solution and derivative tolerances.  ( = ntol )  
C               we require  0 .lt. ntol .le. mstar.                     
C                                                                       
C     info(5) = dimension of fspace.     ( = ndimf )                    
C                                                                       
C     info(6) = dimension of ispace.     ( = ndimi )                    
C                                                                       
C     info(7) -  output control ( = iprint )                            
C              = -1 for full diagnostic printout                        
C              = 0 for selected printout                                
C              = 1 for no printout                                      
C                                                                       
C     info(8)     ( = iread )                                           
C             = 0 causes colcon to generate a uniform initial mesh.     
C             = 1 if the initial mesh is provided by the user.  it      
C                 is defined in fspace as follows:  the mesh            
C                 aleft=x(1).lt.x(2).lt. ... .lt.x(n).lt.x(n+1)=aright  
C                 will occupy  fspace(1), ..., fspace(n+1). the         
C                 user needs to supply only the interior mesh           
C                 points  fspace(j) = x(j), j = 2, ..., n.              
C             = 2 if the initial mesh is supplied by the user           
C                 as with info(8)=1, and in addition no adaptive        
C                 mesh selection is to be done.                         
C                                                                       
C     info(9)     ( = iguess )                                          
C             = 0 if no initial guess for the solution is               
C                 provided.                                             
C             = 1 if an initial guess is provided by the user           
C                 in subroutine  guess.                                 
C             = 2 if an initial mesh and approximate solution           
C                 coefficients are provided by the user in  fspace.     
C                 (the former and new mesh are the same).               
C             = 3 if a former mesh and an approximate solution          
C                 coefficients are provided by the user in fspace,      
C                 and the new mesh is to be taken twice as coarse;      
C                 i.e.,every second point from the former mesh.         
C             = 4 if in addition to a former initial mesh and an        
C                 approximate solution coefficients, a new mesh         
C                 is provided in fspace as well.                        
C                 (see description of output for further details        
C                 on iguess = 2, 3, and 4.)                             
C                                                                       
C     info(10)= 0 if the problem is regular                             
C             = 1 if the first relax factor is =rstart, and the         
C                 nonlinear iteration does not rely on past covergence  
C                 (use for an extra sensitive nonlinear problem only).  
C             = 2 if we are to return immediately upon  (a) two         
C                 successive nonconvergences, or  (b) after obtaining   
C                 error estimate for the first time.                    
C                                                                       
C     info(11)= no. of fixed points in the mesh other than aleft        
C               and aright. ( = nfxpnt , the dimension of fixpnt)       
C               the code requires that all side condition points        
C               other than aleft and aright (see description of         
C               zeta ) be included as fixed points in fixpnt.           
C                                                                       
C     info(12)    ( = npar )                                            
C               no. of parameters appearing in the problem.             
C                                                                       
C     info(13)    ( = icon )                                            
C               icon-th parameter is continuation parameter             
C               where  1 .le. icon .le. npar.                           
C                                                                       
C     info(14)    ( = idirec )                                          
C               initial direction for continuation.                     
C            = 1 starting in positive parameter direction.              
C            =-1 starting in negative parameter direction.              
C                                                                       
C     info(15)    ( = maxstp )                                          
C               maximal number of continuation steps.                   
C               if info(15) = 0 then colcon sets maxstp = 50.           
C                                                                       
C     info(16)    ( = ijob )                                            
C              not used in this version.                                
C                                                                       
C     ltol  -  an array of dimension info(4). ltol(j) = l  specifies    
C              that the j-th tolerance in  tol  controls the error      
C              in the l-th component of z(u).   also require that       
C              1.le.ltol(1).lt.ltol(2).lt. ... .lt.ltol(ntol).le.mstar  
C                                                                       
C     tol    - an array of dimension info(4). tol(j) is the             
C              error tolerance on the ltol(j) -th component             
C              of z(u). thus, the code attempts to satisfy              
C              for j=1,...,ntol  on each subinterval                    
C              abs(z(v)-z(u))       .le. tol(j)*abs(z(u))       +tol(j) 
C                            ltol(j)                     ltol(j)        
C                                                                       
C              if v(x) is the approximate solution vector.              
C                                                                       
C     fixpnt - an array of dimension info(11).   it contains            
C              the points, other than aleft and aright, which           
C              are to be included in every mesh.                        
C                                                                       
C     window - an array of dimension 3.    it contains the              
C              specifications on interval of continuation               
C              parameter par(icon).                                     
C              window(1) = left endpoint of the interval                
C              window(2) = starting point for continuation              
C              window(3) = right endpoint of interval.                  
C              also require that                                        
C              window(1) .le. window(2) .le. window(3) .                
C                                                                       
C     ispace - an integer work array of dimension info(6).              
C              its size provides a constraint on nmax,                  
C              the maximum number of subintervals. choose               
C              info(6) according to the formula                         
C                      info(6)  .ge.  nmax*nsizei + mside               
C                where                                                  
C                      nsizei = 3 + kdm                                 
C                with                                                   
C                      kdm = kd + mstar  ;  kd = k * ncomp ;            
C                      nrec = no. of right end boundary conditions.     
C                                                                       
C                                                                       
C     fspace - a real work array of dimension info(5).                  
C              its size provides a constraint on nmax.                  
C              choose info(5) according to the formula                  
C                      info(5)  .ge.  nmax*nsizef + nfixf               
C                where                                                  
C                      nsizef = 5 + 3/2 * kdm + kd * (kdm + npar + 6) + 
C                               mstar*(k+npar+8+2*(2*(mstar+npar)-nrec))
C                      nfixf  = 3 + mstate * (npar + 8) +               
C                               2 * mstar * (npar + nrec).              
C                                                                       
C                                                                       
C     iflag - the mode of return from colcon.                           
C           = 1 for normal return                                       
C           = 0 if the collocation matrix is singular.                  
C           =-1 if the expected no. of subintervals exceeds storage     
C               specifications.                                         
C           =-2 if the nonlinear iteration has not converged.           
C           =-3 if there is an input data error.                        
C           =-4 if there is no regular starting point.                  
C           =-5 if maximal number of continuation steps exceeded.       
C           =-6 if continuation stepsize too small                      
C                                                                       
C                                                                       
C********************************************************************** 
C                                                                       
C     *************    user supplied subroutines   *************        
C                                                                       
C                                                                       
C     the following subroutines must be declared external in the        
C     main program which calls colcon.                                  
C                                                                       
C                                                                       
C     fsub  - name of subroutine for evaluating f(x,z(u(x),par)) =      
C                            t                                          
C             (f ,...,f     )   at a point x in (aleft,aright).         
C               1      ncomp                                            
C             it should have the heading                                
C                                                                       
C                       subroutine fsub (x , z , f)                     
C                                                                       
C             where f is the vector containing the value of             
C             fi(x,z(u(x),par)) in the i-th component and               
C                                                                 t     
C             z(u(x),par)=(z(1),...,z(mstar),par(1),...,par(npar))      
C             is defined as above under  purpose .                      
C                                                                       
C                                                                       
C     dfsub - name of subroutine for evaluating the jacobian of         
C             f(x,z(u,par)) at a point x.  it should have the           
C             heading                                                   
C                                                                       
C                       subroutine dfsub (x , z , df)                   
C                                                                       
C             where z(u(x),par) is defined as for fsub and the (ncomp)  
C             by (mstate) array df should be filled by the partial      
C             derivatives of f, viz, for a particular call one cal-     
C             culates                                                   
C                                df(i,j) = dfi / dzj, i=1,...,ncomp     
C                                                     j=1,...,mstate.   
C                                                                       
C                                                                       
C     gsub  - name of subroutine for evaluating the i-th component of   
C             g(x,z(u(x),par)) = g (zeta(i),z(u(zeta(i)),par))          
C                             i                                         
C              at a point x = zeta(i) where 1.le.i.le.mside.            
C              it should have the heading                               
C                                                                       
C                       subroutine gsub (i , z , g)                     
C                                                                       
C             where z is as for fsub, and i and g=g  are as above.      
C                                                  i                    
C             note that in contrast to f in  fsub , here                
C             only one value per call is returned in g.                 
C                                                                       
C                                                                       
C     dgsub - name of subroutine for evaluating the i-th row of         
C             the jacobian of g(x,u(x)).  it should have the heading    
C                                                                       
C                       subroutine dgsub (i , z , dg)                   
C                                                                       
C             where z is as for fsub, i as for gsub and the mside-      
C             vector dg should be filled with the partial derivatives   
C             of g, viz, for a particular call one calculates           
C                   dg(i,j) = dgi / dzj      i=1,...,mside              
C                                            j=1,...,mstate.            
C                                                                       
C                                                                       
C     guess - name of subroutine to evaluate the initial                
C             approximation for  z(u(x),par) and for dmval(u(x),par)=   
C             vector of the mj-th derivatives of u(x). it should        
C             have the heading                                          
C                                                                       
C                       subroutine guess (x , z , dmval)                
C                                                                       
C             note that this subroutine is needed only if using         
C             info(9) = 1, and then all  mstate  components of z        
C             and  ncomp  components of  dmval  should be specified     
C             for any x,  aleft .le. x .le. aright ,                    
C             with the exception of the continuation parameter          
C             par(icon) = z(mstar+icon) which is given by               
C             window(2)  (see above) .                                  
C                                                                       
C                                                                       
C********************************************************************** 
C                                                                       
C     ********   use of output from colcon or colpar  ********          
C                                                                       
C                 ***   solution evaluation   ***                       
C                                                                       
C     on return from colcon or colpar, the arrays fspace and ispace     
C     contain information specifying the approximate solution.          
C     the user can produce the solution vector z( u(x),par ) at         
C     any point x, aleft .le. x .le. aright, by the statement,          
C                                                                       
C           call appsln (x, z, fspace, ispace)                          
C                                                                       
C     when saving the coefficients for later reference, only            
C     ispace(1),...,ispace(7+ncomp)    and                              
C     fspace(1),...,fspace(ispace(7))    need to be saved as            
C     these are the quantities used by appsln.                          
C                                                                       
C       ********   combining colcon and colpar  ********                
C                                                                       
C     the code colpar may be used to provide an initial solution        
C     for a continuation run with colcon.                               
C     a formerly obtained solution can easily be used as the            
C     first approximation for the nonlinear iteration for a             
C     new problem by setting   (iguess =) info(9) = 2, 3 or 4.          
C                                                                       
C     if the former solution has just been obtained then the            
C     values needed to define the first approximation are               
C     already in ispace and fspace.                                     
C     alternatively, if the former solution was obtained in a           
C     previous run and its coefficients were saved then those           
C     coefficients must be put back into                                
C     ispace(1),..., ispace(7+ncomp)    and                             
C     fspace(1),..., fspace(ispace(7)).                                 
C                                                                       
C     for info(9) = 2 or 3 set info(3) = ispace(1) ( = the              
C     size of the previous mesh ).                                      
C                                                                       
C     for info(9) = 4 the user specifies a new mesh of n subintervals   
C     as follows.                                                       
C     the values in  fspace(1),...,fspace(ispace(7))  have to be        
C     shifted by n+1 locations to  fspace(n+2),..,fspace(ispace(7)+n+1) 
C     and the new mesh is then specified in fspace(1),..., fspace(n+1). 
C     also set info(3) = n.                                             
C                                                                       
C                                                                       
C********************************************************************** 
C                                                                       
C     ***************      package subroutines      ***************     
C                                                                       
C     the following description gives a brief overview of how the       
C     procedure is broken down into the subroutines which make up       
C     the package called  colcon . for further details the              
C     user should refer to documentation in the various subroutines     
C     and to the references cited above.                                
C                                                                       
C     the subroutines fall into four groups:                            
C                                                                       
C part 1 - the main storage allocation and program control subroutines  
C                                                                       
C     colcon - tests input values, does initialization and breaks up    
C              the work areas, fspace and ispace, into the arrays       
C              used by the program.                                     
C                                                                       
C     pthcon - is the actual driver of the package. this routine        
C              contains the strategy controling the continuation        
C              process.                                                 
C                                                                       
C     corrct - this routine performs the nonlinear equation solving.    
C                                                                       
C     sclcon - provides scaling for the control                         
C              of convergence in the nonlinear iteration.               
C                                                                       
C                                                                       
C part 2 - mesh selection and error estimation subroutines              
C                                                                       
C     consts - is called once by  colcon  to initialize constants       
C              which are used for error estimation and mesh selection.  
C                                                                       
C     newmsh - generates meshes. it contains the test to decide         
C              whether or not to redistribute a mesh.                   
C                                                                       
C     errchk - produces error estimates and checks against the          
C              tolerances at each subinterval                           
C                                                                       
C                                                                       
C part 3 - collocation system set-up subroutines                        
C                                                                       
C     lsyslv - controls the set-up and solution of the linear           
C              algebraic systems of collocation equations which         
C              arise at each newton iteration.                          
C                                                                       
C     gderiv - is used by lsyslv to set up the equation associated      
C              with a side condition point.                             
C                                                                       
C     vwblok - is used by lsyslv to set up the equation(s) associated   
C              with a collocation point.                                
C                                                                       
C     gblock - is used by lsyslv to construct a block of the global     
C              collocation matrix or the corresponding right hand       
C              side.                                                    
C                                                                       
C     tangnt - controls the computation of a tangent approximation.     
C                                                                       
C                                                                       
C part 4 - service subroutines                                          
C                                                                       
C     appsln - sets up a standard call to  approx .                     
C                                                                       
C     user   - provides solution information for each                   
C              continuation step.                                       
C                                                                       
C     approx - evaluates a piecewise polynomial solution.               
C                                                                       
C     rkbas  - evaluates the mesh independent runge-kutta basis         
C                                                                       
C     vmonde - solves a vandermonde system for given right hand         
C              side                                                     
C                                                                       
C     horder - evaluates the highest order derivatives of the           
C              current collocation solution used for mesh refinement.   
C                                                                       
C     dmzsol - evaluates m-th order derivatives of the solution.        
C                                                                       
C     l2dot  - computes approximation for l2 dot product.               
C                                                                       
C     zint   - interpolates solution on new mesh.                       
C                                                                       
C                                                                       
C part 5 - linear algebra  subroutines                                  
C                                                                       
C     to solve the global linear systems of collocation equations       
C     constructed in part 3,  colcon  uses a column oriented version    
C     of the package  solveblok due to de boor and weiss.               
C     additionally routines performing column interchanges instead      
C     of row interchanges and for the computation of determinants       
C     are included.                                                     
C                                                                       
C     to solve the linear systems for static parameter condensation     
C     in each block of the collocation equations, the linpack           
C     routines  dgefa and  dgesl  are included. but these               
C     may be replaced when solving problems on vector processors        
C     or when solving large scale sparse jacobian problems.             
C                                                                       
C---------------------------------------------------------------------- 
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION M(1), ZETA(1), INFO(1), LTOL(1), TOL(1), DUMMY(1),      
     1          FIXPNT(1), WINDOW(1), ISPACE(1), FSPACE(1)              
C                                                                       
      COMMON /COLOUT/ PRECIS, IOUT, IPRINT                              
      COMMON /COLLOC/ RHO(7), COEF(49)                                  
      COMMON /COLAPR/ N, NOLD, NMAX, NZ, NZPAR, NDMZ                    
      COMMON /COLORD/ K, NC, MSTAR, NPAR, MSTATE, KD, MMAX, MT(20)      
      COMMON /COLMSH/ MSHFLG, MSHNUM, MSHLMT, MSHALT                    
      COMMON /COLSID/ TZETA(40), TLEFT, TRIGHT, MSIDE, IZETA, IDUM      
      COMMON /COLNLN/ NONLIN, ITER, LIMIT, ICARE, IGUESS, IREAD         
      COMMON /COLPTH/ IPATH, ICON, LPAR                                 
      COMMON /COLPRM/ TWIN(3), IDIREC, MAXSTP, IJOB                     
      COMMON /COLEST/ WGTMSH(40), WGTERR(40), TTOL(40),                 
     1                ROOT(40), JTOL(40), LTTOL(40), NTOL               
C                                                                       
      EXTERNAL FSUB, DFSUB, GSUB, DGSUB, GUESS                          
C                                                                       
C*********************************************************************  
C                                                                       
C     the actual subroutine colcon serves as an interface with          
C     the package of subroutines referred to collectively as            
C     colpar. the subroutine serves to test some of the input           
C     parameters, rename some of the parameters (to make under-         
C     standing of the coding easier), to do some initialization,        
C     and to break the work areas fspace and ispace up into the         
C     arrays needed by the program.                                     
C                                                                       
C********************************************************************** 
C                                                                       
C...  specify machine dependent output unit  iout  and compute machine  
C...  dependent constant  precis = 100 * machine unit roundoff          
C                                                                       
      IOUT = 6                                                          
C                                                                       
      PRECIS = 1.D0                                                     
   10 PRECIS = PRECIS / 2.D0                                            
      PRECP1 = PRECIS + 1.D0                                            
      IF ( PRECP1 .GT. 1.D0 )                       GO TO 10            
      PRECIS = PRECIS * 100.D0                                          
C                                                                       
C...  in case incorrect input data is detected, the program returns     
C...  immediately with iflag=-3.                                        
C                                                                       
      IFLAG = -3                                                        
      IF ( NCOMP .LT. 1 .OR. NCOMP .GT. 20 )        RETURN              
      DO 20 I=1,NCOMP                                                   
         IF ( M(I) .LT. 1 .OR. M(I) .GT. 4 )        RETURN              
   20 CONTINUE                                                          
C                                                                       
C...  rename some of the parameters and set default values.             
C                                                                       
      NONLIN = INFO(1)                                                  
      K = INFO(2)                                                       
      N = INFO(3)                                                       
      IF ( N .EQ. 0 )  N = 5                                            
      ICARE = INFO(10)                                                  
      NTOL = INFO(4)                                                    
      NDIMF = INFO(5)                                                   
      NDIMI = INFO(6)                                                   
      IPRINT = INFO(7)                                                  
      IREAD = INFO(8)                                                   
      IGUESS = INFO(9)                                                  
      NPAR = INFO(12)                                                   
      ICON = INFO(13)                                                   
      IDIREC = INFO(14)                                                 
      IF ( IDIREC .GE. 0 )  IDIREC = 1                                  
      IF ( IDIREC .LT. 0 )  IDIREC = -1                                 
      MAXSTP = INFO(15)                                                 
      IF ( MAXSTP .LE. 0 )  MAXSTP = 50                                 
      IJOB = 0                                                          
      IF ( NONLIN .EQ. 0 .AND. IGUESS .EQ. 1 )  IGUESS = 0              
      IF ( IGUESS .GE. 2 .AND. IREAD .EQ. 0 )   IREAD = 1               
      NFXPNT = INFO(11)                                                 
      MSTAR = 0                                                         
      MMAX = 0                                                          
      DO  30 I = 1, NCOMP                                               
         MMAX = MAX0 ( MMAX, M(I) )                                     
         MSTAR = MSTAR + M(I)                                           
         MT(I) = M(I)                                                   
   30 CONTINUE                                                          
      MSIDE = MSTAR + NPAR - 1                                          
      MSTATE = MSTAR + NPAR                                             
      IF ( K .EQ. 0 )   K = MAX0( MMAX + 1 , 5 - MMAX )                 
      DO 40 I = 1, MSIDE                                                
   40 TZETA(I) = ZETA(I)                                                
      DO 50 I = 1, NTOL                                                 
         LTTOL(I) = LTOL(I)                                             
   50 TTOL(I) = TOL(I)                                                  
      TLEFT = ALEFT                                                     
      TRIGHT = ARIGHT                                                   
      TWIN(1) = WINDOW(1)                                               
      TWIN(2) = WINDOW(2)                                               
      TWIN(3) = WINDOW(3)                                               
      NC = NCOMP                                                        
      KD = K * NCOMP                                                    
C                                                                       
C...  print the input data for checking.                                
C                                                                       
      IF ( IPRINT .LT. 1 )  WRITE(IOUT,260)                             
      IF ( IPRINT .GT. -1 )                         GO TO 80            
      IF ( NONLIN .GT. 0 )                          GO TO 60            
      WRITE (IOUT,270) NCOMP, (M(IP), IP=1,NCOMP)                       
      GO TO 70                                                          
   60 WRITE (IOUT,280) NCOMP, (M(IP), IP=1,NCOMP)                       
   70 WRITE (IOUT,290) NPAR                                             
      WRITE (IOUT,300) (ZETA(IP), IP=1,MSIDE)                           
      IF ( NFXPNT .GT. 0 )                                              
     1   WRITE (IOUT,310) NFXPNT, (FIXPNT(IP), IP=1,NFXPNT)             
      WRITE (IOUT,320) K                                                
      WRITE (IOUT,330) (LTOL(IP), IP=1,NTOL)                            
      WRITE (IOUT,340) (TOL(IP), IP=1,NTOL)                             
      IF (IGUESS .GE. 2) WRITE (IOUT,350)                               
      IF (IREAD .EQ. 2) WRITE (IOUT,360)                                
      WRITE(IOUT,370) ICON                                              
      WRITE(IOUT,380) TWIN(1),TWIN(3)                                   
      IF ( IDIREC .EQ. 1 )  WRITE(IOUT,390) WINDOW(2)                   
      IF ( IDIREC .EQ.-1 )  WRITE(IOUT,400) WINDOW(2)                   
      WRITE(IOUT,410) MAXSTP                                            
   80 CONTINUE                                                          
C                                                                       
C...  check for correctness of data                                     
C                                                                       
      IF ( K .LT. 0 .OR. K .GT. 7 )                 RETURN              
      IF ( N .LT. 0 )                               RETURN              
      IF ( IREAD .LT. 0 .OR. IREAD .GT. 2 )         RETURN              
      IF ( IGUESS .LT. 0 .OR. IGUESS .GT. 4 )       RETURN              
      IF ( ICARE .LT. 0 .OR. ICARE .GT. 2 )         RETURN              
      IF ( NTOL .LT. 0 .OR. NTOL .GT. MSTAR )       RETURN              
      IF ( NFXPNT .LT. 0 )                          RETURN              
      IF ( IPRINT .LT. (-1) .OR. IPRINT .GT. 1 )    RETURN              
      IF ( NPAR .LT. 1 )                            RETURN              
      IF ( MSTAR .LT. 0 .OR. MSTATE .GT. 40 )       RETURN              
      IP = 1                                                            
      DO 110 I = 1, MSIDE                                               
      IF ( DABS(ZETA(I) - ALEFT) .LT. PRECIS .OR.                       
     1     DABS(ZETA(I) - ARIGHT) .LT. PRECIS )     GO TO 110           
   90 IF ( IP .GT. NFXPNT )                         RETURN              
        IF ( ZETA(I) - PRECIS .LT. FIXPNT(IP) )     GO TO 100           
        IP = IP + 1                                                     
      GO TO 90                                                          
  100 IF ( ZETA(I) + PRECIS .LT. FIXPNT(IP) )       RETURN              
  110 CONTINUE                                                          
      IF ( ICON .LT. 1 .OR. ICON .GT. NPAR )        RETURN              
      IF ( WINDOW(1) .GT. WINDOW(3) )               RETURN              
      IF ( WINDOW(2) .LT. WINDOW(1) )               RETURN              
      IF ( WINDOW(2) .GT. WINDOW(3) )               RETURN              
      IF ( WINDOW(2) - PRECIS .LT. WINDOW(1)                            
     1     .AND. IDIREC .EQ. -1 )                   RETURN              
      IF ( WINDOW(2) + PRECIS .GT. WINDOW(3)                            
     1     .AND. IDIREC .EQ. 1 )                    RETURN              
C                                                                       
C...  compute the maxium possible n for the given sizes of              
C...  ispace  and  fspace.                                              
C                                                                       
      NREC = 0                                                          
      DO 120 I = 1, MSIDE                                               
           IB = MSIDE + 1 - I                                           
           IF ( ZETA(IB)-PRECIS .GE. ARIGHT )  NREC = I                 
  120 CONTINUE                                                          
      NFIXI = MSIDE                                                     
      NSIZEI = 3 + KD + MSTAR                                           
      NFIXF = 2 * MSTAR * ( NPAR + NREC ) + MSTATE * ( NPAR + 7 )       
     1        + MSIDE + 4                                               
      NSIZEF = 2 * MSTAR * (MSTAR + MSIDE - NREC) + MSTAR * K           
     1        + KD * (KD + MSTATE + 6) + MSTAR * (NPAR + 8) + 4         
      NSIZEF = 2 * NSIZEF + 3 * ( KD + MSTAR ) + 1                      
      NHALF = (NDIMF - NFIXF) / NSIZEF                                  
      NMAXF = 2 * NHALF                                                 
      NMAXI = (NDIMI - NFIXI) / NSIZEI                                  
      IF ( IPRINT .LT. 1 )  WRITE(IOUT,420) NMAXF, NMAXI                
      NMAX = MIN0( NMAXF, NMAXI )                                       
      NHALF = ( NMAX + 1 )  / 2                                         
      IF ( NMAX .LT. N )                            RETURN              
      IF ( NMAX .LT. NFXPNT+1 )                     RETURN              
      IF (NMAX .LT. 2*NFXPNT+2 .AND. IPRINT .LT. 1)  WRITE(IOUT,430)    
C                                                                       
C...  generate pointers to break up  fspace  and  ispace .              
C                                                                       
      NMAX1 = NMAX + 1                                                  
      KDN = KD * NMAX                                                   
      NHALF1 = NHALF + 1                                                
      KDNH = KD * NHALF                                                 
C                                                                       
      LXI = 1                                                           
      LG = LXI + NMAX1                                                  
      LXIOLD = LG + 2*MSTAR * (NMAX * (MSTAR+MSIDE-NREC) + NREC)        
     1         + NPAR * MSTAR * NMAX1 + NPAR * (MSTAR + MSIDE)          
      LW     = LXIOLD + NMAX1                                           
      LV     = LW + KD * KDN                                            
      LZ     = LV + MSTATE * KDN                                        
      LDMZ   = LZ + MSTAR * NMAX1 + NPAR                                
      LDELZ  = LDMZ + KDN                                               
      LDELDZ = LDELZ + MSTAR * NMAX1 + NPAR                             
      LDQZ   = LDELDZ + KDN                                             
      LDQDMZ = LDQZ + MSTAR * NMAX1 + NPAR                              
      LXI0   = LDQDMZ + KDN                                             
      LZ0    = LXI0 + NHALF1                                            
      LDMZ0  = LZ0 + MSTAR * NHALF1 + NPAR                              
      LTGT0  = LDMZ0 + KDNH                                             
      LDMTG0 = LTGT0 + MSTAR * NHALF1 + NPAR                            
      LTGT   = LDMTG0 + KDNH                                            
      LDMTGT = LTGT + MSTAR * NMAX1 + NPAR                              
      LZOLD  = LDMTGT + KDN                                             
      LDMZOL = LZOLD + MSTAR * NHALF1 + NPAR                            
      LRHS   = LDMZOL + KDNH                                            
      LVALST = LRHS   + KDN + MSIDE                                     
      LSLOPE = LVALST + 4 * MSTAR * NMAX                                
      LACCUM = LSLOPE + NMAX                                            
      LSCL   = LACCUM + NMAX1                                           
      LDSCL  = LSCL + MSTAR * K * NMAX + NPAR                           
      LPVTG  = 1                                                        
      LPVTW  = LPVTG + MSTAR * NMAX + MSIDE                             
      LINTEG = LPVTW + KDN                                              
C                                                                       
C...  if  iguess .ge. 2, move  xiold, z, and  dmz  to their proper      
C...  locations in  fspace.                                             
C                                                                       
      IF ( IGUESS .LT. 2 )                          GO TO 180           
      NOLD = N                                                          
      NPOLD = ISPACE(6)                                                 
      IF (IGUESS .EQ. 4)  NOLD = ISPACE(1)                              
      NZPAR = MSTAR * (NOLD + 1) + NPAR                                 
      NZPOLD = MSTAR * (NOLD + 1) + NPOLD                               
      NDMZ = KD * NOLD                                                  
      NP1 = N + 1                                                       
      IF ( IGUESS .EQ. 4 )  NP1 = NP1 + NOLD + 1                        
      DO 130 I=1,NZPOLD                                                 
  130 FSPACE( LZ+I-1 )  =  FSPACE( NP1+I )                              
      IDMZ = NP1 + NZPOLD                                               
      DO 140 I=1,NDMZ                                                   
  140 FSPACE( LDMZ+I-1 )  =  FSPACE( IDMZ+I )                           
      NP1 = NOLD + 1                                                    
      IF ( IGUESS .EQ. 4 )                          GO TO 160           
      DO 150 I=1,NP1                                                    
  150 FSPACE( LXIOLD+I-1 )  =  FSPACE( LXI+I-1 )                        
      GO TO 180                                                         
  160 DO 170 I=1,NP1                                                    
  170 FSPACE( LXIOLD+I-1 )  =  FSPACE( N+1+I )                          
  180 CONTINUE                                                          
C                                                                       
C...  initialize collocation points and error constants.                
C                                                                       
      CALL CONSTS ( K, RHO, COEF )                                      
C                                                                       
C...  determine first approximation, if the problem is nonlinear.       
C                                                                       
      IF ( NONLIN .EQ. 0  .OR. IGUESS .GE. 1 )      GO TO 210           
C                                                                       
C...  system provides first approximation of the solution.              
C...  choose z(j) = 0  for j=1,...,mstar.                               
C                                                                       
      NZPAR = ( N + 1 ) * MSTAR + NPAR                                  
      NDMZ = KD * N                                                     
      DO 190 I=1, NZPAR                                                 
  190 FSPACE( LZ-1+I ) = 0.D0                                           
      DO 200 I=1, NDMZ                                                  
  200 FSPACE( LDMZ-1+I ) = 0.D0                                         
  210 CONTINUE                                                          
C                                                                       
      NZ = ( N + 1 ) * MSTAR                                            
      FSPACE( LZ+NZ-1+ICON ) = WINDOW(2)                                
C                                                                       
      CALL PTHCON (FSPACE(LXI),FSPACE(LXIOLD),FSPACE(LZ),FSPACE(LDMZ),  
     1     FSPACE(LRHS),FSPACE(LDELZ),FSPACE(LDELDZ),FSPACE(LDQZ),      
     2     FSPACE(LDQDMZ),FSPACE(LG),FSPACE(LW),FSPACE(LV),             
     3     FSPACE(LVALST),FSPACE(LSLOPE),FSPACE(LSCL),FSPACE(LDSCL),    
     4     FSPACE(LXI0),FSPACE(LZ0),FSPACE(LDMZ0),FSPACE(LTGT0),        
     5     FSPACE(LDMTG0),FSPACE(LTGT),FSPACE(LDMTGT),                  
     6     FSPACE(LZOLD),FSPACE(LDMZOL),                                
     7     FSPACE(LACCUM),ISPACE(LPVTG),ISPACE(LINTEG),ISPACE(LPVTW),   
     8     NFXPNT,FIXPNT,IFLAG,FSUB,DFSUB,GSUB,DGSUB,GUESS)             
C                                                                       
C...  prepare output                                                    
C                                                                       
      IF ( IFLAG .EQ. 0 .OR. IFLAG .EQ. -4 )        RETURN              
C                                                                       
C... UPDATE DIMENSIONS                                                  
C                                                                       
      N = 2 * N                                                         
      NZPAR = (N+1) * MSTAR + NPAR                                      
      NDMZ = N * KD                                                     
C                                                                       
      ISPACE(1) = N                                                     
      ISPACE(2) = K                                                     
      ISPACE(3) = NCOMP                                                 
      ISPACE(4) = MSTAR                                                 
      ISPACE(5) = MMAX                                                  
      ISPACE(6) = NPAR                                                  
      LENGTH = N + 1 + NZPAR + NDMZ + K * K                             
      ISPACE(7) = LENGTH                                                
      DO 220 I = 1, NCOMP                                               
  220 ISPACE(7+I) = M(I)                                                
C                                                                       
      DO 230 I = 1, NZPAR                                               
  230 FSPACE( N+1+I ) = FSPACE( LZ-1+I )                                
      IDMZ = N + 1 + NZPAR                                              
      DO 240 I = 1, NDMZ                                                
  240 FSPACE( IDMZ+I ) = FSPACE( LDMZ-1+I )                             
      IC = IDMZ + NDMZ                                                  
      K2 = K * K                                                        
      DO 250 I = 1, K2                                                  
  250 FSPACE( IC+I ) = COEF(I)                                          
      RETURN                                                            
C---------------------------------------------------------------------- 
  260 FORMAT(47H1THIS IS  COLCON  IN THE VERSION FROM OCT. 1987//)      
  270 FORMAT(37H THE NUMBER OF (LINEAR) DIFF EQNS IS , I3/ 1X,          
     1       16HTHEIR ORDERS ARE, 20I3)                                 
  280 FORMAT(40H THE NUMBER OF (NONLINEAR) DIFF EQNS IS , I3/ 1X,       
     1       16HTHEIR ORDERS ARE, 20I3)                                 
  290 FORMAT(24H NUMBER OF PARAMETERS IS, I3)                           
  300 FORMAT(27H SIDE CONDITION POINTS ZETA, 8F10.6, 4( / 27X, 8F10.6)) 
  310 FORMAT(10H THERE ARE ,I5,27H FIXED POINTS IN THE MESH - ,         
     1       10(6F10.6/))                                               
  320 FORMAT(37H NUMBER OF COLLOC PTS PER INTERVAL IS, I3)              
  330 FORMAT(39H COMPONENTS OF Z REQUIRING TOLERANCES -,8(7X,I2,1X),    
     1       4(/38X,8I10))                                              
  340 FORMAT(33H CORRESPONDING ERROR TOLERANCES -,6X,8D10.2,            
     1       4(/39X,8D10.2))                                            
  350 FORMAT(44H INITIAL MESH(ES) AND Z,DMZ PROVIDED BY USER)           
  360 FORMAT(27H NO ADAPTIVE MESH SELECTION)                            
  370 FORMAT(//43H CONTINUATION WITH RESPECT TO PARAMETER NO.,I4)       
  380 FORMAT(25H IN PARAMETER INTERVAL  (,D15.7,1H,,D15.7,1H))          
  390 FORMAT(31H STARTING FROM PARAMETER VALUE ,D15.7,                  
     1       23H  IN POSITIVE DIRECTION)                                
  400 FORMAT(31H STARTING FROM PARAMETER VALUE ,D15.7,                  
     1       23H  IN NEGATIVE DIRECTION)                                
  410 FORMAT(41H MAXIMAL NUMBER OF CONTINUATION STEPS IS ,I4)           
  420 FORMAT(44H THE MAXIMUM NUMBER OF SUBINTERVALS IS MIN (, I4,       
     1       23H (ALLOWED FROM FSPACE),,I4, 24H (ALLOWED FROM ISPACE) ))
  430 FORMAT(/53H INSUFFICIENT SPACE TO DOUBLE MESH FOR ERROR ESTIMATE) 
      END                                                               
C---------------------------------------------------------------------- 
C             p a r t  1   of COLPAR                                    
C        main storage allocation and program control subroutines        
C---------------------------------------------------------------------- 
C                                                                       
      SUBROUTINE COLPAR (NCOMP, M, ALEFT, ARIGHT, ZETA, INFO, LTOL,     
     1                   TOL, FIXPNT, ISPACE, FSPACE, IFLAG,            
     2                   FSUB, DFSUB, GSUB, DGSUB, GUESS)               
C                                                                       
C                                                                       
C********************************************************************** 
C                                                                       
C                                                                       
C     written by                                                        
C                  g. bader and p. kunkel                               
C                            institut f. angewandte mathematik          
C                            university of heidelberg                   
C                            im neuenheimer feld 294                    
C                            d-6900 heidelberg 1, frg.                  
C                                                                       
C     this code is a generalization of COLNEW implemented by            
C                                                                       
C                  u. ascher,                                           
C                            department of computer science,            
C                            university of british columbia,            
C                            vancouver, b. c., canada   v6t 1w5         
C                  g. bader,                                            
C                            institut f. angewandte mathematik          
C                            university of heidelberg                   
C                            im neuenheimer feld 294                    
C                            d-6900 heidelberg 1                        
C                                                                       
C********************************************************************** 
C                                                                       
C     purpose                                                           
C                                                                       
C     this package solves a parameter dependent multi-point             
C     boundary value problem for a mixed order system of                
C     ode-s given by                                                    
C                                                                       
C          (m(i))                                                       
C         u       =  f  ( x; z(u(x),par) )      i = 1, ... ,ncomp       
C          i          i                                                 
C                                                                       
C                                          aleft .lt. x .lt. aright,    
C                                                                       
C                                                                       
C         g  ( zeta(j); z(u(zeta(j),par)) ) = 0   j = 1, ... ,mside     
C          j                                                            
C                                    mstar = m(1)+m(2)+...+m(ncomp),    
C                                    mside = mstar + npar - 1,          
C                                    mstate = mstar + npar,             
C                                                                       
C                                                                       
C         where                       t                                 
C            u = (u , u , ... ,u     )   is the exact solution vector   
C                  1   2        ncomp                                   
C                                                                       
C             (mi)                                                      
C            u     is the mi=m(i) th  derivative of u                   
C             i                                      i                  
C                                                                       
C                                 (1)        (m1-1)       (mncomp-1)    
C           z(u(x),par) = (u (x),u  (x),...,u    (x),...,u      (x),par)
C                           1     1          1            ncomp         
C                                                                       
C           f (x,z(u,par))   is a (generally) nonlinear function of     
C            i                                                          
C                            z(u,par)=z(u(x),par).                      
C                                                                       
C           g (zeta(j);z(u,par))  is a (generally) nonlinear function   
C            j                                                          
C                               used to represent a boundary condition. 
C                                                                       
C         the boundary points satisfy                                   
C             aleft .le. zeta(1) .le. .. .le. zeta(mside) .le. aright   
C                                                                       
C         the orders mi of the differential equations satisfy           
C                            1 .le. m(i) .le. 4.                        
C                                                                       
C                                                                       
C********************************************************************** 
C                                                                       
C     method                                                            
C                                                                       
C        the method used to approximate the solution u is               
C     collocation at gaussian points, requiring m(i)-1 continuous       
C     derivatives in the i-th component, i = 1, ..., ncomp.             
C     here, k is the number of collocation points (stages) per          
C     subinterval and is chosen such that k .ge. max m(i).              
C     a runge-kutta-monomial basis is utilized.                         
C                                                                       
C     references                                                        
C                                                                       
C     [1] u. ascher, j. christiansen and r.d. russell,                  
C         collocation software for boundary-value odes,                 
C         acm trans. math software 7 (1981), 209-222.                   
C                                                                       
C     [2] g. bader and u. ascher,                                       
C         a new basis implementation for a mixed order                  
C         boundary value ode solver,                                    
C         tech. rep. 85-11, dept. computer science, univ. b.c.,         
C         vancouver, canada. 1985.                                      
C                                                                       
C     [3] u. ascher, j. christiansen and r.d. russell,                  
C         a collocation solver for mixed order                          
C         systems of boundary value problems,                           
C         math. comp. 33 (1979), 659-679.                               
C                                                                       
C     [4] u. ascher, j. christiansen and r.d. russell,                  
C         colsys - a collocation code for boundary                      
C         value problems,                                               
C         lecture notes comp.sc. 76, springer verlag,                   
C         b. childs et. al. (eds.) (1979), 164-185.                     
C                                                                       
C     [5] c. deboor and r. weiss,                                       
C                                                                       
C         solveblok: a package for solving almost block diagonal        
C         linear systems,                                               
C         acm trans. math. software 6 (1980), 80-87.                    
C                                                                       
C********************************************************************** 
C                                                                       
C     ***************     input to colpar     ***************           
C                                                                       
C     variables                                                         
C                                                                       
C     ncomp - no. of differential equations   (ncomp .le. 20)           
C                                                                       
C     m(j) - order of the j-th differential equation                    
C            ( mstar = m(1) + ... + m(ncomp) .le. 40 )                  
C                                                                       
C     aleft - left end of interval                                      
C                                                                       
C     aright - right end of interval                                    
C                                                                       
C     zeta(j) - j-th side condition point (boundary point). must        
C               have  zeta(j) .le. zeta(j+1). all side condition        
C               points must be mesh points in all meshes used,          
C               see description of info(11) and fixpnt below.           
C                                                                       
C     info - an integer array dimensioned at least 12.                  
C            a list of the parameters in info and their meaning follows 
C            some parameters are renamed in colpar; these new names are 
C            given in parentheses.                                      
C                                                                       
C     info(1)     ( = nonlin )                                          
C             = 0 if the problem is linear                              
C             = 1 if the problem is nonlinear                           
C                                                                       
C     info(2) = no. of collocation points per subinterval  (= k )       
C               where max m(i) .le.  k .le. 7 . if info(2)=0 then       
C               colpar sets  k = max ( max m(i)+1, 5-max m(i) )         
C                                                                       
C     info(3) = no. of subintervals in the initial mesh  ( = n ).       
C               if info(3) = 0 then colpar arbitrarily sets n = 5.      
C                                                                       
C     info(4) = no. of solution and derivative tolerances.  ( = ntol )  
C               we require  0 .lt. ntol .le. mstar.                     
C                                                                       
C     info(5) = dimension of fspace.     ( = ndimf )                    
C                                                                       
C     info(6) = dimension of ispace.     ( = ndimi )                    
C                                                                       
C     info(7) -  output control ( = iprint )                            
C              = -1 for full diagnostic printout                        
C              = 0 for selected printout                                
C              = 1 for no printout                                      
C                                                                       
C     info(8)     ( = iread )                                           
C             = 0 causes colpar to generate a uniform initial mesh.     
C             = 1 if the initial mesh is provided by the user.  it      
C                 is defined in fspace as follows:  the mesh            
C                 aleft=x(1).lt.x(2).lt. ... .lt.x(n).lt.x(n+1)=aright  
C                 will occupy  fspace(1), ..., fspace(n+1). the         
C                 user needs to supply only the interior mesh           
C                 points  fspace(j) = x(j), j = 2, ..., n.              
C             = 2 if the initial mesh is supplied by the user           
C                 as with info(8)=1, and in addition no adaptive        
C                 mesh selection is to be done.                         
C                                                                       
C     info(9)     ( = iguess )                                          
C             = 0 if no initial guess for the solution is               
C                 provided.                                             
C             = 1 if an initial guess is provided by the user           
C                 in subroutine  guess.                                 
C             = 2 if an initial mesh and approximate solution           
C                 coefficients are provided by the user in  fspace.     
C                 (the former and new mesh are the same).               
C             = 3 if a former mesh and an approximate solution          
C                 coefficients are provided by the user in fspace,      
C                 and the new mesh is to be taken twice as coarse;      
C                 i.e.,every second point from the former mesh.         
C             = 4 if in addition to a former initial mesh and an        
C                 approximate solution coefficients, a new mesh         
C                 is provided in fspace as well.                        
C                 (see description of output for further details        
C                 on iguess = 2, 3, and 4.)                             
C                                                                       
C     info(10)= 0 if the problem is regular                             
C             = 1 if the first relax factor is =rstart, and the         
C                 nonlinear iteration does not rely on past covergence  
C                 (use for an extra sensitive nonlinear problem only).  
C             = 2 if we are to return immediately upon  (a) two         
C                 successive nonconvergences, or  (b) after obtaining   
C                 error estimate for the first time.                    
C                                                                       
C     info(11)= no. of fixed points in the mesh other than aleft        
C               and aright. ( = nfxpnt , the dimension of fixpnt)       
C               the code requires that all side condition points        
C               other than aleft and aright (see description of         
C               zeta ) be included as fixed points in fixpnt.           
C                                                                       
C     info(12)    ( = npar )                                            
C               no. of parameters appearing in the problem.             
C                                                                       
C     ltol  -  an array of dimension info(4). ltol(j) = l  specifies    
C              that the j-th tolerance in  tol  controls the error      
C              in the l-th component of z(u).   also require that       
C              1.le.ltol(1).lt.ltol(2).lt. ... .lt.ltol(ntol).le.mstar  
C                                                                       
C     tol    - an array of dimension info(4). tol(j) is the             
C              error tolerance on the ltol(j) -th component             
C              of z(u). thus, the code attempts to satisfy              
C              for j=1,...,ntol  on each subinterval                    
C              abs(z(v)-z(u))       .le. tol(j)*abs(z(u))       +tol(j) 
C                            ltol(j)                     ltol(j)        
C                                                                       
C              if v(x) is the approximate solution vector.              
C                                                                       
C     fixpnt - an array of dimension info(11).   it contains            
C              the points, other than aleft and aright, which           
C              are to be included in every mesh.                        
C                                                                       
C     ispace - an integer work array of dimension info(6).              
C              its size provides a constraint on nmax,                  
C              the maximum number of subintervals. choose               
C              info(6) according to the formula                         
C                      info(6)  .ge.  nmax*nsizei                       
C                where                                                  
C                      nsizei = 3 + kdm                                 
C                with                                                   
C                      kdm = kd + mstar  ;  kd = k * ncomp ;            
C                      nrec = no. of right end boundary conditions.     
C                                                                       
C                                                                       
C     fspace - a real work array of dimension info(5).                  
C              its size provides a constraint on nmax.                  
C              choose info(5) according to the formula                  
C                      info(5)  .ge.  nmax*nsizef                       
C                where                                                  
C                      nsizef = 4 + 3 * mstar + (5+kd) * kdm +          
C                              (2*mstar-nrec) * 2*mstar.                
C                                                                       
C                                                                       
C     iflag - the mode of return from colpar.                           
C           = 1 for normal return                                       
C           = 0 if the collocation matrix is singular.                  
C           =-1 if the expected no. of subintervals exceeds storage     
C               specifications.                                         
C           =-2 if the nonlinear iteration has not converged.           
C           =-3 if there is an input data error.                        
C                                                                       
C                                                                       
C********************************************************************** 
C                                                                       
C     *************    user supplied subroutines   *************        
C                                                                       
C                                                                       
C     the following subroutines must be declared external in the        
C     main program which calls colpar.                                  
C                                                                       
C                                                                       
C     fsub  - name of subroutine for evaluating f(x,z(u(x))) =          
C                            t                                          
C             (f ,...,f     )  at a point x in (aleft,aright).  it      
C               1      ncomp                                            
C             should have the heading                                   
C                                                                       
C                       subroutine fsub (x , z , f)                     
C                                                                       
C             where f is the vector containing the value of fi(x,z(u))  
C             in the i-th component and                            t    
C                                       z(u(x))=(z(1),...,z(mstar))     
C             is defined as above under  purpose .                      
C                                                                       
C                                                                       
C     dfsub - name of subroutine for evaluating the jacobian of         
C             f(x,z(u)) at a point x.  it should have the heading       
C                                                                       
C                       subroutine dfsub (x , z , df)                   
C                                                                       
C             where z(u(x)) is defined as for fsub and the (ncomp) by   
C             (mstar) array df should be filled by the partial deriv-   
C             atives of f, viz, for a particular call one calculates    
C                                df(i,j) = dfi / dzj, i=1,...,ncomp     
C                                                     j=1,...,mstar.    
C                                                                       
C                                                                       
C     gsub  - name of subroutine for evaluating the i-th component of   
C             g(x,z(u(x))) = g (zeta(i),z(u(zeta(i)))) at a point x =   
C                             i                                         
C             zeta(i) where 1.le.i.le.mstar. it should have the heading 
C                                                                       
C                       subroutine gsub (i , z , g)                     
C                                                                       
C             where z(u) is as for fsub, and i and g=g  are as above.   
C                                                     i                 
C             note that in contrast to f in  fsub , here                
C             only one value per call is returned in g.                 
C                                                                       
C                                                                       
C     dgsub - name of subroutine for evaluating the i-th row of         
C             the jacobian of g(x,u(x)).  it should have the heading    
C                                                                       
C                       subroutine dgsub (i , z , dg)                   
C                                                                       
C             where z(u) is as for fsub, i as for gsub and the mstar-   
C             vector dg should be filled with the partial derivatives   
C             of g, viz, for a particular call one calculates           
C                   dg(i,j) = dgi / dzj      j=1,...,mstar.             
C                                                                       
C                                                                       
C     guess - name of subroutine to evaluate the initial                
C             approximation for  z(u(x)) and for dmval(u(x))= vector    
C             of the mj-th derivatives of u(x). it should have the      
C             heading                                                   
C                                                                       
C                       subroutine guess (x , z , dmval)                
C                                                                       
C             note that this subroutine is needed only if using         
C             info(9) = 1, and then all  mstar  components of z         
C             and  ncomp  components of  dmval  should be specified     
C             for any x,  aleft .le. x .le. aright .                    
C                                                                       
C                                                                       
C********************************************************************** 
C                                                                       
C     ************   use of output from colpar   ************           
C                                                                       
C                 ***   solution evaluation   ***                       
C                                                                       
C     on return from colpar, the arrays fspace and ispace               
C     contain information specifying the approximate solution.          
C     the user can produce the solution vector  z( u(x) )  at           
C     any point x, aleft .le. x .le. aright, by the statement,          
C                                                                       
C           call appsln (x, z, fspace, ispace)                          
C                                                                       
C     when saving the coefficients for later reference, only            
C     ispace(1),...,ispace(7+ncomp)    and                              
C     fspace(1),...,fspace(ispace(7))    need to be saved as            
C     these are the quantities used by appsln.                          
C                                                                       
C                                                                       
C                 ***   simple continuation   ***                       
C                                                                       
C                                                                       
C     a formerly obtained solution can easily be used as the            
C     first approximation for the nonlinear iteration for a             
C     new problem by setting   (iguess =) info(9) = 2, 3 or 4.          
C                                                                       
C     if the former solution has just been obtained then the            
C     values needed to define the first approximation are               
C     already in ispace and fspace.                                     
C     alternatively, if the former solution was obtained in a           
C     previous run and its coefficients were saved then those           
C     coefficients must be put back into                                
C     ispace(1),..., ispace(7+ncomp)    and                             
C     fspace(1),..., fspace(ispace(7)).                                 
C                                                                       
C     for info(9) = 2 or 3 set info(3) = ispace(1) ( = the              
C     size of the previous mesh ).                                      
C                                                                       
C     for info(9) = 4 the user specifies a new mesh of n subintervals   
C     as follows.                                                       
C     the values in  fspace(1),...,fspace(ispace(7))  have to be        
C     shifted by n+1 locations to  fspace(n+2),..,fspace(ispace(7)+n+1) 
C     and the new mesh is then specified in fspace(1),..., fspace(n+1). 
C     also set info(3) = n.                                             
C                                                                       
C                                                                       
C********************************************************************** 
C                                                                       
C     ***************      package subroutines      ***************     
C                                                                       
C     the following description gives a brief overview of how the       
C     procedure is broken down into the subroutines which make up       
C     the package called  colpar . for further details the              
C     user should refer to documentation in the various subroutines     
C     and to the references cited above.                                
C                                                                       
C     the subroutines fall into four groups:                            
C                                                                       
C part 1 - the main storage allocation and program control subr         
C                                                                       
C     colpar - tests input values, does initialization and breaks up    
C              the work areas, fspace and ispace, into the arrays       
C              used by the program.                                     
C                                                                       
C     contrl - is the actual driver of the package. this routine        
C              contains the strategy for nonlinear equation solving.    
C                                                                       
C     skale  - provides scaling for the control                         
C              of convergence in the nonlinear iteration.               
C                                                                       
C                                                                       
C part 2 - mesh selection and error estimation subroutines              
C                                                                       
C     consts - is called once by  colpar  to initialize constants       
C              which are used for error estimation and mesh selection.  
C                                                                       
C     newmsh - generates meshes. it contains the test to decide         
C              whether or not to redistribute a mesh.                   
C                                                                       
C     errchk - produces error estimates and checks against the          
C              tolerances at each subinterval                           
C                                                                       
C                                                                       
C part 3 - collocation system set-up subroutines                        
C                                                                       
C     lsyslv - controls the set-up and solution of the linear           
C              algebraic systems of collocation equations which         
C              arise at each newton iteration.                          
C                                                                       
C     gderiv - is used by lsyslv to set up the equation associated      
C              with a side condition point.                             
C                                                                       
C     vwblok - is used by lsyslv to set up the equation(s) associated   
C              with a collocation point.                                
C                                                                       
C     gblock - is used by lsyslv to construct a block of the global     
C              collocation matrix or the corresponding right hand       
C              side.                                                    
C                                                                       
C                                                                       
C part 4 - service subroutines                                          
C                                                                       
C     appsln - sets up a standard call to  approx .                     
C                                                                       
C     approx - evaluates a piecewise polynomial solution.               
C                                                                       
C     rkbas  - evaluates the mesh independent runge-kutta basis         
C                                                                       
C     vmonde - solves a vandermonde system for given right hand         
C              side                                                     
C                                                                       
C     horder - evaluates the highest order derivatives of the           
C              current collocation solution used for mesh refinement.   
C                                                                       
C                                                                       
C part 5 - linear algebra  subroutines                                  
C                                                                       
C     to solve the global linear systems of collocation equations       
C     constructed in part 3,  colpar  uses a column oriented version    
C     of the package  solveblok due to de boor and weiss.               
C                                                                       
C     to solve the linear systems for static parameter condensation     
C     in each block of the collocation equations, the linpack           
C     routines  dgefa and  dgesl  are included. but these               
C     may be replaced when solving problems on vector processors        
C     or when solving large scale sparse jacobian problems.             
C                                                                       
C---------------------------------------------------------------------- 
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION M(1), ZETA(1), INFO(1), LTOL(1), TOL(1), DUMMY(1),      
     1          FIXPNT(1), ISPACE(1), FSPACE(1)                         
C                                                                       
      COMMON /COLOUT/ PRECIS, IOUT, IPRINT                              
      COMMON /COLLOC/ RHO(7), COEF(49)                                  
      COMMON /COLAPR/ N, NOLD, NMAX, NZ, NZPAR, NDMZ                    
      COMMON /COLORD/ K, NC, MSTAR, NPAR, MSTATE, KD, MMAX, MT(20)      
      COMMON /COLSID/ TZETA(40), TLEFT, TRIGHT, MSIDE, IZETA, IDUM      
      COMMON /COLNLN/ NONLIN, ITER, LIMIT, ICARE, IGUESS, IREAD         
      COMMON /COLPTH/ IPATH, ICON, LPAR                                 
      COMMON /COLMSH/ MSHFLG, MSHNUM, MSHLMT, MSHALT                    
      COMMON /COLEST/ WGTMSH(40), WGTERR(40), TTOL(40),                 
     1                ROOT(40), JTOL(40), LTTOL(40), NTOL               
C                                                                       
      EXTERNAL FSUB, DFSUB, GSUB, DGSUB, GUESS                          
C                                                                       
C*********************************************************************  
C                                                                       
C     the actual subroutine colpar serves as an interface with          
C     the package of subroutines referred to collectively as            
C     colpar. the subroutine serves to test some of the input           
C     parameters, rename some of the parameters (to make under-         
C     standing of the coding easier), to do some initialization,        
C     and to break the work areas fspace and ispace up into the         
C     arrays needed by the program.                                     
C                                                                       
C********************************************************************** 
C                                                                       
C...  specify machine dependent output unit  iout  and compute machine  
C...  dependent constant  precis = 100 * machine unit roundoff          
C                                                                       
      IOUT = 6                                                          
C                                                                       
      PRECIS = 1.D0                                                     
   10 PRECIS = PRECIS / 2.D0                                            
      PRECP1 = PRECIS + 1.D0                                            
      IF ( PRECP1 .GT. 1.D0 )                       GO TO 10            
      PRECIS = PRECIS * 100.D0                                          
C                                                                       
C...  in case incorrect input data is detected, the program returns     
C...  immediately with iflag=-3.                                        
C                                                                       
      IFLAG = -3                                                        
      IF ( NCOMP .LT. 1 .OR. NCOMP .GT. 20 )        RETURN              
      DO 20 I=1,NCOMP                                                   
         IF ( M(I) .LT. 1 .OR. M(I) .GT. 4 )        RETURN              
   20 CONTINUE                                                          
C                                                                       
C...  rename some of the parameters and set default values.             
C                                                                       
      NONLIN = INFO(1)                                                  
      K = INFO(2)                                                       
      N = INFO(3)                                                       
      IF ( N .EQ. 0 )  N = 5                                            
      ICARE = INFO(10)                                                  
      NTOL = INFO(4)                                                    
      NDIMF = INFO(5)                                                   
      NDIMI = INFO(6)                                                   
      IPRINT = INFO(7)                                                  
      IREAD = INFO(8)                                                   
      IGUESS = INFO(9)                                                  
      IF ( NONLIN .EQ. 0 .AND. IGUESS .EQ. 1 )  IGUESS = 0              
      IF ( IGUESS .GE. 2 .AND. IREAD .EQ. 0 )   IREAD = 1               
      NFXPNT = INFO(11)                                                 
      NPAR = INFO(12)                                                   
      MSTAR = 0                                                         
      MMAX = 0                                                          
      DO  30 I = 1, NCOMP                                               
         MMAX = MAX0 ( MMAX, M(I) )                                     
         MSTAR = MSTAR + M(I)                                           
         MT(I) = M(I)                                                   
   30 CONTINUE                                                          
      MSTATE = MSTAR + NPAR                                             
      MSIDE = MSTATE                                                    
      IF ( K .EQ. 0 )   K = MAX0( MMAX + 1 , 5 - MMAX )                 
      DO 40 I = 1, MSIDE                                                
   40 TZETA(I) = ZETA(I)                                                
      DO 50 I = 1, NTOL                                                 
         LTTOL(I) = LTOL(I)                                             
   50 TTOL(I) = TOL(I)                                                  
      TLEFT = ALEFT                                                     
      TRIGHT = ARIGHT                                                   
      NC = NCOMP                                                        
      KD = K * NCOMP                                                    
C                                                                       
C...  set default values for continuation option                        
C                                                                       
      IPATH = 0                                                         
      ICON = 0                                                          
C                                                                       
C...  print the input data for checking.                                
C                                                                       
      IF ( IPRINT .LT. 1 )  WRITE(IOUT,410)                             
      IF ( IPRINT .GT. -1 )                         GO TO 80            
      IF ( NONLIN .GT. 0 )                          GO TO 60            
      WRITE (IOUT,260) NCOMP, (M(IP), IP=1,NCOMP)                       
      GO TO 70                                                          
   60 WRITE (IOUT,270) NCOMP, (M(IP), IP=1,NCOMP)                       
   70 WRITE (IOUT,275) NPAR                                             
      WRITE (IOUT,280) (ZETA(IP), IP=1,MSIDE)                           
      IF ( NFXPNT .GT. 0 )                                              
     1   WRITE (IOUT,340) NFXPNT, (FIXPNT(IP), IP=1,NFXPNT)             
      WRITE (IOUT,290) K                                                
      WRITE (IOUT,300) (LTOL(IP), IP=1,NTOL)                            
      WRITE (IOUT,310) (TOL(IP), IP=1,NTOL)                             
      IF (IGUESS .GE. 2) WRITE (IOUT,320)                               
      IF (IREAD .EQ. 2) WRITE (IOUT,330)                                
   80 CONTINUE                                                          
C                                                                       
C...  check for correctness of data                                     
C                                                                       
      IF ( K .LT. 0 .OR. K .GT. 7 )                 RETURN              
      IF ( N .LT. 0 )                               RETURN              
      IF ( IREAD .LT. 0 .OR. IREAD .GT. 2 )         RETURN              
      IF ( IGUESS .LT. 0 .OR. IGUESS .GT. 4 )       RETURN              
      IF ( ICARE .LT. 0 .OR. ICARE .GT. 2 )         RETURN              
      IF ( NTOL .LT. 0 .OR. NTOL .GT. MSTAR )       RETURN              
      IF ( NFXPNT .LT. 0 )                          RETURN              
      IF ( NPAR .LT. 0 )                            RETURN              
      IF ( IPRINT .LT. (-1) .OR. IPRINT .GT. 1 )    RETURN              
      IF ( MSTATE .LT. 1 .OR. MSTATE .GT. 40 )      RETURN              
      IP = 1                                                            
      DO 100 I = 1, MSIDE                                               
      IF ( DABS(ZETA(I) - ALEFT) .LT. PRECIS .OR.                       
     1     DABS(ZETA(I) - ARIGHT) .LT. PRECIS )     GO TO 100           
   90 IF ( IP .GT. NFXPNT )                         RETURN              
        IF ( ZETA(I) - PRECIS .LT. FIXPNT(IP) )     GO TO 95            
        IP = IP + 1                                                     
      GO TO 90                                                          
   95 IF ( ZETA(I) + PRECIS .LT. FIXPNT(IP) )       RETURN              
  100 CONTINUE                                                          
C                                                                       
C...  set limits on iterations and initialize counters.                 
C...  limit = maximum number of newton iterations per mesh.             
C...  see subroutine  newmsh  for the roles of  mshlmt , mshflg ,       
C...  mshnum , and  mshalt .                                            
C                                                                       
      MSHLMT = 3                                                        
      MSHFLG = 0                                                        
      MSHNUM = 1                                                        
      MSHALT = 1                                                        
      LIMIT = 40                                                        
C                                                                       
C...  compute the maxium possible n for the given sizes of              
C...  ispace  and  fspace.                                              
C                                                                       
      NREC = 0                                                          
      DO 110 I = 1, MSIDE                                               
           IB = MSIDE + 1 - I                                           
           IF ( ZETA(IB) .GE. ARIGHT )  NREC = I                        
  110 CONTINUE                                                          
      NFIXI = MSIDE                                                     
      NSIZEI = 3 + KD + MSTAR                                           
      NFIXF = 2 * MSTAR * NREC + (2 * MSTAR + MSIDE) * NPAR             
     1      + 4 * MSTATE + MSIDE + 3                                    
      NSIZEF = 2 * MSTAR * (MSTAR + MSIDE - NREC)                       
     1        + KD * (KD + MSTATE + 5) + (NPAR + 8) * MSTAR + 4         
      NMAXF = (NDIMF - NFIXF) / NSIZEF                                  
      NMAXI = (NDIMI - NFIXI) / NSIZEI                                  
      IF ( IPRINT .LT. 1 )  WRITE(IOUT,350) NMAXF, NMAXI                
      NMAX = MIN0( NMAXF, NMAXI )                                       
      IF ( NMAX .LT. N )                            RETURN              
      IF ( NMAX .LT. NFXPNT+1 )                     RETURN              
      IF (NMAX .LT. 2*NFXPNT+2 .AND. IPRINT .LT. 1)  WRITE(IOUT,360)    
C                                                                       
C...  generate pointers to break up  fspace  and  ispace .              
C                                                                       
      NMAX1 = NMAX + 1                                                  
      KDN = KD * NMAX                                                   
C                                                                       
      LXI = 1                                                           
      LG = LXI + NMAX1                                                  
      LXIOLD = LG + 2*MSTAR * (NMAX * (MSTAR+MSIDE-NREC) + NREC)        
     1       + NPAR * MSTAR * NMAX1 + NPAR * (MSTAR + MSIDE)            
      LW     = LXIOLD + NMAX1                                           
      LV     = LW + KD * KDN                                            
      LZ     = LV + MSTATE * KDN                                        
      LDMZ   = LZ + MSTAR * NMAX1 + NPAR                                
      LDELZ  = LDMZ + KDN                                               
      LDELDZ = LDELZ + MSTAR * NMAX1 + NPAR                             
      LDQZ   = LDELDZ + KDN                                             
      LDQDMZ = LDQZ + MSTAR * NMAX1 + NPAR                              
      LRHS   = LDQDMZ + KDN                                             
      LVALST = LRHS   + KDN + MSIDE                                     
      LSLOPE = LVALST + 4 * MSTAR * NMAX                                
      LACCUM = LSLOPE + NMAX                                            
      LSCL   = LACCUM + NMAX1                                           
      LDSCL  = LSCL + MSTAR * NMAX1                                     
      LPVTG = 1                                                         
      LPVTW = LPVTG + MSTAR * NMAX + MSIDE                              
      LINTEG = LPVTW + KDN                                              
C                                                                       
C...  if  iguess .ge. 2, move  xiold, z, and  dmz  to their proper      
C...  locations in  fspace.                                             
C                                                                       
      IF ( IGUESS .LT. 2 )                          GO TO 160           
      NOLD = N                                                          
      NPOLD = ISPACE(6)                                                 
      IF (IGUESS .EQ. 4)  NOLD = ISPACE(1)                              
      NZPAR = MSTAR * (NOLD + 1) + NPAR                                 
      NZPOLD = MSTAR * (NOLD + 1) + NPOLD                               
      NDMZ = KD * NOLD                                                  
      NP1 = N + 1                                                       
      IF ( IGUESS .EQ. 4 )  NP1 = NP1 + NOLD + 1                        
      DO 120 I=1,NZPOLD                                                 
  120 FSPACE( LZ+I-1 )  =  FSPACE( NP1+I )                              
      IDMZ = NP1 + NZPOLD                                               
      DO 125 I=1,NDMZ                                                   
  125 FSPACE( LDMZ+I-1 )  =  FSPACE( IDMZ+I )                           
      NP1 = NOLD + 1                                                    
      IF ( IGUESS .EQ. 4 )                          GO TO 140           
      DO 130 I=1,NP1                                                    
  130 FSPACE( LXIOLD+I-1 )  =  FSPACE( LXI+I-1 )                        
      GO TO 160                                                         
  140 DO 150 I=1,NP1                                                    
  150 FSPACE( LXIOLD+I-1 )  =  FSPACE( N+1+I )                          
  160 CONTINUE                                                          
C                                                                       
C...  initialize collocation points, constants, mesh.                   
C                                                                       
      CALL CONSTS ( K, RHO, COEF )                                      
      CALL NEWMSH (3+IREAD, FSPACE(LXI), FSPACE(LXIOLD), DUMMY,         
     1             DUMMY, DUMMY, DUMMY, DUMMY, NFXPNT, FIXPNT)          
C                                                                       
C...  determine first approximation, if the problem is nonlinear.       
C                                                                       
      IF (IGUESS .GE. 2)                            GO TO 230           
      NP1 = N + 1                                                       
      DO 210 I = 1, NP1                                                 
  210 FSPACE( I + LXIOLD - 1 ) = FSPACE( I + LXI - 1 )                  
      NOLD = N                                                          
      IF ( NONLIN .EQ. 0  .OR. IGUESS .EQ. 1 )      GO TO 230           
C                                                                       
C...  system provides first approximation of the solution.              
C...  choose z(j) = 0  for j=1,...,mstar.                               
C                                                                       
      NZPAR = ( N + 1 ) * MSTAR + NPAR                                  
      NDMZ = KD * N                                                     
      DO 220 I=1, NZPAR                                                 
  220 FSPACE( LZ-1+I ) = 0.D0                                           
      DO 225 I=1, NDMZ                                                  
  225 FSPACE( LDMZ-1+I ) = 0.D0                                         
  230 CONTINUE                                                          
C                                                                       
      IF (IGUESS .GE. 2)  IGUESS = 0                                    
      CALL CONTRL (FSPACE(LXI),FSPACE(LXIOLD),FSPACE(LZ),FSPACE(LDMZ),  
     1     FSPACE(LRHS),FSPACE(LDELZ),FSPACE(LDELDZ),FSPACE(LDQZ),      
     2     FSPACE(LDQDMZ),FSPACE(LG),FSPACE(LW),FSPACE(LV),             
     3     FSPACE(LVALST),FSPACE(LSLOPE),FSPACE(LSCL),FSPACE(LDSCL),    
     4     FSPACE(LACCUM),ISPACE(LPVTG),ISPACE(LINTEG),ISPACE(LPVTW),   
     5     NFXPNT,FIXPNT,IFLAG,FSUB,DFSUB,GSUB,DGSUB,GUESS )            
C                                                                       
C...  prepare output                                                    
C                                                                       
      IF ( IFLAG .EQ. 0 )                                RETURN         
      ISPACE(1) = N                                                     
      ISPACE(2) = K                                                     
      ISPACE(3) = NCOMP                                                 
      ISPACE(4) = MSTAR                                                 
      ISPACE(5) = MMAX                                                  
      ISPACE(6) = NPAR                                                  
      LENGTH = N + 1 + NZPAR + NDMZ + K * K                             
      ISPACE(7) = LENGTH                                                
      DO 240 I = 1, NCOMP                                               
  240 ISPACE(7+I) = M(I)                                                
C                                                                       
      DO 250 I = 1, NZPAR                                               
  250 FSPACE( N+1+I ) = FSPACE( LZ-1+I )                                
      IDMZ = N + 1 + NZPAR                                              
      DO 255 I = 1, NDMZ                                                
  255 FSPACE( IDMZ+I ) = FSPACE( LDMZ-1+I )                             
      IC = IDMZ + NDMZ                                                  
      K2 = K * K                                                        
      DO 258 I = 1, K2                                                  
  258 FSPACE( IC+I ) = COEF(I)                                          
      RETURN                                                            
C---------------------------------------------------------------------- 
  260 FORMAT(37H THE NUMBER OF (LINEAR) DIFF EQNS IS , I3/ 1X,          
     1       16HTHEIR ORDERS ARE, 20I3)                                 
  270 FORMAT(40H THE NUMBER OF (NONLINEAR) DIFF EQNS IS , I3/ 1X,       
     1       16HTHEIR ORDERS ARE, 20I3)                                 
  275 FORMAT(28H THE NUMBER OF PARAMETERS IS, I3)                       
  280 FORMAT(27H SIDE CONDITION POINTS ZETA, 8F10.6, 4( / 27X, 8F10.6)) 
  290 FORMAT(37H NUMBER OF COLLOC PTS PER INTERVAL IS, I3)              
  300 FORMAT(39H COMPONENTS OF Z REQUIRING TOLERANCES -,8(7X,I2,1X),    
     1       4(/38X,8I10))                                              
  310 FORMAT(33H CORRESPONDING ERROR TOLERANCES -,6X,8D10.2,            
     1       4(/39X,8D10.2))                                            
  320 FORMAT(44H INITIAL MESH(ES) AND Z,DMZ PROVIDED BY USER)           
  330 FORMAT(27H NO ADAPTIVE MESH SELECTION)                            
  340 FORMAT(10H THERE ARE ,I5,27H FIXED POINTS IN THE MESH - ,         
     1       10(6F10.6/))                                               
  350 FORMAT(44H THE MAXIMUM NUMBER OF SUBINTERVALS IS MIN (, I4,       
     1       23H (ALLOWED FROM FSPACE),,I4, 24H (ALLOWED FROM ISPACE) ))
  360 FORMAT(/53H INSUFFICIENT SPACE TO DOUBLE MESH FOR ERROR ESTIMATE) 
  410 FORMAT(47H1THIS IS  COLPAR  IN THE VERSION FROM OCT. 1987//)      
      END                                                               
      SUBROUTINE CONTRL (XI, XIOLD, Z, DMZ, RHS, DELZ, DELDMZ,          
     1           DQZ, DQDMZ, G, W, V, VALSTR, SLOPE, SCALE, DSCALE,     
     2           ACCUM, IPVTG, INTEGS, IPVTW, NFXPNT, FIXPNT, IFLAG,    
     3           FSUB, DFSUB, GSUB, DGSUB, GUESS )                      
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose                                                             
C     this subroutine is the actual driver.  the nonlinear iteration    
C     strategy is controlled here ( see [4] ). upon convergence, errchk 
C     is called to test for satisfaction of the requested tolerances.   
C                                                                       
C   variables                                                           
C                                                                       
C     check  - maximum tolerance value, used as part of criteria for    
C              checking for nonlinear iteration convergence             
C     relax  - the relaxation factor for damped newton iteration        
C     relmin - minimum allowable value for relax  (otherwise the        
C              jacobian is considered singular).                        
C     rlxold - previous relax                                           
C     rstart - initial value for relax when problem is sensitive        
C     ifrz   - number of fixed jacobian iterations                      
C     lmtfrz - maximum value for ifrz before performing a reinversion   
C     iter   - number of iterations (counted only when jacobian         
C              reinversions are performed).                             
C     xi     - current mesh                                             
C     xiold  - previous mesh                                            
C     ipred  = 0  if relax is determined by a correction                
C            = 1  if relax is determined by a prediction                
C     ifreez = 0  if the jacobian is to be updated                      
C            = 1  if the jacobian is currently fixed (frozen)           
C     iconv  = 0  if no previous convergence has been obtained          
C            = 1  if convergence on a previous mesh has been obtained   
C     icare  =-1  no convergence occurred (used for regular problems)   
C            = 0  a regular problem                                     
C            = 1  a sensitive problem                                   
C            = 2  used for continuation (see description of info(10)    
C                 in colpar).                                           
C     rnorm  - norm of rhs (right hand side) for current iteration      
C     rnold  - norm of rhs for previous iteration                       
C     anscl  - scaled norm of newton correction                         
C     anfix  - scaled norm of newton correction at next step            
C     anorm  - scaled norm of a correction obtained with jacobian fixed 
C     nz     - number of components of  z  (see subroutine approx)      
C     ndmz   - number of components of  dmz  (see subroutine approx)    
C     imesh  - a control variable for subroutines newmsh and errchk     
C            = 1  the current mesh resulted from mesh selection         
C                 or is the initial mesh.                               
C            = 2  the current mesh resulted from doubling the           
C                 previous mesh                                         
C                                                                       
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION XI(1), XIOLD(1), Z(1), DMZ(1), RHS(1)                   
      DIMENSION DELZ(1), DELDMZ(1), DQZ(1), DQDMZ(1) , FIXPNT(1)        
      DIMENSION G(1), W(1), V(1), VALSTR(1), SLOPE(1), ACCUM(1)         
      DIMENSION DUMMY(1), SCALE(1), DSCALE(1)                           
      DIMENSION INTEGS(1), IPVTG(1), IPVTW(1)                           
C                                                                       
      COMMON /COLOUT/ PRECIS, IOUT, IPRINT                              
      COMMON /COLAPR/ N, NOLD, NMAX, NZ, NZPAR, NDMZ                    
      COMMON /COLORD/ K, NCOMP, MSTAR, NPAR, MSTATE, KD, MMAX, MT(20)   
      COMMON /COLSID/ TZETA(40), TLEFT, TRIGHT, MSIDE, IZETA, IDUM      
      COMMON /COLNLN/ NONLIN, ITER, LIMIT, ICARE, IGUESS, IREAD         
      COMMON /COLMSH/ MSHFLG, MSHNUM, MSHLMT, MSHALT                    
      COMMON /COLEST/ WGTMSH(40), WGTERR(40), TOL(40),                  
     1                ROOT(40), JTOL(40), LTOL(40), NTOL                
C                                                                       
      EXTERNAL FSUB, DFSUB, GSUB, DGSUB, GUESS                          
C                                                                       
C...  constants for control of nonlinear iteration                      
C                                                                       
      RELMIN = 1.D-3                                                    
      RSTART = 1.D-2                                                    
      LMTFRZ = 4                                                        
C                                                                       
C...  compute the maximum tolerance                                     
C                                                                       
      CHECK = 0.D0                                                      
      DO 10 I = 1, NTOL                                                 
   10   CHECK = DMAX1 ( TOL(I), CHECK )                                 
      IMESH = 1                                                         
      ICONV = 0                                                         
      IF ( NONLIN .EQ. 0 ) ICONV = 1                                    
      NOCONV = 0                                                        
      MSING = 0                                                         
C                                                                       
C...  the main iteration begins here .                                  
C...  loop 20 is executed until error tolerances are satisfied or       
C...  the code fails (due to a singular matrix or storage limitations)  
C                                                                       
   20      CONTINUE                                                     
C                                                                       
C...       initialization for a new mesh                                
C                                                                       
           ITER = 0                                                     
           IF ( NONLIN .GT. 0 )                     GO TO 50            
C                                                                       
C...       the linear case.                                             
C...       set up and solve equations                                   
C                                                                       
           CALL LSYSLV (MSING, XI, XIOLD, DUMMY, DUMMY, Z, DMZ, G,      
     1          W, V, RHS, DUMMY, INTEGS, IPVTG, IPVTW, RNORM, 0,       
     2          FSUB, DFSUB, GSUB, DGSUB, GUESS )                       
C                                                                       
C...       check for a singular matrix                                  
C                                                                       
           IF ( MSING .EQ. 0 )                      GO TO 400           
   30      IF ( MSING .LT. 0 )                      GO TO 40            
           IF ( IPRINT .LT. 1 )  WRITE (IOUT,495)                       
           MSING = 0                                                    
           GO TO 460                                                    
   40      IF ( IPRINT .LT. 1 )  WRITE (IOUT,490)                       
           IFLAG = 0                                                    
           RETURN                                                       
C                                                                       
C...       iteration loop for nonlinear case                            
C...       define the initial relaxation parameter (= relax)            
C                                                                       
   50      RELAX = 1.D0                                                 
C                                                                       
C...       check for previous convergence and problem sensitivity       
C                                                                       
           IF ( ICARE .EQ. 1 .OR. ICARE .EQ. (-1) )  RELAX = RSTART     
           IF ( ICONV .EQ. 0 )                      GO TO 160           
C                                                                       
C...       convergence on a previous mesh has been obtained.    thus    
C...       we have a very good initial approximation for the newton     
C...       process.    proceed with one full newton and then iterate    
C...       with a fixed jacobian.                                       
C                                                                       
           IFREEZ = 0                                                   
C                                                                       
C...       evaluate right hand side and its norm  and                   
C...       find the first newton correction                             
C                                                                       
           CALL LSYSLV (MSING, XI, XIOLD, Z, DMZ, DELZ, DELDMZ, G,      
     1          W, V, RHS, DQDMZ, INTEGS, IPVTG, IPVTW, RNOLD, 1,       
     2          FSUB, DFSUB, GSUB, DGSUB, GUESS )                       
C                                                                       
           IF ( IPRINT .LT. 0 )  WRITE(IOUT,530)                        
           IF ( IPRINT .LT. 0 )  WRITE (IOUT,510) ITER, RNOLD           
           GO TO 70                                                     
C                                                                       
C...       solve for the next iterate .                                 
C...       the value of ifreez determines whether this is a full        
C...       newton step (=0) or a fixed jacobian iteration (=1).         
C                                                                       
   60      IF ( IPRINT .LT. 0 )  WRITE (IOUT,510) ITER, RNORM           
           RNOLD = RNORM                                                
           CALL LSYSLV (MSING, XI, XIOLD, Z, DMZ, DELZ, DELDMZ, G,      
     1          W, V, RHS, DUMMY, INTEGS, IPVTG, IPVTW, RNORM,          
     2          3+IFREEZ, FSUB, DFSUB, GSUB, DGSUB, GUESS )             
C                                                                       
C...       check for a singular matrix                                  
C                                                                       
   70      IF ( MSING .NE. 0 )                      GO TO 30            
           IF ( IFREEZ .EQ. 1 )                     GO TO 80            
C                                                                       
C...       a full newton step                                           
C                                                                       
           ITER = ITER + 1                                              
           IFRZ = 0                                                     
   80      CONTINUE                                                     
C                                                                       
C...       update   z and dmz , compute new  rhs  and its norm          
C                                                                       
           DO 90 I = 1, NZPAR                                           
             Z(I) = Z(I) + DELZ(I)                                      
   90      CONTINUE                                                     
           DO 100 I = 1, NDMZ                                           
             DMZ(I) = DMZ(I) + DELDMZ(I)                                
  100      CONTINUE                                                     
           CALL LSYSLV (MSING, XI, XIOLD, Z, DMZ, DELZ, DELDMZ, G,      
     1          W, V, RHS, DUMMY, INTEGS, IPVTG, IPVTW, RNORM, 2,       
     2          FSUB, DFSUB, GSUB, DGSUB, GUESS )                       
C                                                                       
C...       check monotonicity. if the norm of  rhs  gets smaller,       
C...       proceed with a fixed jacobian; else proceed cautiously,      
C...       as if convergence has not been obtained before (iconv=0).    
C                                                                       
           IF ( RNORM .LT. PRECIS )                 GO TO 390           
           IF ( RNORM .GT. RNOLD )                  GO TO 130           
           IF ( IFREEZ .EQ. 1 )                     GO TO 110           
           IFREEZ = 1                                                   
           GO TO 60                                                     
C                                                                       
C...       verify that the linear convergence with fixed jacobian       
C...       is fast enough.                                              
C                                                                       
  110      IFRZ = IFRZ + 1                                              
           IF ( IFRZ .GE. LMTFRZ )       IFREEZ = 0                     
           IF ( RNOLD .LT. 4.D0*RNORM )  IFREEZ = 0                     
C                                                                       
C...       check convergence (iconv = 1).                               
C                                                                       
           DO 120 IT = 1, NTOL                                          
             INZ = LTOL(IT)                                             
             DO 120 IZ = INZ, NZ, MSTAR                                 
               IF ( DABS(DELZ(IZ)) .GT.                                 
     1           TOL(IT) * ( DABS(Z(IZ)) + 1.D0) )  GO TO 60            
  120      CONTINUE                                                     
C                                                                       
C...       convergence obtained                                         
C                                                                       
           IF ( IPRINT .LT. 1 )  WRITE (IOUT,560) ITER                  
           GO TO 400                                                    
C                                                                       
C...      convergence of fixed jacobian iteration failed.               
C                                                                       
  130      IF ( IPRINT .LT. 0 )  WRITE (IOUT,510) ITER, RNORM           
           IF ( IPRINT .LT. 0 )  WRITE (IOUT,540)                       
           ICONV = 0                                                    
           RELAX = RSTART                                               
           DO 140 I = 1, NZPAR                                          
             Z(I) = Z(I) - DELZ(I)                                      
  140      CONTINUE                                                     
           DO 150 I = 1, NDMZ                                           
             DMZ(I) = DMZ(I) - DELDMZ(I)                                
  150      CONTINUE                                                     
C                                                                       
C...       update old mesh                                              
C                                                                       
           NP1 = N + 1                                                  
           DO 155 I = 1, NP1                                            
  155        XIOLD(I) = XI(I)                                           
           NOLD = N                                                     
C                                                                       
           ITER = 0                                                     
C                                                                       
C...       no previous convergence has been obtained. proceed           
C...       with the damped newton method.                               
C...       evaluate rhs and find the first newton correction.           
C                                                                       
  160      IF ( IPRINT .LT. 0 )  WRITE (IOUT,500)                       
           CALL LSYSLV (MSING, XI, XIOLD, Z, DMZ, DELZ, DELDMZ, G,      
     1          W, V, RHS, DQDMZ, INTEGS, IPVTG, IPVTW, RNOLD, 1,       
     2          FSUB, DFSUB, GSUB, DGSUB, GUESS )                       
C                                                                       
C...       check for a singular matrix                                  
C                                                                       
           IF ( MSING .NE. 0 )                       GO TO 30           
C                                                                       
C...       bookkeeping for first mesh                                   
C                                                                       
           IF ( IGUESS .EQ. 1 )  IGUESS = 0                             
C                                                                       
C...       find initial scaling                                         
C                                                                       
           CALL SKALE (N, KD, Z, XI, SCALE, DSCALE)                     
           GO TO 220                                                    
C                                                                       
C...       main iteration loop                                          
C                                                                       
  170      RNOLD = RNORM                                                
           IF ( ITER .GE. LIMIT )                   GO TO 430           
C                                                                       
C...       update scaling                                               
C                                                                       
           CALL SKALE (N, KD, Z, XI, SCALE, DSCALE)                     
C                                                                       
C...       compute norm of newton correction with new scaling           
C                                                                       
           ANSCL = 0.D0                                                 
           DO 180 I = 1, NZPAR                                          
             ANSCL = ANSCL + (DELZ(I) * SCALE(I))**2                    
  180      CONTINUE                                                     
           DO 190 I = 1, NDMZ                                           
             ANSCL = ANSCL + (DELDMZ(I) * DSCALE(I))**2                 
  190      CONTINUE                                                     
           ANSCL = DSQRT(ANSCL / DFLOAT(NZPAR+NDMZ))                    
C                                                                       
C...       find a newton direction                                      
C                                                                       
           CALL LSYSLV (MSING, XI, XIOLD, Z, DMZ, DELZ, DELDMZ, G,      
     1          W, V, RHS, DUMMY, INTEGS, IPVTG, IPVTW, RNORM, 3,       
     2          FSUB, DFSUB, GSUB, DGSUB, GUESS )                       
C                                                                       
C...       check for a singular matrix                                  
C                                                                       
           IF ( MSING .NE. 0 )                      GO TO 30            
C                                                                       
C...       predict relaxation factor for newton step.                   
C                                                                       
           ANDIF = 0.D0                                                 
           DO 200 I = 1, NZPAR                                          
             ANDIF = ANDIF + ((DQZ(I) - DELZ(I)) * SCALE(I))**2         
  200      CONTINUE                                                     
           DO 210 I = 1, NDMZ                                           
             ANDIF = ANDIF + ((DQDMZ(I) - DELDMZ(I)) * DSCALE(I))**2    
  210      CONTINUE                                                     
           ANDIF = DSQRT(ANDIF/DFLOAT(NZPAR+NDMZ) + PRECIS)             
           RELAX = RELAX * ANSCL / ANDIF                                
           IF ( RELAX .GT. 1.D0 )  RELAX = 1.D0                         
  220      RLXOLD = RELAX                                               
           IPRED = 1                                                    
           ITER = ITER + 1                                              
C                                                                       
C...       determine a new  z and dmz  and find new  rhs  and its norm  
C                                                                       
           DO 230 I = 1, NZPAR                                          
             Z(I) = Z(I)  +  RELAX * DELZ(I)                            
  230      CONTINUE                                                     
           DO 240 I = 1, NDMZ                                           
             DMZ(I) = DMZ(I)  +  RELAX * DELDMZ(I)                      
  240      CONTINUE                                                     
  250      CALL LSYSLV (MSING, XI, XIOLD, Z, DMZ, DQZ, DQDMZ, G,        
     1          W, V, RHS, DUMMY, INTEGS, IPVTG, IPVTW, RNORM, 2,       
     2          FSUB, DFSUB, GSUB, DGSUB, GUESS )                       
C                                                                       
C...       compute a fixed jacobian iterate (used to control relax)     
C                                                                       
           CALL LSYSLV (MSING, XI, XIOLD, Z, DMZ, DQZ, DQDMZ, G,        
     1          W, V, RHS, DUMMY, INTEGS, IPVTG, IPVTW, RNORM, 4,       
     2          FSUB, DFSUB, GSUB, DGSUB, GUESS )                       
C                                                                       
C...       find scaled norms of various terms used to correct relax     
C                                                                       
           ANORM = 0.D0                                                 
           ANFIX = 0.D0                                                 
           DO 260 I = 1, NZPAR                                          
             ANORM = ANORM  +  (DELZ(I) * SCALE(I))**2                  
             ANFIX = ANFIX  +  (DQZ(I) * SCALE(I))**2                   
  260      CONTINUE                                                     
           DO 270 I = 1, NDMZ                                           
             ANORM = ANORM  +  (DELDMZ(I) * DSCALE(I))**2               
             ANFIX = ANFIX  +  (DQDMZ(I) * DSCALE(I))**2                
  270      CONTINUE                                                     
           ANORM = DSQRT(ANORM / DFLOAT(NZPAR+NDMZ))                    
           ANFIX = DSQRT(ANFIX / DFLOAT(NZPAR+NDMZ))                    
           IF ( IPRED .NE. 1 )                        GO TO 280         
           IF (IPRINT .LT. 0)  WRITE (IOUT,520) ITER, RELAX, ANORM,     
     1           ANFIX, RNOLD, RNORM                                    
           GO TO 290                                                    
  280      IF (IPRINT .LT. 0)  WRITE (IOUT,550) RELAX, ANORM, ANFIX,    
     1           RNOLD, RNORM                                           
  290      CONTINUE                                                     
C                                                                       
C...       check for monotonic decrease in  delz and deldmz.            
C                                                                       
           IF (ANFIX.LT.PRECIS .OR. RNORM.LT.PRECIS)  GO TO 390         
           IF ( ANFIX .GT. ANORM )                    GO TO 300         
C                                                                       
C...       we have a decrease.                                          
C...       if  dqz  and dqdmz  small, check for convergence             
C                                                                       
           IF ( ANFIX .LE. CHECK )                    GO TO 350         
C                                                                       
C...       correct the predicted  relax  unless the corrected           
C...       value is within 10 percent of the predicted one.             
C                                                                       
           IF ( IPRED .NE. 1 )                        GO TO 170         
  300      IF ( ITER .GE. LIMIT )                     GO TO 430         
C                                                                       
C...       correct the relaxation factor.                               
C                                                                       
           IPRED = 0                                                    
           ARG = (ANFIX/ANORM - 1.D0) / RELAX + 1.D0                    
           IF ( ARG .LT. 0.D0 )                       GO TO 170         
           IF (ARG .LE. .25D0*RELAX+.125D0*RELAX**2)  GO TO 310         
           FACTOR = -1.D0 + DSQRT (1.D0+8.D0 * ARG)                     
           IF ( DABS(FACTOR-1.D0) .LT. .1D0*FACTOR )  GO TO 170         
           IF ( FACTOR .LT. 0.5D0 )  FACTOR = 0.5D0                     
           RELAX = RELAX / FACTOR                                       
           GO TO 320                                                    
  310      IF ( RELAX .GE. .9D0 )                     GO TO 170         
           RELAX = 1.D0                                                 
  320      IF ( RELAX .LT. RELMIN )                   GO TO 440         
           FACT = RELAX - RLXOLD                                        
           DO 330 I = 1, NZPAR                                          
            Z(I) = Z(I)  +  FACT * DELZ(I)                              
  330      CONTINUE                                                     
           DO 340 I = 1, NDMZ                                           
             DMZ(I) = DMZ(I)  +  FACT * DELDMZ(I)                       
  340      CONTINUE                                                     
           RLXOLD = RELAX                                               
           GO TO 250                                                    
C                                                                       
C...       check convergence (iconv = 0).                               
C                                                                       
  350      CONTINUE                                                     
           DO 360 IT = 1, NTOL                                          
             INZ = LTOL(IT)                                             
             DO 360 IZ = INZ, NZ, MSTAR                                 
               IF ( DABS(DQZ(IZ)) .GT.                                  
     1           TOL(IT) * (DABS(Z(IZ)) + 1.D0) )   GO TO 170           
  360      CONTINUE                                                     
C                                                                       
C...       convergence obtained                                         
C                                                                       
           IF ( IPRINT .LT. 1 )  WRITE (IOUT,560) ITER                  
C                                                                       
C...       since convergence obtained, update  z and dmz  with term     
C...       from the fixed jacobian iteration.                           
C                                                                       
           DO 370 I = 1, NZPAR                                          
             Z(I) = Z(I)  +  DQZ(I)                                     
  370      CONTINUE                                                     
           DO 380 I = 1, NDMZ                                           
             DMZ(I) = DMZ(I)  +  DQDMZ(I)                               
  380      CONTINUE                                                     
  390      IF ( (ANFIX .LT. PRECIS .OR. RNORM .LT. PRECIS)              
     1          .AND. IPRINT .LT. 1 )  WRITE (IOUT,560) ITER            
           ICONV = 1                                                    
           IF ( ICARE .EQ. (-1) )  ICARE = 0                            
C                                                                       
C...       if full output has been requested, print values of the       
C...       solution components   z  at the meshpoints.                  
C                                                                       
  400      IF ( IPRINT .GE. 0 )                     GO TO 420           
           DO 410 J = 1, MSTAR                                          
             WRITE(IOUT,610) J                                          
  410      WRITE(IOUT,620) (Z(LJ), LJ = J, NZ, MSTAR)                   
           IF ( NPAR .EQ. 0 )                       GO TO 420           
           WRITE(IOUT,630)                                              
           DO 412 IPAR = 1, NPAR                                        
           LJ = MSTAR + IPAR                                            
  412      WRITE(IOUT,640) LJ, Z(NZ+IPAR)                               
C                                                                       
C...       check for error tolerance satisfaction                       
C                                                                       
  420      IFIN = 1                                                     
           IF (IMESH .EQ. 2) CALL ERRCHK (XI, Z, DMZ, VALSTR, IFIN)     
           IF ( IMESH .EQ. 1 .OR.                                       
     1          IFIN .EQ. 0 .AND. ICARE .NE. 2)     GO TO 460           
           IFLAG = 1                                                    
           RETURN                                                       
C                                                                       
C...       diagnostics for failure of nonlinear iteration.              
C                                                                       
  430      IF ( IPRINT .LT. 1 )  WRITE (IOUT,570) ITER                  
           GO TO 450                                                    
  440      IF( IPRINT .LT. 1 )  WRITE(IOUT,580) RELAX, RELMIN           
  450      IFLAG = -2                                                   
           NOCONV = NOCONV + 1                                          
           IF ( ICARE .EQ. 2 .AND. NOCONV .GT. 1 )  RETURN              
           IF ( ICARE .EQ. 0 )  ICARE = -1                              
C                                                                       
C...       update old mesh                                              
C                                                                       
  460      NP1 = N + 1                                                  
           DO 470 I = 1, NP1                                            
  470        XIOLD(I) = XI(I)                                           
           NOLD = N                                                     
C                                                                       
C...       pick a new mesh                                              
C...       check safeguards for mesh refinement                         
C                                                                       
           IMESH = 1                                                    
           IF ( ICONV .EQ. 0 .OR. MSHNUM .GE. MSHLMT                    
     1                       .OR. MSHALT .GE. MSHLMT )  IMESH = 2       
           IF ( MSHALT .GE. MSHLMT .AND.                                
     1          MSHNUM .LT. MSHLMT )  MSHALT = 1                        
           CALL NEWMSH (IMESH, XI, XIOLD, Z, DMZ, VALSTR,               
     1                  SLOPE, ACCUM, NFXPNT, FIXPNT)                   
C                                                                       
C...       exit if expected n is too large (but may try n=nmax once)    
C                                                                       
           IF ( N .LE. NMAX )                       GO TO 480           
           N = N / 2                                                    
           IFLAG = -1                                                   
           IF ( ICONV .EQ. 0 .AND. IPRINT .LT. 1 )  WRITE (IOUT,590)    
           IF ( ICONV .EQ. 1 .AND. IPRINT .LT. 1 )  WRITE (IOUT,600)    
           RETURN                                                       
  480      IF ( ICONV .EQ. 0 )  IMESH = 1                               
           IF ( ICARE .EQ. 1 )  ICONV = 0                               
      GO TO 20                                                          
C     ---------------------------------------------------------------   
  490 FORMAT(//35H THE GLOBAL BVP-MATRIX IS SINGULAR )                  
  495 FORMAT(//40H A LOCAL ELIMINATION MATRIX IS SINGULAR )             
  500 FORMAT(/30H FULL DAMPED NEWTON ITERATION,)                        
  510 FORMAT(13H ITERATION = , I3, 15H  NORM (RHS) = , D10.2)           
  520 FORMAT(13H ITERATION = ,I3,22H  RELAXATION FACTOR = ,D10.2        
     1       /33H NORM OF SCALED RHS CHANGES FROM ,D10.2,3H TO,D10.2    
     2       /33H NORM   OF   RHS  CHANGES  FROM  ,D10.2,3H TO,D10.2)   
  530 FORMAT(/27H FIXED JACOBIAN ITERATIONS,)                           
  540 FORMAT(/35H SWITCH TO DAMPED NEWTON ITERATION,)                   
  550 FORMAT(40H RELAXATION FACTOR CORRECTED TO RELAX = , D10.2         
     1       /33H NORM OF SCALED RHS CHANGES FROM ,D10.2,3H TO,D10.2    
     2       /33H NORM   OF   RHS  CHANGES  FROM  ,D10.2,3H TO,D10.2)   
  560 FORMAT(/18H CONVERGENCE AFTER , I3,11H ITERATIONS /)              
  570 FORMAT(/22H NO CONVERGENCE AFTER , I3, 11H ITERATIONS/)           
  580 FORMAT(/37H NO CONVERGENCE.  RELAXATION FACTOR =,D10.3            
     1       ,24H IS TOO SMALL (LESS THAN, D10.3, 1H)/)                 
  590 FORMAT(18H  (NO CONVERGENCE) )                                    
  600 FORMAT(50H  (PROBABLY TOLERANCES TOO STRINGENT, OR NMAX TOO       
     1       ,6HSMALL) )                                                
  610 FORMAT( 19H MESH VALUES FOR Z(, I2, 2H), )                        
  620 FORMAT(1H , 8D15.7)                                               
  630 FORMAT(' PARAMETER VALUES,')                                      
  640 FORMAT('   Z(',I2,'), ',D15.7)                                    
      END                                                               
      SUBROUTINE SKALE (N, KD, Z, XI, SCALE, DSCALE)                    
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose                                                             
C            provide a proper scaling of the state variables, used      
C            to control the damping factor for a newton iteration [2].  
C                                                                       
C   variables                                                           
C                                                                       
C            n      = number of mesh subintervals                       
C            kd     = number of unknowns in dmz                         
C            z      = the global unknown vector                         
C            xi     = the current mesh                                  
C            scale  = scaling vector for z                              
C            dscale = scaling vector for dmz                            
C                                                                       
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION Z(1), SCALE(1), DSCALE(KD,1)                            
      DIMENSION XI(1), BASM(5)                                          
C                                                                       
      COMMON /COLORD/ K, NCOMP, MSTAR, NPAR, MSTATE, ID2, MMAX, M(20)   
C                                                                       
      IZ = 1                                                            
      BASM(1) = 1.D0                                                    
      DO 50 J=1,N                                                       
        H = XI(J+1) - XI(J)                                             
        DO 10 L = 1, MMAX                                               
          BASM(L+1) = BASM(L) * H / DFLOAT(L)                           
  10    CONTINUE                                                        
        DO 40 ICOMP = 1, NCOMP                                          
          SCAL = (DABS(Z(IZ)) + DABS(Z(IZ+MSTAR))) * .5D0 + 1.D0        
          MJ = M(ICOMP)                                                 
          DO 20 L = 1, MJ                                               
            SCALE(IZ) = BASM(L) / SCAL                                  
            IZ = IZ + 1                                                 
  20      CONTINUE                                                      
          SCAL = BASM(MJ+1) / SCAL                                      
          DO 30 IDMZ = ICOMP, KD, NCOMP                                 
            DSCALE(IDMZ,J) = SCAL                                       
  30      CONTINUE                                                      
  40    CONTINUE                                                        
  50  CONTINUE                                                          
      DO 60 I = 1, MSTAR                                                
        SCALE(IZ) = SCALE(IZ-MSTAR)                                     
        IZ = IZ + 1                                                     
  60  CONTINUE                                                          
C                                                                       
C...  compute scaling for parameter components                          
C                                                                       
      IF ( NPAR .EQ. 0 )                            RETURN              
      DO 70 IPAR = 1, NPAR                                              
        SCALE(IZ) = 1.D0 / ( DABS(Z(IZ)) + 1.D0 )                       
        IZ = IZ + 1                                                     
   70 CONTINUE                                                          
      RETURN                                                            
      END                                                               
      SUBROUTINE PTHCON (XI, XIOLD, Z, DMZ, RHS, DELZ, DELDMZ,          
     1           DQZ, DQDMZ, G, W, V, VALSTR, SLOPE, SCALE, DSCALE,     
     2           XI0, Z0, DMZ0, TGT0, DMTGT0, TGT, DMTGT, ZOLD, DMZOLD, 
     3           ACCUM, IPVTG, INTEGS, IPVTW, NFXPNT, FIXPNT, IFLAG,    
     4           FSUB, DFSUB, GSUB, DGSUB, GUESS)                       
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose                                                             
C     pthcon is the actual driver routine for the continuation process. 
C     it controls the continuation steplength, the gauss-newton         
C     corrector, and the mesh according to the required tolerances.     
C                                                                       
C   variables                                                           
C                                                                       
C     xi     - current mesh                                             
C     xiold  - previous mesh                                            
C     xi0    - mesh of last accepted continuation point                 
C     z      - solution vector on mesh xi                               
C     dmz    - its m-th order derivatives                               
C     zold   - solution vector on mesh xiold                            
C     dmzold - its m-th order derivatives                               
C     z0     - solution vector on mesh xi0                              
C     dmz0   - its m-th order derivatives                               
C     tgt    - local tangent vector                                     
C     dmtgt  - its m-th order derivatives                               
C     tgt0   - tangent vector at solution point z0                      
C     dmtgt0 - its m-th order derivatives                               
C     ipath  - current status of continuation process                   
C            = 0  check consistency of initial guess                    
C            = 1  perform tangent continuation                          
C            = 2  hit final value                                       
C     sigma  - continuation steplength                                  
C     siglmt - minimal allowable value for sigma                        
C     nz     - number of components of  z    (without parameters)       
C     nzpar  - number of components of  z    (including parameters)     
C     ndmz   - number of components of  dmz                             
C     nzcon  - component of continuation parameter in  z                
C                                                                       
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION XI(1), XIOLD(1), Z(1), DMZ(1), RHS(1)                   
      DIMENSION DELZ(1), DELDMZ(1), DQZ(1), DQDMZ(1), FIXPNT(1)         
      DIMENSION G(1), W(1), V(1), VALSTR(1), SLOPE(1), ACCUM(1)         
      DIMENSION XI0(1), Z0(1), DMZ0(1), ZOLD(1), DMZOLD(1)              
      DIMENSION TGT0(1), DMTGT0(1), TGT(1), DMTGT(1)                    
      DIMENSION SCALE(1), DSCALE(1), DUMMY(1)                           
      DIMENSION INTEGS(1), IPVTG(1), IPVTW(1)                           
      DIMENSION DETNEW(2),DETOLD(2),DETSAV(2),DETPAR(2),DETPRS(2)       
C                                                                       
      COMMON /COLOUT/ PRECIS, IOUT, IPRINT                              
      COMMON /COLAPR/ N, NOLD, NMAX, NZ, NZPAR, NDMZ                    
      COMMON /COLORD/ K, NCOMP, MSTAR, NPAR, MSTATE, KD, MMAX, M(20)    
      COMMON /COLSID/ ZETA(40), ALEFT, ARIGHT, MSIDE, IZETA, IZSAVE     
      COMMON /COLMSH/ MSHFLG, MSHNUM, MSHLMT, MSHALT                    
      COMMON /COLNLN/ NONLIN, ITER, LIMIT, ICARE, IGUESS, IREAD         
      COMMON /COLPTH/ IPATH, ICON, LPAR                                 
      COMMON /COLPRM/ WINDOW(3), IDIREC, MAXSTP, IJOB                   
      COMMON /COLHOM/ ANORM, ANFIX, ANSUM, DOT                          
C                                                                       
      EXTERNAL FSUB, DFSUB, GSUB, DGSUB, GUESS                          
C                                                                       
C...  set constants for control of continuation process                 
C...  rename some parameters                                            
C                                                                       
      THMAX = 0.25D0                                                    
      THRESH = 0.25D0                                                   
      SIGLMT = 1.D-3                                                    
      PARMIN = WINDOW(1)                                                
      PARMAX = WINDOW(3)                                                
C                                                                       
C...  check consistency of initial guess                                
C                                                                       
      IF ( IPRINT .LT. 1 )  WRITE(IOUT,600)                             
      IPATH = 0                                                         
C                                                                       
C...  set limits on iterations and initialize counters.                 
C...  limit = maximum number of newton iterations per mesh.             
C...  see subroutine  newmsh  for the roles of  mshlmt , mshflg ,       
C...  mshnum , and  mshalt .                                            
C                                                                       
      MSHLMT = 3                                                        
      MSHFLG = 0                                                        
      MSHNUM = 1                                                        
      MSHALT = 1                                                        
      IMESH = 1                                                         
      ICONV = 0                                                         
      NOCONV = 0                                                        
      LIMIT = 9                                                         
      ICASE = 1                                                         
C                                                                       
C...  initialize mesh                                                   
C                                                                       
      CALL NEWMSH (3+IREAD, XI, XIOLD, DUMMY, DUMMY,                    
     1             DUMMY, DUMMY, DUMMY, NFXPNT, FIXPNT)                 
      IF ( IGUESS .GE. 2 )                          GO TO 20            
      NOLD = N                                                          
      NP1 = N + 1                                                       
      DO 10 I = 1, NP1                                                  
   10 XIOLD(I) = XI(I)                                                  
   20 IF ( IGUESS .GE. 2 )  IGUESS = 0                                  
C                                                                       
C...  invoke newton's method  (fixed continuation parameter)            
C                                                                       
   30 CALL CORRCT (XI, XIOLD, Z, DMZ, RHS, DELZ, DELDMZ,                
     1           DQZ, DQDMZ, TGT, DMTGT, DUMMY, DUMMY, G, W, V,         
     2           SCALE, DSCALE, IPVTG, INTEGS, IPVTW, 0, KFLAG,         
     3           FSUB, DFSUB, GSUB, DGSUB, GUESS)                       
C                                                                       
C...  branch according to error flag                                    
C                                                                       
      GO TO  (410, 430, 500, 420, 420), KFLAG                           
C                                                                       
C...  print values of the solution components  z                        
C...  at the meshpoints.                                                
C                                                                       
   40 IF ( IPRINT .EQ. 1 )                     GO TO 70                 
        WRITE(IOUT,610)                                                 
        DO 50 J = 1, MSTAR                                              
          WRITE(IOUT,620) J                                             
   50   WRITE(IOUT,630) (Z(LJ), LJ = J, NZ, MSTAR)                      
        WRITE(IOUT,640)                                                 
        DO 60 IPAR = 1, NPAR                                            
          LJ = MSTAR + IPAR                                             
   60   WRITE(IOUT,650) LJ, Z(NZ+IPAR)                                  
        WRITE(IOUT,660)                                                 
   70 CONTINUE                                                          
C                                                                       
C...  prepare first continuation step                                   
C                                                                       
      ISTEP = 1                                                         
      IPATH = 1                                                         
      LIMIT = 3                                                         
C                                                                       
C...  save solution with respect to old mesh                            
C                                                                       
      N0 = NOLD                                                         
      N = NOLD                                                          
      NZ = ( NOLD + 1 ) * MSTAR                                         
      NDMZ = KD * NOLD                                                  
      NZPAR = NZ + NPAR                                                 
      NZCON = NZ + ICON                                                 
      NP1 = NOLD + 1                                                    
      DO 80 I = 1, NP1                                                  
         XI0(I) = XIOLD(I)                                              
   80 CONTINUE                                                          
      DO 90 I = 1, NZPAR                                                
         Z0(I) = ZOLD(I)                                                
   90 CONTINUE                                                          
      DO 100 I = 1, NDMZ                                                
         DMZ0(I) = DMZOLD(I)                                            
  100 CONTINUE                                                          
C                                                                       
C...  determine scaling                                                 
C                                                                       
      CALL SCLCON (XI0, Z0, DMZ0, SCALE, DSCALE)                        
C                                                                       
C...  compute initial tangent                                           
C                                                                       
      CALL CORRCT (XI0, XIOLD, Z0, DMZ0, DUMMY, DUMMY, DUMMY,           
     1           DUMMY, DUMMY, TGT0, DMTGT0, DUMMY, DUMMY, G, W, V,     
     2           SCALE, DSCALE, IPVTG, INTEGS, IPVTW, 3, KFLAG,         
     3           FSUB, DFSUB, GSUB, DGSUB, GUESS)                       
      IF ( KFLAG .GT. 1 )                           GO TO 500           
C                                                                       
C...  compute determinants at initial point                             
C                                                                       
      INDPAR = 2 * MSTAR + ICON                                         
      INDOLD = INDPAR                                                   
C                                                                       
      S = 0.D0                                                          
      DO 110 I = 1, MSTATE                                              
        IND = N0 * MSTAR + I                                            
        T = DABS( TGT0(IND) ) / ( 1.D0 + DABS( Z0(IND) ) )              
        IF ( T .LE. S )                             GO TO 110           
           S = T                                                        
           INDNEW = I                                                   
  110 CONTINUE                                                          
      INDNEW = INDNEW + MSTAR                                           
C                                                                       
      CALL BLKDET( G, INTEGS, N0, IPVTG, MSTAR, IZSAVE,                 
     1     INDPAR, INDNEW, INDOLD, DETPRS, DETSAV, DETOLD )             
      INDOLD = INDNEW                                                   
C                                                                       
C...  provide solution information                                      
C                                                                       
      IEMBED = (N - 1) * MSTAR + INDOLD                                 
      CALL USER( XI0, Z0, DMZ0, TGT0, DMTGT0, 0, IEMBED, 0 )            
C                                                                       
C...  control sign of tangent                                           
C                                                                       
      SIGNUM = DSIGN( 1.D0, TGT0(NZCON) * DFLOAT(IDIREC) )              
C                                                                       
C...  initial steplength                                                
C                                                                       
      SIGMA = 0.1D0                                                     
      IF ( IPRINT .LT. 1 ) WRITE(IOUT,670) ISTEP                        
      IF ( IPRINT .LT. 1 ) WRITE(IOUT,680) SIGMA                        
      GO TO 260                                                         
C                                                                       
C...  check for acceptence of last computed point                       
C                                                                       
  120 IPATH = 1                                                         
C                                                                       
C...  update scaling                                                    
C                                                                       
      N = NOLD                                                          
      NZ = ( NOLD + 1 ) * MSTAR                                         
      NDMZ = KD * NOLD                                                  
      NZPAR = NZ + NPAR                                                 
      NZCON = NZ + ICON                                                 
      CALL SCLCON (XIOLD, ZOLD, DMZOLD, SCALE, DSCALE)                  
C                                                                       
C...  interpolate old tangent at new mesh points                        
C                                                                       
      CALL ZINT (NOLD, N0, XIOLD, XI0, DQZ, DQDMZ, TGT0, DMTGT0)        
      CALL L2DOT (XIOLD, DQZ, DQDMZ, DUMMY, DUMMY,                      
     1            SCALE, DSCALE, DUM, 1)                                
C                                                                       
C...  compute tangent direction                                         
C                                                                       
      CALL CORRCT (XIOLD, XIOLD, ZOLD, DMZOLD, DUMMY, DUMMY, DUMMY,     
     1           DUMMY, DUMMY, TGT, DMTGT, DUMMY, DUMMY, G, W, V,       
     2           SCALE, DSCALE, IPVTG, INTEGS, IPVTW, 3, KFLAG,         
     3           FSUB, DFSUB, GSUB, DGSUB, GUESS)                       
      IF ( KFLAG .GT. 1 )                           GO TO 240           
C                                                                       
C...  control sign of tangent                                           
C                                                                       
      CALL L2DOT (XIOLD, TGT, DMTGT, DQZ, DQDMZ,                        
     1            SCALE, DSCALE, DOTTGT, 2)                             
      IF ( DABS(DOTTGT) .LT. 0.3D0 )                GO TO 240           
      SIGNEW = DSIGN( 1.D0, DOTTGT ) * SIGNUM                           
C                                                                       
C...  calculate determinants                                            
C                                                                       
      S = 0.D0                                                          
      DO 130 I = 1, MSTATE                                              
        IND = NOLD * MSTAR + I                                          
        T = DABS( TGT(IND) ) / ( 1.D0 + DABS( ZOLD(IND) ) )             
        IF ( T .LE. S )                             GO TO 130           
            S = T                                                       
           INDNEW = I                                                   
  130 CONTINUE                                                          
      INDNEW = INDNEW + MSTAR                                           
C                                                                       
      CALL BLKDET( G, INTEGS, NOLD, IPVTG, MSTAR, IZSAVE,               
     1     INDPAR, INDNEW, INDOLD, DETPAR, DETNEW, DETOLD )             
C                                                                       
C...  compute and check safeguards                                      
C...  sgtgo - sign change of tangent wrt. old embedding                 
C...  sgtgn - sign change of tangent wrt. new embedding                 
C...  sgtgp - sign change of tangent wrt. continuation parm.            
C...  sgdte - sign change of determ. wrt. old embedding                 
C...  sgdtp - sign change of determ. wrt. continuation parm.            
C                                                                       
      SG = SIGNUM * SIGNEW                                              
      IND = ( N - 1 ) * MSTAR + INDOLD                                  
      SGTGO = DQZ(IND) * TGT(IND) * SG                                  
      IND = ( N - 1 ) * MSTAR + INDNEW                                  
      SGTGN = DQZ(IND) * TGT(IND) * SG                                  
      IF ( SGTGO .LT. 0.D0 .AND. SGTGN .LT. 0.D0 )  GO TO 240           
C                                                                       
      SGTGP = DQZ(NZCON) * TGT(NZCON) * SG                              
      SGDTP = DETPRS(1) * DETPAR(1)                                     
      SGDTE = DETSAV(1) * DETOLD(1) * SGTGO                             
      IF ( SGTGP * SGDTP * SGDTE .LT. 0.D0 )        GO TO 240           
C                                                                       
C...  printout for singular points                                      
C                                                                       
      IF ( IPRINT .EQ. 1 )                          GO TO 170           
      IF ( SGTGP .GE. 0.D0 .AND. SGDTP .GE. 0.D0                        
     1     .AND. SGDTE .GE. 0.D0 )                  GO TO 140           
      IF ( SGTGP .LT. 0.D0 )                                            
     1     ALPHA = DQZ(NZCON) / ( DQZ(NZCON) - TGT(NZCON) )             
C     IF ( SGTGP .GT. 0.D0 )  ALPHA = DETSAV(1) /                       
C    1   ( DETSAV(1)-DETOLD(1)*10.D0**(DETOLD(2)-DETSAV(2)) )           
      IND = ( N0 + 1 ) * MSTAR + ICON                                   
      PARLOC = Z0(IND) + ALPHA * ( ZOLD(NZCON) - Z0(IND) )              
C                                                                       
C     IF ( SGDTE .GT. 0.D0 )  WRITE(IOUT,690) PARLOC                    
C     IF ( SGDTP .GT. 0.D0 )  WRITE(IOUT,700) PARLOC                    
C     IF ( SGTGP .GT. 0.D0 )  WRITE(IOUT,710) PARLOC                    
      IF ( SGTGP .LT. 0.D0 )  WRITE(IOUT,690) PARLOC                    
  140 CONTINUE                                                          
C                                                                       
C...  print values of the solution components  z                        
C...  at the meshpoints.                                                
C                                                                       
         NZH = (2*N + 1) * MSTAR                                        
         WRITE(IOUT,610)                                                
         DO 150 J = 1, MSTAR                                            
            WRITE(IOUT,620) J                                           
  150    WRITE(IOUT,630) (Z(LJ), LJ = J, NZH, MSTAR)                    
         WRITE(IOUT,640)                                                
         DO 160 IPAR = 1, NPAR                                          
            LJ = MSTAR + IPAR                                           
  160    WRITE(IOUT,650) LJ, Z(NZH+IPAR)                                
  170 CONTINUE                                                          
C                                                                       
C...  compute turning point                                             
C...  not in this version                                               
C                                                                       
C     IF ( IJOB .NE. 1 .OR. SGTGP .GE. 0.D0 )       GO TO 180           
C     IND = INDNEW                                                      
C     IF ( DABS(SGTGO) .GT. DABS(SGTGN) )  IND = INDOLD                 
C                                                                       
C     CALL TURN (XI, XIOLD, Z, DMZ, RHS, DELZ, DELDMZ,                  
C    1     DQZ, DQDMZ, G, W, V, VALSTR, SLOPE, SCALE, DSCALE,           
C    2     N0, Z0, DMZ0, DQZ, DQDMZ, TGT, DMTGT, ZOLD, DMZOLD,          
C    3     ACCUM, IPVTG, INTEGS, IPVTW, NFXPNT, FIXPNT, SIGMA,          
C    4     IND, KFLAG, FSUB, DFSUB, GSUB, DGSUB, GUESS)                 
C     MODE = 2                                                          
C     IF ( KFLAG .GT. 1 )                           GO TO 240           
C 180 CONTINUE                                                          
C                                                                       
C...  continuation step accepted                                        
C...  check for having reached the endpoint                             
C                                                                       
      IF ( ICASE .EQ. 3 )                           GO TO 190           
      IF ( ZOLD(NZCON) .LE. PARMIN )                GO TO 340           
      IF ( ZOLD(NZCON) .GE. PARMAX )                GO TO 340           
C                                                                       
C...  provide solution information                                      
C                                                                       
  190 IEMBED = (N - 1) * MSTAR + INDOLD                                 
      CALL USER( XIOLD, ZOLD, DMZOLD, TGT, DMTGT, ISTEP, IEMBED, 1 )    
      IF ( IPRINT .LT. 1 )  WRITE(IOUT,660)                             
      IF ( ICASE .EQ. 3 )                           GO TO 480           
      IF ( ISTEP .GE. MAXSTP )                      GO TO 510           
C                                                                       
C...  prepare next continuation step                                    
C                                                                       
      SIGNUM = SIGNEW                                                   
      INDOLD = INDNEW                                                   
      DETPRS(1) = DETPAR(1)                                             
      DETPRS(2) = DETPAR(2)                                             
      DETSAV(1) = DETNEW(1)                                             
      DETSAV(2) = DETNEW(2)                                             
C                                                                       
C...  save solution with respect to old mesh                            
C                                                                       
      N0 = NOLD                                                         
      NP1 = NOLD + 1                                                    
      DO 200 I = 1, NP1                                                 
         XI0(I) = XIOLD(I)                                              
  200 CONTINUE                                                          
      DO 210 I = 1, NZPAR                                               
         Z0(I) = ZOLD(I)                                                
         TGT0(I) = TGT(I)                                               
  210 CONTINUE                                                          
      DO 220 I = 1, NDMZ                                                
         DMZ0(I) = DMZOLD(I)                                            
         DMTGT0(I) = DMTGT(I)                                           
  220 CONTINUE                                                          
C                                                                       
C...  steplength predictor                                              
C                                                                       
      ISTEP = ISTEP + 1                                                 
      IF ( IPRINT .LT. 1 )  WRITE(IOUT,670) ISTEP                       
      DOTCS = DABS( DOTSAV * DOTTGT ) +                                 
     1   DSQRT( (1.D0 - DOTSAV**2) * (1.D0 - DOTTGT**2) )               
      IF ( DOTCS .LT. DOTSAV )  DOTCS = DOTSAV                          
      SNOM = THMAX * (ANOSAV * DOTSAV)**2                               
      SDEN = ANFSAV * ANSSAV * DABS(DOTCS)**3                           
      IF ( SDEN .LT. THRESH * SNOM )  SDEN = THRESH * SNOM              
      RATIO = DSQRT( SNOM / SDEN )                                      
      IF ( RATIO .LT. 0.2D0 )  RATIO = 0.2D0                            
      SIGMA = RATIO * SIGMA                                             
      IF ( SIGMA .LT. SIGLMT )  SIGMA = SIGLMT                          
      IF ( IPRINT .LT. 1 )  WRITE(IOUT,680) SIGMA                       
      GO TO 260                                                         
C                                                                       
C...  steplength corrector                                              
C                                                                       
  230 CONTINUE                                                          
      THETA = ANFIX / ANORM                                             
      RATIO = DSQRT( THMAX / THETA )                                    
      IF ( RATIO .LT. 0.2D0 )  RATIO = 0.2D0                            
      GO TO 250                                                         
  240 CONTINUE                                                          
      RATIO = 0.5D0                                                     
  250 IF ( SIGMA .LT. SIGLMT )                      GO TO 520           
      SIGMA = RATIO * SIGMA                                             
      IF ( IPRINT .LT. 1 )  WRITE(IOUT,720) SIGMA                       
      IPATH = 1                                                         
C                                                                       
C...  perform a continuation step                                       
C                                                                       
  260 CONTINUE                                                          
C                                                                       
C...  load current mesh                                                 
C                                                                       
      IF ( MODE .EQ. 1 )                            GO TO 280           
      N = N0                                                            
      NOLD = N0                                                         
      NP1 = N + 1                                                       
      DO 270 I = 1, NP1                                                 
         XI(I) = XI0(I)                                                 
         XIOLD(I) = XI0(I)                                              
  270 CONTINUE                                                          
      CALL NEWMSH (4, XI, XIOLD, Z, DMZ, VALSTR,                        
     1             SLOPE, ACCUM, NFXPNT, FIXPNT)                        
      NZCON = NZ + ICON                                                 
      CALL SCLCON ( XI0, Z0, DMZ0, SCALE, DSCALE )                      
C                                                                       
C...  compute predictor                                                 
C...  if steplength is large enough try to hit final value              
C                                                                       
  280 CONTINUE                                                          
      SIGMIN = 0.D0                                                     
      SIGMAX = 0.D0                                                     
      IF ( DABS(TGT0(NZCON)) .LT. PRECIS )          GO TO 290           
      S = SIGNUM * TGT0(NZCON)                                          
      SIGMIN = ( PARMIN - Z0(NZCON) ) / S                               
      SIGMAX = ( PARMAX - Z0(NZCON) ) / S                               
      IF ( SIGMIN .GT. 0.D0 .AND. SIGMA .GT. SIGMIN )  SIGMA = SIGMIN   
      IF ( SIGMAX .GT. 0.D0 .AND. SIGMA .GT. SIGMAX )  SIGMA = SIGMAX   
  290 CONTINUE                                                          
      S = SIGNUM * SIGMA                                                
      DO 300 I = 1, NZPAR                                               
        Z(I) = Z0(I) + S * TGT0(I)                                      
  300 CONTINUE                                                          
      DO 310 I = 1, NDMZ                                                
        DMZ(I) = DMZ0(I) + S * DMTGT0(I)                                
  310 CONTINUE                                                          
      IF ( SIGMA .EQ. SIGMIN .OR.                                       
     1     SIGMA .EQ. SIGMAX )                      GO TO 380           
C                                                                       
C...  correct                                                           
C                                                                       
      MSHLMT = 3                                                        
      MSHFLG = 0                                                        
      MSHNUM = 1                                                        
      MSHALT = 1                                                        
      IMESH = 1                                                         
      ICONV = 0                                                         
      NOCONV = 0                                                        
      MODE = 1                                                          
      ICASE = 2                                                         
  320 CONTINUE                                                          
      CALL CORRCT (XI, XIOLD, Z, DMZ, RHS, DELZ, DELDMZ,                
     1           DQZ, DQDMZ, TGT, DMTGT, TGT0, DMTGT0, G, W, V,         
     2           SCALE, DSCALE, IPVTG, INTEGS, IPVTW, MODE, KFLAG,      
     3           FSUB, DFSUB, GSUB, DGSUB, GUESS)                       
C                                                                       
C...  save quantities for steplength control                            
C                                                                       
      IF ( MODE .NE. 1 .OR. KFLAG .NE. 1 )          GO TO 330           
      ANOSAV = ANORM                                                    
      ANFSAV = ANFIX                                                    
      ANSSAV = ANSUM                                                    
      DOTSAV = DOT                                                      
  330 CONTINUE                                                          
C                                                                       
C...  branch according to error flag                                    
C                                                                       
      GO TO  (410, 430, 240, 420, 240), KFLAG                           
C                                                                       
C...  hit final value                                                   
C                                                                       
  340 CONTINUE                                                          
      N = NOLD                                                          
      NP1 = NOLD + 1                                                    
      DO 350 I = 1, NP1                                                 
         XI(I) = XIOLD(I)                                               
  350 CONTINUE                                                          
      CALL NEWMSH (4, XI, XIOLD, Z, DMZ, VALSTR,                        
     1             SLOPE, ACCUM, NFXPNT, FIXPNT)                        
      NZCON = NZ + ICON                                                 
      DO 360 I = 1, NZPAR                                               
         Z(I) = ZOLD(I)                                                 
  360 CONTINUE                                                          
      IF ( ZOLD(NZCON) .LT. PARMIN )  Z(NZCON) = PARMIN                 
      IF ( ZOLD(NZCON) .GT. PARMAX )  Z(NZCON) = PARMAX                 
      DO 370 I = 1, NDMZ                                                
         DMZ(I) = DMZOLD(I)                                             
  370 CONTINUE                                                          
      GO TO 390                                                         
  380 CONTINUE                                                          
      IF ( SIGMA .EQ. SIGMIN )  Z(NZCON) = PARMIN                       
      IF ( SIGMA .EQ. SIGMAX )  Z(NZCON) = PARMAX                       
  390 CONTINUE                                                          
      IPATH = 2                                                         
      IF ( IPRINT .LT. 1 )  WRITE(IOUT,730)                             
      MSHLMT = 3                                                        
      MSHFLG = 0                                                        
      MSHNUM = 1                                                        
      MSHALT = 1                                                        
      IMESH = 1                                                         
      ICONV = 0                                                         
      NOCONV = 0                                                        
      MODE = 1                                                          
      IF ( Z(NZCON) .EQ. PARMIN .AND.                                   
     1     TGT0(NZCON) * SIGNUM .GE. 0.D0 )         GO TO 240           
      IF ( Z(NZCON) .EQ. PARMAX .AND.                                   
     1     TGT0(NZCON) * SIGNUM .LE. 0.D0 )         GO TO 240           
      ICASE = 3                                                         
  400 CONTINUE                                                          
      CALL CORRCT (XI, XIOLD, Z, DMZ, RHS, DELZ, DELDMZ,                
     1           DQZ, DQDMZ, TGT, DMTGT, DUMMY, DUMMY, G, W, V,         
     2           SCALE, DSCALE, IPVTG, INTEGS, IPVTW, 0, KFLAG,         
     3           FSUB, DFSUB, GSUB, DGSUB, GUESS)                       
C                                                                       
C...  branch according to error flag                                    
C                                                                       
      GO TO  (410, 430, 240, 420, 420), KFLAG                           
C                                                                       
C...       check for error tolerance satisfaction                       
C                                                                       
  410      ICONV = 1                                                    
           IF ( IMESH .EQ. 1 )                      GO TO 430           
           CALL ERRCHK (XI, Z, DMZ, VALSTR, IFIN)                       
           IF ( IFIN .EQ. 0 )                       GO TO 430           
           GOTO (40,120,120),ICASE                                      
C                                                                       
C...       diagnostics for failure of nonlinear iteration.              
C                                                                       
  420      CONTINUE                                                     
           NOCONV = NOCONV + 1                                          
           IF ( NOCONV .GT. 1 .AND. IPATH .NE. 0 )  GO TO 230           
           IF ( NOCONV .GE. 3 )                     GO TO 500           
C                                                                       
C...       update old mesh                                              
C                                                                       
  430      NP1 = N + 1                                                  
           DO 440 I = 1, NP1                                            
  440        XIOLD(I) = XI(I)                                           
           NOLD = N                                                     
C                                                                       
C...       pick a new mesh                                              
C...       check safeguards for mesh refinement                         
C                                                                       
           IMESH = 1                                                    
           IF ( ICONV .EQ. 0 .OR. MSHNUM .GE. MSHLMT                    
     1                       .OR. MSHALT .GE. MSHLMT )  IMESH = 2       
           IF ( MSHALT .GE. MSHLMT .AND.                                
     1          MSHNUM .LT. MSHLMT )  MSHALT = 1                        
           CALL NEWMSH (IMESH, XI, XIOLD, Z, DMZ, VALSTR,               
     1                  SLOPE, ACCUM, NFXPNT, FIXPNT)                   
           NZCON = NZ + ICON                                            
C                                                                       
C...     save solution with respect to xiold                            
C                                                                       
         IF ( IMESH .NE. 2 )                        GO TO 470           
         NZPOLD = ( NOLD + 1) * MSTAR + NPAR                            
         NDMOLD = NOLD * KD                                             
         DO 450 I = 1, NZPOLD                                           
            ZOLD(I) = Z(I)                                              
  450    CONTINUE                                                       
         DO 460 I = 1, NDMOLD                                           
            DMZOLD(I) = DMZ(I)                                          
  460    CONTINUE                                                       
C                                                                       
C...       exit if expected n is too large (but may try n=nmax once)    
C                                                                       
  470      IF ( N .GT. NMAX )                       GO TO 490           
           IF ( ICONV .EQ. 0 )  IMESH = 1                               
           MODE = 2                                                     
         GOTO (30,320,400),ICASE                                        
C                                                                       
C...  solution exit                                                     
C                                                                       
  480 IFLAG = 1                                                         
      RETURN                                                            
C                                                                       
C...  error exits                                                       
C...  expected number of subintervals exceeds storage specifications    
C                                                                       
  490 IFLAG = -1                                                        
      N = N / 2                                                         
      IF ( ICONV .EQ. 0 .AND. IPRINT .LT. 1 )  WRITE (IOUT,740)         
      IF ( ICONV .EQ. 1 .AND. IPRINT .LT. 1 )  WRITE (IOUT,750)         
      RETURN                                                            
C                                                                       
C...  no regular starting point found                                   
C                                                                       
  500 IFLAG = -4                                                        
      IF ( IPRINT .LT. 1 )  WRITE(IOUT,760)                             
      RETURN                                                            
C                                                                       
C...  maximal number of continuation steps reached                      
C                                                                       
  510 IFLAG = -5                                                        
      IF ( IPRINT .LT. 1 )  WRITE(IOUT,770)                             
      RETURN                                                            
C                                                                       
C...  continuation stepsize too small                                   
C                                                                       
  520 IFLAG = -6                                                        
      IF ( IPRINT .LT. 1 )  WRITE(IOUT,780)                             
      RETURN                                                            
C                                                                       
C     ---------------------------------------------------------------   
  600 FORMAT(1H1,132(1H*)//21H CHECK OF CONSISTENCY)                    
  610 FORMAT(1H )                                                       
  620 FORMAT(19H MESH VALUES FOR Z(, I2, 2H),)                          
  630 FORMAT(1H , 8D15.7)                                               
  640 FORMAT(18H PARAMETER VALUES,)                                     
  650 FORMAT(5H   Z(,I2,3H), ,D15.7)                                    
  660 FORMAT(/1H ,132(1H*))                                             
  670 FORMAT(///1H ,132(1H*)//22H CONTINUATION STEP NO.,I5/)            
  680 FORMAT(31H CONTINUATION STEPSIZE SIGMA = ,D10.2)                  
  690 FORMAT(/44H POSSIBLY TURNING POINT NEAR PARAMETER VALUE,D10.2)    
C 700 FORMAT(/40H POSSIBLY PITCHFORK NEAR PARAMETER VALUE,D10.2)        
C 710 FORMAT(/49H POSSIBLY SIMPLE BIFURCATION NEAR PARAMETER VALUE,     
C    1        D10.2)                                                    
  720 FORMAT(/42H CONTINUATION STEPSIZE REDUCED TO SIGMA = ,D10.2)      
  730 FORMAT(/23H TRY TO HIT FINAL VALUE)                               
  740 FORMAT(/18H  (NO CONVERGENCE) )                                   
  750 FORMAT(/50H  (PROBABLY TOLERANCES TOO STRINGENT, OR NMAX TOO      
     1       ,6HSMALL) )                                                
  760 FORMAT(/32H NO REGULAR STARTING POINT FOUND)                      
  770 FORMAT(/45H MAXIMAL NUMBER OF CONTINUATION STEPS REACHED)         
  780 FORMAT(/32H CONTINUATION STEPSIZE TOO SMALL)                      
      END                                                               
      SUBROUTINE CORRCT (XI, XIOLD, Z, DMZ, RHS, DELZ, DELDMZ,          
     1           DQZ, DQDMZ, TGT, DMTGT, TGT0, DMTGT0, G, W, V,         
     2           SCALE, DSCALE, IPVTG, INTEGS, IPVTW, MODE, KFLAG,      
     3           FSUB, DFSUB, GSUB, DGSUB, GUESS)                       
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose                                                             
C     this subroutine performs the gauss-newton corrector of the        
C     continuation process.                                             
C                                                                       
C        corrct operates according to one of 4 modes:                   
C     mode = 0 - perform newton's method.                               
C     mode = 1 - first step of gauss-newton corrector.                  
C     mode = 2 - finish gauss-newton corrector.                         
C     mode = 3 - compute tangent at given point.                        
C                                                                       
C   variables                                                           
C                                                                       
C     xi     - current mesh                                             
C     xiold  - previous mesh                                            
C     z      - the solution vector                                      
C     dmz    - its m-th order derivatives                               
C     delz   - gauss-newton correction                                  
C     deldmz - its m-th order derivatives                               
C     dqz    - simplified gauss-newton correction                       
C     dqdmz  - its m-th order derivatives                               
C     tgt    - local tangent vector                                     
C     dmtgt  - its m-th order derivatives                               
C     tgt0   - tangent vector at last solution point                    
C     dmtgt0 - its m-th order derivatives                               
C     rhs    - residuum of gauss-newton iterations                      
C     check  - checking for nonlinear iteration convergence             
C     ifreez - number of fixed jacobian iterations                      
C     lmtfrz - maximum value for ifreez before performing a reinversion 
C     iter   - number of iterations (counted only when jacobian         
C              reinversions are performed)                              
C     rnorm  - norm of rhs (right hand side) for current iteration      
C     rnold  - norm of rhs for previous iteration                       
C     anfix  - scaled norm of simplified gauss-newton correction        
C     anorm  - scaled norm of gauss-newton correction                   
C     nz     - number of components of  z    (without parameters)       
C     nzpar  - number of components of  z    (including parameters)     
C     ndmz   - number of components of  dmz                             
C     nzcon  - component of continuation parameter in  z                
C                                                                       
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION XI(1), XIOLD(1), Z(1), DMZ(1), RHS(1)                   
      DIMENSION DELZ(1), DELDMZ(1), DQZ(1), DQDMZ(1)                    
      DIMENSION TGT(1), DMTGT(1), TGT0(1), DMTGT0(1)                    
      DIMENSION G(1), W(1), V(1)                                        
      DIMENSION SCALE(1), DSCALE(1), DUMMY(1)                           
      DIMENSION INTEGS(1), IPVTG(1), IPVTW(1)                           
C                                                                       
      COMMON /COLOUT/ PRECIS, IOUT, IPRINT                              
      COMMON /COLAPR/ N, NOLD, NMAX, NZ, NZPAR, NDMZ                    
      COMMON /COLORD/ K, NCOMP, MSTAR, NPAR, MSTATE, KD, MMAX, M(20)    
      COMMON /COLNLN/ NONLIN, ITER, LIMIT, ICARE, IGUESS, IREAD         
      COMMON /COLPTH/ IPATH, ICON, LPAR                                 
      COMMON /COLHOM/ ANORMS, ANFIXS, ANSUMS, DOTS                      
      COMMON /COLEST/ WGTMSH(40), WGTERR(40), TOL(40),                  
     1                ROOT(40), JTOL(40), LTOL(40), NTOL                
C                                                                       
      EXTERNAL FSUB, DFSUB, GSUB, DGSUB, GUESS                          
C                                                                       
C...  constants for control of nonlinear iteration                      
C                                                                       
      MSING = 0                                                         
      KFLAG = 1                                                         
      LMTFRZ = 4                                                        
      NZCON = NZ + ICON                                                 
      IF ( MODE .EQ. 3 )                            GO TO 260           
      INIT = 1                                                          
      IF ( MODE .EQ. 1 )  INIT = 0                                      
      IF ( MODE .EQ. 1 )  ANSUMS = 0.D0                                 
C                                                                       
C...  compute the maximum tolerance                                     
C                                                                       
      CHECK = 0.D0                                                      
      DO 10 I = 1, NTOL                                                 
   10   CHECK = DMAX1 ( TOL(I), CHECK )                                 
C                                                                       
C...       iteration loop                                               
C                                                                       
           IF ( IPRINT .LT. 0 )  WRITE(IOUT,400)                        
           ITER = 0                                                     
C                                                                       
C...       evaluate right hand side and its norm  and                   
C...       find the first newton correction                             
C                                                                       
           DELZ(NZPAR) = 0.D0                                           
           PTEMP = Z( (NOLD+1) * MSTAR + ICON )                         
           CALL LSYSLV (MSING, XI, XIOLD, Z, DMZ, DELZ, DELDMZ, G,      
     1          W, V, RHS, DQDMZ, INTEGS, IPVTG, IPVTW, RNOLD, 1,       
     2          FSUB, DFSUB, GSUB, DGSUB, GUESS )                       
C                                                                       
C...       check for a singular matrix                                  
C                                                                       
           IF ( MSING .NE. 0 )                      GO TO 270           
C                                                                       
C...       bookkeeping for first mesh                                   
C                                                                       
           IF ( IGUESS .EQ. 1 )  IGUESS = 0                             
C                                                                       
C...       determine tangent vector                                     
C                                                                       
           IF ( IPATH .NE. 1 )                      GO TO 50            
           CALL TANGNT (G, V, INTEGS, IPVTG, TGT, DMTGT)                
C                                                                       
C...       correct projection of trajectory with respect to parameter   
C                                                                       
           ALPHA = ( PTEMP - Z(NZCON) ) / TGT(NZCON)                    
           DO 20 I = 1, NZPAR                                           
             Z(I) = Z(I) + ALPHA * TGT(I)                               
   20      CONTINUE                                                     
           DO 30 I = 1, NDMZ                                            
             DMZ(I) = DMZ(I) + ALPHA * DMTGT(I)                         
   30      CONTINUE                                                     
C                                                                       
C...       determine scaling                                            
C                                                                       
           IF ( MODE .EQ. 2 )  CALL SCLCON (XI, Z, DMZ, SCALE, DSCALE)  
C                                                                       
C...       normalize tangent                                            
C                                                                       
           CALL L2DOT (XI, TGT, DMTGT, DUMMY, DUMMY,                    
     1                 SCALE, DSCALE, DUM, 1)                           
C                                                                       
C...       check angle between tangents                                 
C                                                                       
           IF ( INIT .EQ. 1 )                       GO TO 40            
           CALL L2DOT (XI, TGT0, DMTGT0, TGT, DMTGT,                    
     1                 SCALE, DSCALE, DOTS, 2)                          
           IF ( DABS(DOTS) .LT. 0.3D0 )             GO TO 310           
C                                                                       
C...       project for gauss-newton correction                          
C                                                                       
   40      CALL L2DOT (XI, DELZ, DELDMZ, TGT, DMTGT,                    
     1                 SCALE, DSCALE, DUM, 3)                           
           GO TO 70                                                     
C                                                                       
C...       determine scaling in case of newton's method                 
C                                                                       
   50      CALL SCLCON (XI, Z, DMZ, SCALE, DSCALE)                      
           GO TO 70                                                     
C                                                                       
C...       set up new jacobian and solve for the next iterate           
C...       ( ifreez = 0 ) .                                             
C                                                                       
   60      DELZ(NZPAR) = 0.D0                                           
           CALL LSYSLV (MSING, XI, XIOLD, Z, DMZ, DELZ, DELDMZ, G,      
     1          W, V, RHS, DUMMY, INTEGS, IPVTG, IPVTW, RNORM,          
     2          3, FSUB, DFSUB, GSUB, DGSUB, GUESS )                    
C                                                                       
C...       check for a singular matrix                                  
C                                                                       
           IF ( MSING .NE. 0 )                      GO TO 280           
C                                                                       
C...       project for gauss-newton correction                          
C                                                                       
           IF ( IPATH .NE. 1 )                      GO TO 70            
           CALL TANGNT (G, V, INTEGS, IPVTG, TGT, DMTGT)                
           CALL L2DOT (XI, TGT, DMTGT, DUMMY, DUMMY,                    
     1                 SCALE, DSCALE, DUM, 1)                           
           CALL L2DOT (XI, DELZ, DELDMZ, TGT, DMTGT,                    
     1                 SCALE, DSCALE, DUM, 3)                           
C                                                                       
C...       compute norm of gauss-newton correction with scaling         
C                                                                       
   70      CALL L2DOT (XI, DELZ, DELDMZ, DUMMY, DUMMY,                  
     1                 SCALE, DSCALE, ANORM, 0)                         
           ANORM = DSQRT(ANORM)                                         
C                                                                       
C...       a full newton step                                           
C                                                                       
           ITER = ITER + 1                                              
           IF ( ITER .GT. LIMIT )                   GO TO 290           
           IFREEZ = 0                                                   
C                                                                       
C...       update   z and dmz , compute new  rhs  and its norm          
C                                                                       
   80      DO 90 I = 1, NZPAR                                           
             Z(I) = Z(I) + DELZ(I)                                      
   90      CONTINUE                                                     
           DO 100 I = 1, NDMZ                                           
             DMZ(I) = DMZ(I) + DELDMZ(I)                                
  100      CONTINUE                                                     
C                                                                       
C...       evaluate new  rhs  and its norm                              
C                                                                       
           CALL LSYSLV (MSING, XI, XIOLD, Z, DMZ, DELZ, DELDMZ, G,      
     1          W, V, RHS, DUMMY, INTEGS, IPVTG, IPVTW, RNORM, 2,       
     2          FSUB, DFSUB, GSUB, DGSUB, GUESS )                       
C                                                                       
           IF ( RNORM .LT. PRECIS )                 GO TO 250           
C                                                                       
C...       evaluate fixed jacobian correction                           
C                                                                       
           DQZ(NZPAR) = 0.D0                                            
           CALL LSYSLV (MSING, XI, XIOLD, Z, DMZ, DQZ, DQDMZ, G,        
     1          W, V, RHS, DUMMY, INTEGS, IPVTG, IPVTW, RNORM, 4,       
     2          FSUB, DFSUB, GSUB, DGSUB, GUESS )                       
C                                                                       
C...       project for gauss-newton correction                          
C                                                                       
           IF ( IPATH .EQ. 1 )                                          
     1        CALL L2DOT (XI, DQZ, DQDMZ, TGT, DMTGT,                   
     2                    SCALE, DSCALE, DUM, 3)                        
C                                                                       
C...       compute norm of gauss-newton correction with scaling         
C                                                                       
           CALL L2DOT (XI, DQZ, DQDMZ, DUMMY, DUMMY,                    
     1                 SCALE, DSCALE, ANFIX, 0)                         
           ANFIX = DSQRT(ANFIX)                                         
C                                                                       
C...       check monotonicity. if the norm of the corrections           
C...       decrease fast enough proceed with a fixed jacobian;          
C...       else proceed cautiously.                                     
C                                                                       
           IF ( ANFIX .GT. ANORM .AND.                                  
     1      ( ANFIX .GT. CHECK .OR. ITER .NE. 1 .OR. IFREEZ .NE. 0 ) )  
     2                                              GO TO 220           
           IF ( IPRINT .LT. 0 )  WRITE (IOUT,410) ITER, ANORM, ANFIX,   
     1                                                  RNOLD, RNORM    
           RNOLD = RNORM                                                
C                                                                       
C...       save values and sum up norm of corrections                   
C...       for steplength control of continuation process.              
C                                                                       
           IF ( MODE .NE. 1 )                       GO TO 130           
           IF ( INIT .EQ. 1 )                       GO TO 120           
           ANORMS = ANORM                                               
           ANFIXS = ANFIX                                               
           INIT = 1                                                     
  120      ANSUMS = ANSUMS + ANORM                                      
           IF ( ANSUMS .GT. 2.D0 * ANORMS )         GO TO 320           
C                                                                       
C...       verify that the linear convergence with fixed jacobian       
C...       is fast enough.                                              
C                                                                       
  130      IFREEZ = IFREEZ + 1                                          
           IF ( IFREEZ .GT. LMTFRZ )     IFREEZ = 0                     
           IF ( ANORM .LT. 4.D0*ANFIX )  IFREEZ = 0                     
C                                                                       
C...       check convergence.                                           
C                                                                       
           IF ( ANFIX .GT. CHECK )                  GO TO 190           
           DO 160 IT = 1, NTOL                                          
             INZ = LTOL(IT)                                             
             DO 160 IZ = INZ, NZ, MSTAR                                 
               IF ( DABS(DQZ(IZ)) .GT.                                  
     1           TOL(IT) * ( DABS(Z(IZ)) + 1.D0) )  GO TO 190           
  160      CONTINUE                                                     
C                                                                       
C...       convergence obtained                                         
C                                                                       
           DO 170 I = 1, NZPAR                                          
             Z(I) = Z(I) + DQZ(I)                                       
  170      CONTINUE                                                     
           DO 180 I = 1, NDMZ                                           
             DMZ(I) = DMZ(I) + DQDMZ(I)                                 
  180      CONTINUE                                                     
           GO TO 250                                                    
C                                                                       
C...       prepare next iteration                                       
C                                                                       
  190      IF ( IFREEZ .EQ. 0 )                     GO TO 60            
           ANORM = ANFIX                                                
           DO 200 I = 1, NZPAR                                          
             DELZ(I) = DQZ(I)                                           
  200      CONTINUE                                                     
           DO 210 I = 1, NDMZ                                           
             DELDMZ(I) = DQDMZ(I)                                       
  210      CONTINUE                                                     
           GO TO 80                                                     
C                                                                       
C...      convergence of fixed jacobian iteration failed.               
C...      for ifreez > 0 try a full newton step.                        
C                                                                       
  220      DO 230 I = 1, NZPAR                                          
             Z(I) = Z(I) - DELZ(I)                                      
  230      CONTINUE                                                     
           DO 240 I = 1, NDMZ                                           
             DMZ(I) = DMZ(I) - DELDMZ(I)                                
  240      CONTINUE                                                     
           IF ( IFREEZ .EQ. 0 )                     GO TO 300           
C                                                                       
C...       evaluate new  rhs  and its norm                              
C                                                                       
           CALL LSYSLV (MSING, XI, XIOLD, Z, DMZ, DELZ, DELDMZ, G,      
     1          W, V, RHS, DUMMY, INTEGS, IPVTG, IPVTW, RNOLD, 2,       
     2          FSUB, DFSUB, GSUB, DGSUB, GUESS )                       
           GO TO 60                                                     
C                                                                       
C...       solution of gauss-newton iteration obtained                  
C                                                                       
  250      IF ( IPRINT .LT. 1 )  WRITE(IOUT,430) ITER                   
           IF ( INIT .EQ. 1 )    RETURN                                 
           ANORMS = 1.D0                                                
           ANFIXS = 0.D0                                                
           RETURN                                                       
C                                                                       
C...       set up new jacobian and compute tangent vector               
C                                                                       
  260      CALL LSYSLV (MSING, XI, XIOLD, Z, DMZ, DELZ, DELDMZ, G,      
     1          W, V, RHS, DQDMZ, INTEGS, IPVTG, IPVTW, RNORM,          
     2          5, FSUB, DFSUB, GSUB, DGSUB, GUESS )                    
C                                                                       
C...       check for a singular matrix                                  
C                                                                       
           IF ( MSING .NE. 0 )                      GO TO 270           
C                                                                       
C...       normalize tangent                                            
C                                                                       
           CALL TANGNT (G, V, INTEGS, IPVTG, TGT, DMTGT)                
           CALL L2DOT (XI, TGT, DMTGT, DUMMY, DUMMY,                    
     1                 SCALE, DSCALE, DUM, 1)                           
           RETURN                                                       
C                                                                       
C...       error returns                                                
C                                                                       
  270      IF ( MSING .LT. 0 )                      GO TO 280           
C                                                                       
C...       local elimination matrix is singular                         
C                                                                       
           IF ( IPRINT .LT. 1 )  WRITE (IOUT,440)                       
           KFLAG = 2                                                    
           RETURN                                                       
C                                                                       
C...       global bvp-matrix is singular                                
C                                                                       
  280      IF ( IPRINT .LT. 1 )  WRITE (IOUT,450)                       
           KFLAG = 3                                                    
           RETURN                                                       
C                                                                       
C...       no convergence after itmax iterations                        
C                                                                       
  290      IF ( IPRINT .LT. 1 )  WRITE (IOUT,460)                       
           KFLAG = 3                                                    
           RETURN                                                       
C                                                                       
C...       no convergence of gauss-newton method                        
C                                                                       
  300      IF ( IPRINT .LT. 0 )  WRITE (IOUT,410) ITER, ANORM, ANFIX,   
     1                                                  RNOLD, RNORM    
           IF ( IPRINT .LT. 1 )  WRITE (IOUT,420) ITER                  
           KFLAG = 4                                                    
           ANORMS = ANORM                                               
           ANFIXS = ANFIX                                               
           RETURN                                                       
C                                                                       
C...       scalar product of tangents too small                         
C                                                                       
  310      IF ( IPRINT .LT. 1 )  WRITE(IOUT,470) DOTS                   
           KFLAG = 5                                                    
           RETURN                                                       
C                                                                       
C...       convergence too slow                                         
C                                                                       
  320      IF ( IPRINT .LT. 1 )  WRITE(IOUT,480) ANSUMS                 
           KFLAG = 5                                                    
           RETURN                                                       
C     ---------------------------------------------------------------   
  400 FORMAT(/27H NEWTON CHORD ITERATIONS,  )                           
  410 FORMAT(13H ITERATION = ,I3                                        
     1       /33H NORM OF SCALED RHS CHANGES FROM ,D10.2,3H TO,D10.2    
     2       /33H NORM   OF   RHS   CHANGES  FROM ,D10.2,3H TO,D10.2)   
  420 FORMAT(/22H NO CONVERGENCE AFTER , I3, 11H ITERATIONS/)           
  430 FORMAT(/18H CONVERGENCE AFTER , I3,11H ITERATIONS /)              
  440 FORMAT(//40H A LOCAL ELIMINATION MATRIX IS SINGULAR )             
  450 FORMAT(//35H THE GLOBAL BVP-MATRIX IS SINGULAR )                  
  460 FORMAT(//40H MAXIMAL NUMBER OF ITERATIONS REACHED   )             
  470 FORMAT(//40H SCALAR PRODUCT OF TANGENTS TOO SMALL   ,D10.2)       
  480 FORMAT(//40H CONVERGENCE TOO SLOW                   ,D10.2)       
      END                                                               
      SUBROUTINE SCLCON (XI, Z, DMZ, SCALE, DSCALE)                     
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose                                                             
C            provide a proper scaling of the state variables.           
C                                                                       
C   variables                                                           
C                                                                       
C            xi     = the current mesh                                  
C            z      = the solution vector                               
C            dmz    = m-th order derivatives                            
C            scale  = scaling vector for z                              
C            dscale = scaling vector for dmz                            
C                                                                       
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION XI(1), Z(1), DMZ(1), SCALE(1), DSCALE(1)                
      DIMENSION ZVAL(40)                                                
C                                                                       
      COMMON /COLLOC/ RHO(7), COEF(49)                                  
      COMMON /COLBAS/ B(28), ACOL(28,7), ASAVE(28,4)                    
      COMMON /COLAPR/ N, NOLD, NMAX, NZ, NZPAR, NDMZ                    
      COMMON /COLORD/ K, NCOMP, MSTAR, NPAR, MSTATE, KD, MMAX, M(20)    
C                                                                       
C...  compute scaling for differential components                       
C                                                                       
      LSCALE = 1                                                        
      LDSCAL = 1                                                        
      DO 40 I = 1, N                                                    
        XMSH = XI(I)                                                    
        HI = XI(I+1) - XI(I)                                            
        DO 30 J = 1, K                                                  
          XCOL = XMSH + HI * RHO(J)                                     
          CALL APPROX (I, XCOL, ZVAL, ACOL(1,J), DUMMY, XI, N,          
     1          Z, DMZ, K, NCOMP, MMAX, M, MSTAR, 4, DUMMY, 0)          
          DO 10 L = 1, MSTAR                                            
            SCALE(LSCALE) = 1.D0 / ( DABS(ZVAL(L)) + 1.D0 )             
            LSCALE = LSCALE + 1                                         
   10     CONTINUE                                                      
          IZ = 0                                                        
          DO 20 ICOMP = 1, NCOMP                                        
          MJ = M(ICOMP)                                                 
          IZ = IZ + MJ                                                  
          DSCALE(LDSCAL) = HI / ( DABS(ZVAL(IZ)) + 1.D0 )               
          LDSCAL = LDSCAL + 1                                           
   20     CONTINUE                                                      
   30   CONTINUE                                                        
   40 CONTINUE                                                          
C                                                                       
C...  compute scaling for parameter components                          
C                                                                       
      IF ( NPAR .EQ. 0 )                            RETURN              
      DO 50 IPAR = 1, NPAR                                              
        SCALE(LSCALE) = 1.D0 / ( DABS(Z(NZ+IPAR)) + 1.D0 )              
        LSCALE = LSCALE + 1                                             
   50 CONTINUE                                                          
      RETURN                                                            
      END                                                               
C---------------------------------------------------------------------- 
C                p a r t  2  of COLCON and COLPAR                       
C          mesh selection, error estimation, (and related               
C          constant assignment) routines -- see [3], [4]                
C---------------------------------------------------------------------- 
C                                                                       
      SUBROUTINE NEWMSH (MODE, XI, XIOLD, Z, DMZ, VALSTR,               
     1                   SLOPE, ACCUM, NFXPNT, FIXPNT)                  
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose                                                             
C            select a mesh on which a collocation solution is to be     
C            determined                                                 
C                                                                       
C                           there are 5 possible modes of action:       
C            mode = 5,4,3 - deal mainly with definition of an initial   
C                           mesh for the current boundary value problem 
C                 = 2,1   - deal with definition of a new mesh, either  
C                           by simple mesh halving or by mesh selection 
C            more specifically, for                                     
C            mode = 5  an initial (generally nonuniform) mesh is        
C                      defined by the user and no mesh selection is to  
C                      be performed                                     
C                 = 4  an initial (generally nonuniform) mesh is        
C                      defined by the user                              
C                 = 3  a simple uniform mesh (except possibly for some  
C                      fixed points) is defined; n= no. of subintervals 
C                 = 1  the automatic mesh selection procedure is used   
C                      (see [3] for details)                            
C                 = 2  a simple mesh halving is performed               
C                                                                       
C********************************************************************** 
C                                                                       
C   variables                                                           
C                                                                       
C            n      = number of mesh subintervals                       
C            nold   = number of subintervals for former mesh            
C            xi     - mesh point array                                  
C            xiold  - former mesh point array                           
C            mshlmt - maximum no. of mesh selections which are permitted
C                     for a given n before mesh halving                 
C            mshnum - no. of mesh selections which have actually been   
C                     performed for the given n                         
C            mshalt - no. of consecutive times ( plus 1 ) the mesh      
C                     selection has alternately halved and doubled n.   
C                     if mshalt .ge. mshlmt then  contrl  requires      
C                     that the current mesh be halved.                  
C            mshflg = 1  the mesh is a halving of its former mesh       
C                       (so an error estimate has been calculated)      
C                   = 0  otherwise                                      
C            iguess - info(9) in subroutine colpar.  it is used         
C                     here only for mode=5 and 4, where                 
C                   = 2 the subroutine sets xi=xiold.  this is          
C                       used e.g. if continuation is being per-         
C                       formed, and a mesh for the old differen-        
C                       tial equation is being used                     
C                   = 3 same as for =2, except xi uses every other      
C                       point of xiold (so mesh xiold is mesh xi        
C                       halved)                                         
C                   = 4 xi has been defined by the user, and an old     
C                       mesh xiold is also available                    
C                       otherwise, xi has been defined by the user      
C                       and we set xiold=xi in this subroutine          
C            slope  - an approximate quantity to be equidistributed for 
C                     mesh selection (see [3]), viz,                    
C                             .                        (k+mj)           
C                     slope(i)=     max   (weight(l) *u      (xi(i)))   
C                               1.le.l.le.ntol         j                
C                                                                       
C                     where j=jtol(l)                                   
C            slphmx - maximum of slope(i)*(xiold(i+1)-xiold(i)) for     
C                     i = 1 ,..., nold.                                 
C            accum  - accum(i) is the integral of  slope  from  aleft   
C                     to  xiold(i).                                     
C            valstr - is assigned values needed in  errchk  for the     
C                     error estimate.                                   
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION D1(40), D2(40), SLOPE(1), ACCUM(1), VALSTR(1)           
      DIMENSION XI(1), XIOLD(1), Z(1), DMZ(1), FIXPNT(1), DUMMY(1)      
C                                                                       
      COMMON /COLOUT/ PRECIS, IOUT, IPRINT                              
      COMMON /COLAPR/ N, NOLD, NMAX, NZ, NZPAR, NDMZ                    
      COMMON /COLORD/ K, NCOMP, MSTAR, NPAR, MSTATE, KD, MMAX, M(20)    
      COMMON /COLMSH/ MSHFLG, MSHNUM, MSHLMT, MSHALT                    
      COMMON /COLNLN/ NONLIN, ITER, LIMIT, ICARE, IGUESS, IREAD         
      COMMON /COLSID/ ZETA(40), ALEFT, ARIGHT, MSIDE, IZETA, IDUM       
      COMMON /COLBAS/ B(28), ACOL(28,7), ASAVE(28,4)                    
      COMMON /COLEST/ WGTMSH(40), WGTERR(40), TOL(40),                  
     1                ROOT(40), JTOL(40), LTOL(40), NTOL                
C                                                                       
      NFXP1 = NFXPNT + 1                                                
      GO TO (180, 100, 50, 20, 10), MODE                                
C                                                                       
C...  mode=5   set mshlmt=1 so that no mesh selection is performed      
C                                                                       
   10 MSHLMT = 1                                                        
C                                                                       
C...  mode=4   the user-specified initial mesh is already in place.     
C                                                                       
   20 IF ( IGUESS .LT. 2 )                          GO TO 40            
C                                                                       
C...  iguess=2, 3 or 4.                                                 
C                                                                       
      NOLDP1 = NOLD + 1                                                 
      IF (IPRINT .LT. 1)  WRITE(IOUT,360) NOLD, (XIOLD(I), I=1,NOLDP1)  
      IF ( IGUESS .NE. 3 )                          GO TO 40            
C                                                                       
C...  if iread ( info(8) ) .ge. 1 and iguess ( info(9) ) .eq. 3         
C...  then the first mesh is every second point of the                  
C...  mesh in  xiold .                                                  
C                                                                       
      N = NOLD /2                                                       
      I = 0                                                             
      DO 30 J = 1, NOLD, 2                                              
           I = I + 1                                                    
   30 XI(I) = XIOLD(J)                                                  
   40 CONTINUE                                                          
      NP1 = N + 1                                                       
      XI(1) = ALEFT                                                     
      XI(NP1) = ARIGHT                                                  
      GO TO 320                                                         
C                                                                       
C...  mode=3   generate a (piecewise) uniform mesh. if there are        
C...  fixed points then ensure that the n being used is large enough.   
C                                                                       
   50 IF ( N .LT. NFXP1 )  N = NFXP1                                    
      NP1 = N + 1                                                       
      XI(1) = ALEFT                                                     
      ILEFT = 1                                                         
      XLEFT = ALEFT                                                     
C                                                                       
C...  loop over the subregions between fixed points.                    
C                                                                       
      DO 90 J = 1, NFXP1                                                
           XRIGHT = ARIGHT                                              
           IRIGHT = NP1                                                 
           IF ( J .EQ. NFXP1 )                      GO TO 60            
           XRIGHT = FIXPNT(J)                                           
C                                                                       
C...       determine where the j-th fixed point should fall in the      
C...       new mesh - this is xi(iright) and the (j-1)st fixed          
C...       point is in xi(ileft)                                        
C                                                                       
           NMIN = (XRIGHT-ALEFT) / (ARIGHT-ALEFT) * DFLOAT(N) + 1.5D0   
           IF (NMIN .GT. N-NFXPNT+J)  NMIN = N - NFXPNT + J             
           IRIGHT = MAX0 (ILEFT+1, NMIN)                                
   60      XI(IRIGHT) = XRIGHT                                          
C                                                                       
C...       generate equally spaced points between the j-1st and the     
C...       j-th fixed points.                                           
C                                                                       
           NREGN = IRIGHT - ILEFT - 1                                   
           IF ( NREGN .EQ. 0 )                      GO TO 80            
           DX = (XRIGHT - XLEFT) / DFLOAT(NREGN+1)                      
           DO 70 I = 1, NREGN                                           
   70      XI(ILEFT+I) = XLEFT  +  DFLOAT(I) * DX                       
   80      ILEFT = IRIGHT                                               
           XLEFT = XRIGHT                                               
   90 CONTINUE                                                          
      GO TO 320                                                         
C                                                                       
C...  mode=2  halve the current mesh (i.e. double its size)             
C                                                                       
  100 N2 = 2 * N                                                        
C                                                                       
C...  check that n does not exceed storage limitations                  
C                                                                       
      IF ( N2 .LE. NMAX )                           GO TO 120           
C                                                                       
C...  if possible, try with n=nmax. redistribute first.                 
C                                                                       
      IF ( MODE .EQ. 2 )                            GO TO 110           
      N = NMAX / 2                                                      
      GO TO 220                                                         
  110 IF ( IPRINT .LT. 1 )  WRITE(IOUT,370)                             
      N = N2                                                            
      RETURN                                                            
C                                                                       
C...  calculate the old approximate solution values at                  
C...  points to be used in  errchk  for error estimates.                
C...  if  mshflg  =1 an error estimate was obtained for                 
C...  for the old approximation so half the needed values               
C...  will already be in  valstr .                                      
C                                                                       
  120 IF ( MSHFLG .EQ. 0 )                          GO TO 140           
C                                                                       
C...  save in  valstr  the values of the old solution                   
C...  at the relative positions 1/6 and 5/6 in each subinterval.        
C                                                                       
      KSTORE = 1                                                        
      DO 130 I = 1, NOLD                                                
          HD6 = (XIOLD(I+1) - XIOLD(I)) / 6.D0                          
          X = XIOLD(I) + HD6                                            
          CALL APPROX (I, X, VALSTR(KSTORE), ASAVE(1,1), DUMMY, XIOLD,  
     1         NOLD, Z, DMZ, K, NCOMP, MMAX, M, MSTAR, 4, DUMMY, 0)     
          X = X + 4.D0 * HD6                                            
          KSTORE = KSTORE  +  3 * MSTAR                                 
          CALL APPROX (I, X, VALSTR(KSTORE), ASAVE(1,4), DUMMY, XIOLD,  
     1         NOLD, Z, DMZ, K, NCOMP, MMAX, M, MSTAR, 4, DUMMY, 0)     
          KSTORE = KSTORE  +  MSTAR                                     
  130 CONTINUE                                                          
      GO TO 160                                                         
C                                                                       
C...  save in  valstr  the values of the old solution                   
C...  at the relative positions 1/6, 2/6, 4/6 and 5/6 in                
C...  each subinterval.                                                 
C                                                                       
  140 KSTORE = 1                                                        
      DO 150 I = 1, N                                                   
         X = XI(I)                                                      
         HD6 = (XI(I+1) - XI(I)) / 6.D0                                 
         DO 150 J = 1, 4                                                
           X = X + HD6                                                  
           IF ( J.EQ.3 )  X = X + HD6                                   
           CALL APPROX (I, X, VALSTR(KSTORE), ASAVE(1,J), DUMMY, XIOLD, 
     1          NOLD, Z, DMZ, K, NCOMP, MMAX, M, MSTAR, 4, DUMMY, 0)    
           KSTORE = KSTORE  +  MSTAR                                    
  150 CONTINUE                                                          
  160 MSHFLG = 0                                                        
      MSHNUM = 1                                                        
      MODE = 2                                                          
C                                                                       
C...  generate the halved mesh.                                         
C                                                                       
      J = 2                                                             
      DO 170 I = 1, N                                                   
           XI(J) = (XIOLD(I) + XIOLD(I+1)) / 2.D0                       
           XI(J+1) = XIOLD(I+1)                                         
  170 J = J + 2                                                         
      N = N2                                                            
      GO TO 320                                                         
C                                                                       
C...  mode=1  we do mesh selection if it is deemed worthwhile           
C                                                                       
  180 IF ( NOLD .EQ. 1 )                            GO TO 100           
      IF ( NOLD .LE. 2*NFXPNT )                     GO TO 100           
C                                                                       
C...  the first interval has to be treated separately from the          
C...  other intervals (generally the solution on the (i-1)st and ith    
C...  intervals will be used to approximate the needed derivative, but  
C...  here the 1st and second intervals are used.)                      
C                                                                       
      I = 1                                                             
      HIOLD = XIOLD(2) - XIOLD(1)                                       
      CALL HORDER ( 1, D1, HIOLD, DMZ )                                 
      HIOLD = XIOLD(3) - XIOLD(2)                                       
      CALL HORDER ( 2, D2, HIOLD, DMZ )                                 
      ACCUM(1) = 0.D0                                                   
      SLOPE(1) = 0.D0                                                   
      ONEOVH = 4.D0 / ( XIOLD(3) - XIOLD(1) )                           
      DO 190 J = 1, NTOL                                                
           JJ = JTOL(J)                                                 
           JZ = LTOL(J)                                                 
           JZM = JZ + MSTAR                                             
  190 SLOPE(1) = DMAX1(SLOPE(1),(DABS(D2(JJ)-D1(JJ))*WGTMSH(J)*         
     1  ONEOVH / (2.D0+DABS(Z(JZ))+DABS(Z(JZM)))) **ROOT(J))            
      SLPHMX = SLOPE(1) * (XIOLD(2) - XIOLD(1))                         
      ACCUM(2) = SLPHMX                                                 
      IFLIP = 1                                                         
C                                                                       
C...  go through the remaining intervals generating  slope              
C...  and  accum .                                                      
C                                                                       
      DO 210 I = 2, NOLD                                                
           HIOLD = XIOLD(I+1) - XIOLD(I)                                
           IF ( IFLIP .EQ. -1 )  CALL HORDER ( I, D1, HIOLD, DMZ )      
           IF ( IFLIP .EQ. 1 )   CALL HORDER ( I, D2, HIOLD, DMZ )      
           ONEOVH = 4.D0 / ( XIOLD(I+1) - XIOLD(I-1) )                  
           SLOPE(I) = 0.D0                                              
C                                                                       
C...       evaluate function to be equidistributed                      
C                                                                       
           DO 200 J = 1, NTOL                                           
             JJ = JTOL(J)                                               
             JZM = LTOL(J)  +  I * MSTAR                                
             JZ = JZM - MSTAR                                           
  200      SLOPE(I) = DMAX1(SLOPE(I),(DABS(D2(JJ)-D1(JJ))*WGTMSH(J)*    
     1      ONEOVH / (2.D0+DABS(Z(JZ))+DABS(Z(JZM)))) **ROOT(J))        
C                                                                       
C...       accumulate approximate integral of function to be            
C...       equidistributed                                              
C                                                                       
           TEMP = SLOPE(I) * (XIOLD(I+1)-XIOLD(I))                      
           SLPHMX = DMAX1(SLPHMX,TEMP)                                  
           ACCUM(I+1) = ACCUM(I) + TEMP                                 
  210 IFLIP = - IFLIP                                                   
      AVRG = ACCUM(NOLD+1) / DFLOAT(NOLD)                               
      DEGEQU = AVRG / DMAX1(SLPHMX,PRECIS)                              
C                                                                       
C...  naccum=expected n to achieve .1x user requested tolerances        
C                                                                       
      NACCUM = 1.25D0 * ACCUM(NOLD+1) + 1.D0                            
      NACCUM = MAX0( NACCUM, 2 )                                        
      IF ( IPRINT .LT. 0 )  WRITE(IOUT,350) DEGEQU, NACCUM              
C                                                                       
C...  decide if mesh selection is worthwhile (otherwise, halve)         
C                                                                       
      IF ( AVRG .LT. PRECIS )                       GO TO 100           
      IF ( DEGEQU .GE. .5D0 .AND.                                       
     1                NACCUM .GE. NOLD )            GO TO 100           
C                                                                       
C...  nmx assures mesh has at least half as many subintervals as the    
C...  previous mesh                                                     
C                                                                       
      NMX = MAX0 ( NOLD+1, NACCUM ) / 2                                 
C                                                                       
C...  this assures that halving will be possible later (for error est)  
C                                                                       
      NMAX2 = NMAX / 2                                                  
C                                                                       
C...  the mesh is at most halved                                        
C                                                                       
      N = MIN0 ( NMAX2, NOLD, NMX )                                     
  220 NOLDP1 = NOLD + 1                                                 
      IF ( N .LT. NFXP1 )  N = NFXP1                                    
      MSHNUM = MSHNUM + 1                                               
C                                                                       
C...  if the new mesh is smaller than the old mesh set mshnum           
C...  so that the next call to  newmsh  will produce a halved           
C...  mesh. if n .eq. nold / 2 increment mshalt so there can not        
C...  be an infinite loop alternating between n and n/2 points.         
C                                                                       
      IF ( N .LT. NOLD )  MSHNUM = MSHLMT                               
      IF ( N .GT. NOLD/2 )  MSHALT = 1                                  
      IF ( N .EQ. NOLD/2 )  MSHALT = MSHALT + 1                         
      MSHFLG = 0                                                        
C                                                                       
C...  having decided to generate a new mesh with n subintervals we now  
C...  do so, taking into account that the nfxpnt points in the array    
C...  fixpnt must be included in the new mesh.                          
C                                                                       
      IN = 1                                                            
      ACCL = 0.D0                                                       
      LOLD = 2                                                          
      XI(1) = ALEFT                                                     
      XI(N+1) = ARIGHT                                                  
      DO 310 I = 1, NFXP1                                               
           IF ( I .EQ. NFXP1 )                      GO TO 250           
           DO 230 J = LOLD, NOLDP1                                      
             LNEW = J                                                   
             IF ( FIXPNT(I) .LE. XIOLD(J) )         GO TO 240           
  230      CONTINUE                                                     
  240      CONTINUE                                                     
           ACCR = ACCUM(LNEW) + (FIXPNT(I)-XIOLD(LNEW))*SLOPE(LNEW-1)   
           NREGN = (ACCR-ACCL) / ACCUM(NOLDP1) * DFLOAT(N) - .5D0       
           NREGN = MIN0(NREGN, N - IN - NFXP1 + I)                      
           XI(IN + NREGN + 1) = FIXPNT(I)                               
           GO TO 260                                                    
  250      ACCR = ACCUM(NOLDP1)                                         
           LNEW = NOLDP1                                                
           NREGN = N - IN                                               
  260      IF ( NREGN .EQ. 0 )                      GO TO 300           
           TEMP = ACCL                                                  
           TSUM = (ACCR - ACCL) / DFLOAT(NREGN+1)                       
           DO 290 J = 1, NREGN                                          
             IN = IN + 1                                                
             TEMP = TEMP + TSUM                                         
             DO 270 L = LOLD, LNEW                                      
               LCARRY = L                                               
               IF ( TEMP .LE. ACCUM(L) )            GO TO 280           
  270        CONTINUE                                                   
  280        CONTINUE                                                   
             LOLD = LCARRY                                              
  290      XI(IN) = XIOLD(LOLD-1) + (TEMP - ACCUM(LOLD-1)) /            
     1     SLOPE(LOLD-1)                                                
  300      IN = IN + 1                                                  
           ACCL = ACCR                                                  
           LOLD = LNEW                                                  
  310 CONTINUE                                                          
      MODE = 1                                                          
  320 CONTINUE                                                          
      NP1 = N + 1                                                       
      IF ( IPRINT .LT. 1 )  WRITE(IOUT,340) N, (XI(I),I=1,NP1)          
      NZ   = MSTAR * (N + 1)                                            
      NZPAR   = NZ + NPAR                                               
      NDMZ = KD * N                                                     
      RETURN                                                            
C----------------------------------------------------------------       
  340 FORMAT(/17H THE NEW MESH (OF,I5,16H SUBINTERVALS), ,100(/8F12.6)) 
  350 FORMAT(/21H MESH SELECTION INFO,/30H DEGREE OF EQUIDISTRIBUTION = 
     1       , F8.5, 28H PREDICTION FOR REQUIRED N = , I8)              
  360 FORMAT(/20H THE FORMER MESH (OF,I5,15H SUBINTERVALS),,            
     1       100(/8F12.6))                                              
  370 FORMAT (/23H  EXPECTED N TOO LARGE  )                             
      END                                                               
      SUBROUTINE CONSTS (K, RHO, COEF)                                  
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose                                                             
C            assign (once) values to various array constants.           
C                                                                       
C   arrays assigned during compilation:                                 
C     cnsts1 - weights for extrapolation error estimate                 
C     cnsts2 - weights for mesh selection                               
C              (the above weights come from the theoretical form for    
C              the collocation error -- see [3])                        
C                                                                       
C   arrays assigned during execution:                                   
C     wgterr - the particular values of cnsts1 used for current run     
C              (depending on k, m)                                      
C     wgtmsh - gotten from the values of cnsts2 which in turn are       
C              the constants in the theoretical expression for the      
C              errors. the quantities in wgtmsh are 10x the values      
C              in cnsts2 so that the mesh selection algorithm           
C              is aiming for errors .1x as large as the user            
C              requested tolerances.                                    
C     jtol   - components of differential system to which tolerances    
C              refer (viz, if ltol(i) refers to a derivative of u(j),   
C              then jtol(i)=j)                                          
C     root   - reciprocals of expected rates of convergence of compo-   
C              nents of z(j) for which tolerances are specified         
C     rho    - the k collocation points on (0,1)                        
C     coef   -                                                          
C     acol  -  the runge-kutta coefficients values at collocation       
C              points                                                   
C                                                                       
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION RHO(7), COEF(K,1), CNSTS1(28), CNSTS2(28), DUMMY(1)     
C                                                                       
      COMMON /COLORD/ KDUM, NCOMP, MSTAR, NPAR, MSTATE, KD, MMAX, M(20) 
      COMMON /COLBAS/ B(28), ACOL(28,7), ASAVE(28,4)                    
      COMMON /COLEST/ WGTMSH(40), WGTERR(40), TOL(40),                  
     1                ROOT(40), JTOL(40), LTOL(40), NTOL                
C                                                                       
      DATA CNSTS1 /    .25D0,     .625D-1,  7.2169D-2, 1.8342D-2,       
     1     1.9065D-2, 5.8190D-2, 5.4658D-3, 5.3370D-3, 1.8890D-2,       
     2     2.7792D-2, 1.6095D-3, 1.4964D-3, 7.5938D-3, 5.7573D-3,       
     3     1.8342D-2, 4.673D-3,  4.150D-4,  1.919D-3,  1.468D-3,        
     4     6.371D-3,  4.610D-3,  1.342D-4,  1.138D-4,  4.889D-4,        
     5     4.177D-4,  1.374D-3,  1.654D-3,  2.863D-3  /                 
      DATA CNSTS2 /   1.25D-1,   2.604D-3,  8.019D-3,  2.170D-5,        
     1     7.453D-5,  5.208D-4,  9.689D-8,  3.689D-7,  3.100D-6,        
     2     2.451D-5,  2.691D-10, 1.120D-9,  1.076D-8,  9.405D-8,        
     3     1.033D-6,  5.097D-13, 2.290D-12, 2.446D-11, 2.331D-10,       
     4     2.936D-9,  3.593D-8,  7.001D-16, 3.363D-15, 3.921D-14,       
     5     4.028D-13, 5.646D-12, 7.531D-11, 1.129D-9  /                 
C                                                                       
C...  assign weights for error estimate                                 
C                                                                       
      KOFF = K * ( K + 1 ) / 2                                          
      IZ = 1                                                            
      DO 10 J = 1, NCOMP                                                
           MJ = M(J)                                                    
           DO 10 L = 1, MJ                                              
             WGTERR(IZ) = CNSTS1(KOFF - MJ + L)                         
             IZ = IZ + 1                                                
   10 CONTINUE                                                          
C                                                                       
C...  assign array values for mesh selection: wgtmsh, jtol, and root    
C                                                                       
      JCOMP = 1                                                         
      MTOT = M(1)                                                       
      DO 40 I = 1, NTOL                                                 
           LTOLI = LTOL(I)                                              
   20      CONTINUE                                                     
           IF ( LTOLI .LE. MTOT )                   GO TO 30            
           JCOMP = JCOMP + 1                                            
           MTOT = MTOT + M(JCOMP)                                       
           GO TO 20                                                     
   30      CONTINUE                                                     
           JTOL(I) = JCOMP                                              
           WGTMSH(I) = 1.D1 * CNSTS2(KOFF+LTOLI-MTOT) / TOL(I)          
           ROOT(I) = 1.D0 / DFLOAT(K+MTOT-LTOLI+1)                      
   40 CONTINUE                                                          
C                                                                       
C...  specify collocation points                                        
C                                                                       
      GO TO (50,60,70,80,90,100,110), K                                 
   50 RHO(1) = 0.D0                                                     
      GO TO 120                                                         
   60 RHO(2) = .57735026918962576451D0                                  
      RHO(1) = - RHO(2)                                                 
      GO TO 120                                                         
   70 RHO(3) = .77459666924148337704D0                                  
      RHO(2) = .0D0                                                     
      RHO(1) = - RHO(3)                                                 
      GO TO 120                                                         
   80 RHO(4) = .86113631159405257523D0                                  
      RHO(3) = .33998104358485626480D0                                  
      RHO(2) = - RHO(3)                                                 
      RHO(1) = - RHO(4)                                                 
      GO TO 120                                                         
   90 RHO(5) = .90617984593866399280D0                                  
      RHO(4) = .53846931010568309104D0                                  
      RHO(3) = .0D0                                                     
      RHO(2) = - RHO(4)                                                 
      RHO(1) = - RHO(5)                                                 
      GO TO 120                                                         
  100 RHO(6) = .93246951420315202781D0                                  
      RHO(5) = .66120938646626451366D0                                  
      RHO(4) = .23861918608319690863D0                                  
      RHO(3) = -RHO(4)                                                  
      RHO(2) = -RHO(5)                                                  
      RHO(1) = -RHO(6)                                                  
      GO TO 120                                                         
  110 RHO(7) = .949107991234275852452D0                                 
      RHO(6) = .74153118559939443986D0                                  
      RHO(5) = .40584515137739716690D0                                  
      RHO(4) = 0.D0                                                     
      RHO(3) = -RHO(5)                                                  
      RHO(2) = -RHO(6)                                                  
      RHO(1) = -RHO(7)                                                  
  120 CONTINUE                                                          
C                                                                       
C...  map (-1,1) to (0,1) by  t = .5 * (1. + x)                         
C                                                                       
      DO 130 J = 1, K                                                   
         RHO(J) = .5D0 * (1.D0 + RHO(J))                                
  130 CONTINUE                                                          
C                                                                       
C...  now find runge-kutta coeffitients b, acol and asave               
C...  the values of asave are to be used in  newmsh  and errchk .       
C                                                                       
      DO 140 J = 1, K                                                   
         DO 135 I = 1, K                                                
  135      COEF(I,J) = 0.D0                                             
         COEF(J,J) = 1.D0                                               
         CALL VMONDE (RHO, COEF(1,J), K)                                
  140 CONTINUE                                                          
      CALL RKBAS ( 1.D0, COEF, K, MMAX, B, DUMMY, 0)                    
      DO 150 I = 1, K                                                   
         CALL RKBAS ( RHO(I), COEF, K, MMAX, ACOL(1,I), DUMMY, 0)       
  150 CONTINUE                                                          
      CALL RKBAS ( 1.D0/6.D0, COEF, K, MMAX, ASAVE(1,1), DUMMY, 0)      
      CALL RKBAS ( 1.D0/3.D0, COEF, K, MMAX, ASAVE(1,2), DUMMY, 0)      
      CALL RKBAS ( 2.D0/3.D0, COEF, K, MMAX, ASAVE(1,3), DUMMY, 0)      
      CALL RKBAS ( 5.D0/6.D0, COEF, K, MMAX, ASAVE(1,4), DUMMY, 0)      
      RETURN                                                            
      END                                                               
      SUBROUTINE ERRCHK (XI, Z, DMZ, VALSTR, IFIN)                      
C                                                                       
C********************************************************************** 
C                                                                       
C      purpose                                                          
C               determine the error estimates and test to see if the    
C               error tolerances are satisfied.                         
C                                                                       
C      variables                                                        
C        xi     - current mesh points                                   
C        valstr - values of the previous solution which are needed      
C                 for the extrapolation- like error estimate.           
C        wgterr - weights used in the extrapolation-like error          
C                 estimate. the array values are assigned in            
C                 subroutine  consts.                                   
C        errest - storage for error estimates                           
C        err    - temporary storage used for error estimates            
C        z      - approximate solution on mesh xi                       
C        ifin   - a 0-1 variable. on return it indicates whether        
C                 the error tolerances were satisfied                   
C        mshflg - is set by errchk to indicate to newmsh whether        
C                 any values of the current solution are stored in      
C                 the array valstr. (0 for no, 1 for yes)               
C                                                                       
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION ERR(40), ERREST(40), DUMMY(1)                           
      DIMENSION XI(1), Z(1), DMZ(1), VALSTR(1)                          
C                                                                       
      COMMON /COLOUT/ PRECIS, IOUT, IPRINT                              
      COMMON /COLAPR/ N, NOLD, NMAX, NZ, NZPAR, NDMZ                    
      COMMON /COLORD/ K, NCOMP, MSTAR, NPAR, MSTATE, KD, MMAX, M(20)    
      COMMON /COLMSH/ MSHFLG, MSHNUM, MSHLMT, MSHALT                    
      COMMON /COLBAS/ B(28), ACOL(28,7), ASAVE(28,4)                    
      COMMON /COLEST/ WGTMSH(40), WGTERR(40), TOL(40),                  
     1                ROOT(40), JTOL(40), LTOL(40), NTOL                
C                                                                       
C...  error estimates are to be generated and tested                    
C...  to see if the tolerance requirements are satisfied.               
C                                                                       
      IFIN = 1                                                          
      MSHFLG = 1                                                        
      DO 10 J = 1, MSTAR                                                
   10   ERREST(J) = 0.D0                                                
      DO 60 IBACK = 1, N                                                
           I = N + 1 - IBACK                                            
C                                                                       
C...       the error estimates are obtained by combining values of      
C...       the numerical solutions for two meshes.                      
C...       for each value of iback we will consider the two             
C...       approximations at 2 points in each of                        
C...       the new subintervals.  we work backwards through             
C...       the subinterval so that new values can be stored             
C...       in valstr in case they prove to be needed later              
C...       for an error estimate. the routine  newmsh                   
C...       filled in the needed values of the old solution              
C...       in valstr.                                                   
C                                                                       
           KNEW = ( 4 * (I-1) + 2 ) * MSTAR + 1                         
           KSTORE = ( 2 * (I-1) + 1 ) * MSTAR + 1                       
           X = XI(I) +  (XI(I+1)-XI(I)) * 2.D0 / 3.D0                   
           CALL APPROX (I, X, VALSTR(KNEW), ASAVE(1,3), DUMMY, XI,      
     1            N, Z, DMZ, K, NCOMP, MMAX, M, MSTAR, 4, DUMMY, 0)     
           DO 20 L = 1,MSTAR                                            
             ERR(L) = WGTERR(L) * DABS(VALSTR(KNEW) -                   
     1       VALSTR(KSTORE))                                            
             KNEW = KNEW + 1                                            
             KSTORE = KSTORE + 1                                        
   20      CONTINUE                                                     
           KNEW = ( 4 * (I-1) + 1 ) * MSTAR + 1                         
           KSTORE = 2 * (I-1) * MSTAR + 1                               
           X = XI(I) +  (XI(I+1)-XI(I)) / 3.D0                          
           CALL APPROX (I, X, VALSTR(KNEW), ASAVE(1,2), DUMMY, XI,      
     1            N, Z, DMZ, K, NCOMP, MMAX, M, MSTAR, 4, DUMMY, 0)     
           DO 30 L = 1,MSTAR                                            
             ERR(L) = ERR(L) + WGTERR(L) * DABS(VALSTR(KNEW) -          
     1       VALSTR(KSTORE))                                            
             KNEW = KNEW + 1                                            
             KSTORE = KSTORE + 1                                        
   30      CONTINUE                                                     
C                                                                       
C...       find component-wise maximum error estimate                   
C                                                                       
           DO 40 L = 1,MSTAR                                            
             ERREST(L) = DMAX1(ERREST(L),ERR(L))                        
   40      CONTINUE                                                     
C                                                                       
C...       test whether the tolerance requirements are satisfied        
C...       in the i-th interval.                                        
C                                                                       
           IF ( IFIN .EQ. 0 )                       GO TO 60            
           DO 50 J = 1, NTOL                                            
             LTOLJ = LTOL(J)                                            
             LTJZM = LTOLJ  +  I * MSTAR                                
             LTJZ = LTJZM - MSTAR                                       
           IF ( ERR(LTOLJ) .GT. TOL(J) *                                
     1        (.5D0*(DABS(Z(LTJZ))+DABS(Z(LTJZM)))+1.D0) )  IFIN = 0    
   50      CONTINUE                                                     
   60 CONTINUE                                                          
      IF ( IPRINT .EQ. 1 )                          RETURN              
      WRITE(IOUT,130)                                                   
      LJ = 1                                                            
      DO 70 J = 1,NCOMP                                                 
           MJ = LJ - 1 + M(J)                                           
           WRITE(IOUT,120) J, (ERREST(L), L= LJ, MJ)                    
           LJ = MJ + 1                                                  
   70 CONTINUE                                                          
      RETURN                                                            
C--------------------------------------------------------------         
  120 FORMAT (3H U(, I2, 3H) -,4D12.4)                                  
  130 FORMAT (/26H THE ESTIMATED ERRORS ARE,)                           
      END                                                               
C---------------------------------------------------------------------  
C          p a r t  3  of COLCON and COLPAR                             
C          collocation system setup routines                            
C---------------------------------------------------------------------  
C                                                                       
      SUBROUTINE LSYSLV (MSING, XI, XIOLD, Z, DMZ, DELZ, DELDMZ,        
     1           G, W, V, RHS, DMZO, INTEGS, IPVTG, IPVTW, RNORM,       
     2           MODE, FSUB, DFSUB, GSUB, DGSUB, GUESS )                
C*********************************************************************  
C                                                                       
C   purpose                                                             
C         this routine controls the set up and solution of a linear     
C      system of collocation equations.                                 
C         the matrix  g  is cast into an almost block diagonal          
C      form by an appropriate ordering of the columns and solved        
C      using the package of de boor-weiss [5]. the matrix is composed   
C      of n blocks. the i-th block has the size                         
C                  integs(1,i) * integs(2,i).                           
C      it contains in its last rows the linearized collocation          
C      equations, condensed as described in [2],                        
C      and the linearized side conditions corresponding to              
C      the i-th subinterval.  integs(3,i)  steps of gaussian            
C      elimination are applied to it to achieve a  partial plu          
C      decomposition.  the right hand side vector is put into  rhs      
C      and the solution vector is returned in  delz and deldmz.         
C                                                                       
C         lsyslv operates according to one of 5 modes:                  
C      mode = 0 - set up the collocation matrices  v , w , g            
C                 and the right hand side  rhs ,  and solve.            
C                 (for linear problems only.)                           
C      mode = 1 - set up the collocation matrices  v , w , g            
C                 and the right hand sides  rhs  and  dmzo ,            
C                 and solve. also set up  integs .                      
C                 (first iteration of nonlinear problems only).         
C      mode = 2 - set up  rhs  only and compute its norm.               
C      mode = 3 - set up  v, w, g  only and solve system.               
C      mode = 4 - perform forward and backward substitution only        
C                 (do not set up the matrices nor form the rhs).        
C      mode = 5 - set up  v, w, g  only and decompose.                  
C                                                                       
C   variables                                                           
C                                                                       
C      ig,izeta  - pointers to g,zeta respectively                      
C                       (necessary to keep track of blocks of g         
C                       during matrix manipulations)                    
C      idmz,irhs,iv,iw - pointers to  rhs,v,w respectively              
C      df    - partial derivatives of f from dfsub                      
C      rnorm - euclidean norm of rhs                                    
C      lside - number of side conditions in current and previous blocks 
C      iguess = 1 when current soln is user specified via  guess        
C             = 0 otherwise                                             
C                                                                       
C*********************************************************************  
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION  Z(1), DMZ(1), DELZ(1), DELDMZ(1), XI(1), XIOLD(1)      
      DIMENSION  G(1), W(1), V(1),  RHS(1) , DMZO(1), DUMMY(1)          
      DIMENSION  INTEGS(3,1), IPVTG(1), IPVTW(1)                        
      DIMENSION  ZVAL(40), F(40), DGZ(40), DMVAL(20), DF(800), AT(28)   
C                                                                       
      COMMON /COLOUT/ PRECIS, IOUT, IPRINT                              
      COMMON /COLLOC/ RHO(7), COEF(49)                                  
      COMMON /COLAPR/ N, NOLD, NMAX, NZ, NZPAR, NDMZ                    
      COMMON /COLORD/ K, NCOMP, MSTAR, NPAR, MSTATE, KD, MMAX, M(20)    
      COMMON /COLSID/ ZETA(40), ALEFT, ARIGHT, MSIDE, IZETA, IZSAVE     
      COMMON /COLNLN/ NONLIN, ITER, LIMIT, ICARE, IGUESS, IREAD         
      COMMON /COLBAS/ B(28), ACOL(28,7), ASAVE(28,4)                    
      COMMON /COLPTH/ IPATH, ICON, LPAR                                 
C                                                                       
      EXTERNAL DFSUB, DGSUB                                             
C                                                                       
      NEQ = N * MSTAR + MSIDE                                           
      IF ( NPAR .EQ. 0 )                            GO TO 4             
      NZH = NZ                                                          
      IF ( MODE .EQ. 1 )  NZH = ( NOLD + 1 ) * MSTAR                    
      JCON = MSTAR + ICON                                               
      IF ( ICON .GT. 0 )  ZCON = Z(NZH+ICON)                            
      IF ( IGUESS .EQ. 1 )                          GO TO 4             
      DO 2 IPAR = 1, NPAR                                               
        ZVAL(MSTAR+IPAR) = Z(NZH+IPAR)                                  
    2 CONTINUE                                                          
    4 CONTINUE                                                          
C                                                                       
      M1 = MODE + 1                                                     
      GO TO (10, 30, 30, 30, 310, 30), M1                               
C                                                                       
C...  linear problem initialization                                     
C                                                                       
   10 DO 20 I = 1, MSTAR                                                
   20 ZVAL(I) = 0.D0                                                    
C                                                                       
C...  initialization                                                    
C                                                                       
   30 IDMZ = 1                                                          
      IDMZO = 1                                                         
      IRHS = 1                                                          
      IG = 1                                                            
      IW = 1                                                            
      IV = 1                                                            
      IPV = MSTAR * KD + 1                                              
      IZETA = 1                                                         
      LSIDE = 0                                                         
      IOLD = 1                                                          
      NCOL = 2 * MSTAR                                                  
C#    RNORM = 0.D0                                                      
      IF ( MODE .GT. 1 .AND. MODE .LT. 5 )          GO TO 80            
C                                                                       
C...  build integs (describing block structure of matrix)               
C                                                                       
      LPAR = 1                                                          
      DO 70 I = 1,N                                                     
           INTEGS(2,I) = NCOL                                           
           IF ( I .LT. N )                          GO TO 40            
           INTEGS(3,N) = MSTAR + MSIDE                                  
           INTEGS(2,N) = NCOL + NPAR                                    
           LSIDE = MSIDE                                                
           GO TO 60                                                     
   40      INTEGS(3,I) = MSTAR                                          
   50      IF( LSIDE .EQ. MSIDE )                   GO TO 60            
           IF ( ZETA(LSIDE+1) .GE. XI(I)+PRECIS )   GO TO 60            
           LSIDE = LSIDE + 1                                            
           GO TO 50                                                     
   60      NROW = MSTAR + LSIDE                                         
           INTEGS(1,I) = NROW                                           
   70 LPAR = LPAR + NROW * INTEGS(2,I)                                  
   80 CONTINUE                                                          
      IP = LPAR                                                         
      IF ( MODE .EQ. 2 )                            GO TO 90            
C                                                                       
C...  zero the matrices to be computed                                  
C                                                                       
      LW = KD * KD * N                                                  
      DO 84 L = 1, LW                                                   
   84   W(L) = 0.D0                                                     
C                                                                       
C...  the do loop 290 sets up the linear system of equations.           
C                                                                       
  90  CONTINUE                                                          
      DO 290 I=1, N                                                     
C                                                                       
C...       construct a block of  a  and a corresponding piece of  rhs.  
C                                                                       
           XMSH = XI(I)                                                 
           H = XI(I+1) - XI(I)                                          
           NROW = INTEGS(1,I)                                           
C                                                                       
C...       go thru the ncomp collocation equations and side conditions  
C...       in the i-th subinterval                                      
C                                                                       
  100      IF ( IZETA .GT. MSIDE )                  GO TO 140           
           IF ( ZETA(IZETA) .GT. XMSH + PRECIS )    GO TO 140           
C                                                                       
C...       build equation for a side condition.                         
C                                                                       
           IF ( MODE .GT. 1 )                       GO TO 106           
           IF ( MODE .EQ. 0 )                       GO TO 110           
           IF ( IGUESS .NE. 1 )                     GO TO 102           
C                                                                       
C...       case where user provided current approximation               
C                                                                       
           CALL GUESS (XMSH, ZVAL, DMVAL)                               
           IF ( ICON .GT. 0 )  ZVAL(JCON) = ZCON                        
           GO TO 110                                                    
C                                                                       
C...       other nonlinear case                                         
C                                                                       
  102      CALL APPROX (IOLD, XMSH, ZVAL, AT, COEF, XIOLD, NOLD,        
     1          Z, DMZ, K, NCOMP, MMAX, M, MSTAR, 2, DUMMY, 0)          
           GO TO 110                                                    
  106      CALL APPROX (I, XMSH, ZVAL, AT, DUMMY, XI, N, Z, DMZ,        
     1                  K, NCOMP, MMAX, M, MSTAR, 1, DUMMY, 0)          
           IF ( MODE .EQ. 3 .OR. MODE .EQ. 5 )      GO TO 120           
C                                                                       
C...       find  rhs  boundary value.                                   
C                                                                       
  110      CALL GSUB (IZETA, ZVAL, GVAL)                                
           RHS(NDMZ+IZETA) = -GVAL                                      
C#         RNORM = RNORM + GVAL**2                                      
           IF ( MODE .EQ. 2 )                       GO TO 130           
C                                                                       
C...       build a row of  a  corresponding to a boundary point         
C                                                                       
  120      CALL GDERIV (G(IG), G(IP), NROW, IZETA, ZVAL, DGZ, 1, DGSUB) 
  130      IZETA = IZETA + 1                                            
           GO TO 100                                                    
C                                                                       
C...       assemble collocation equations                               
C                                                                       
  140      DO 220 J = 1, K                                              
             HRHO = H * RHO(J)                                          
             XCOL = XMSH + HRHO                                         
C                                                                       
C...         this value corresponds to a collocation (interior)         
C...         point. build the corresponding  ncomp  equations.          
C                                                                       
             IF ( MODE .GT. 1 )                     GO TO 190           
             IF ( MODE .EQ. 0 )                     GO TO 200           
             IF ( IGUESS .NE. 1 )                   GO TO 160           
C                                                                       
C...         use initial approximation provided by the user.            
C                                                                       
             CALL GUESS (XCOL, ZVAL, DMZO(IRHS) )                       
             IF ( ICON .GT. 0 )  ZVAL(JCON) = ZCON                      
             GO TO 170                                                  
C                                                                       
C...         find  rhs  values                                          
C                                                                       
  160        CALL APPROX (IOLD, XCOL, ZVAL, AT, COEF, XIOLD, NOLD,      
     1            Z, DMZ, K, NCOMP, MMAX, M, MSTAR, 2, DMZO(IRHS), 1)   
C                                                                       
  170        CALL FSUB (XCOL, ZVAL, F)                                  
             DO 180 JJ = 1, NCOMP                                       
               VALUE = DMZO(IRHS) - F(JJ)                               
               RHS(IRHS) = - VALUE                                      
C#             RNORM = RNORM + VALUE**2                                 
               IRHS = IRHS + 1                                          
  180        CONTINUE                                                   
             GO TO 210                                                  
C                                                                       
C...         evaluate former collocation solution                       
C                                                                       
  190        CALL APPROX (I, XCOL, ZVAL, ACOL(1,J), COEF, XI, N,        
     1            Z, DMZ, K, NCOMP, MMAX, M, MSTAR, 4, DUMMY, 0)        
             IF ( MODE .EQ. 3 .OR. MODE .EQ. 5 )    GO TO 210           
C                                                                       
C...         fill in  rhs  values (and accumulate its norm).            
C                                                                       
             CALL FSUB (XCOL, ZVAL, F)                                  
             DO 195 JJ = 1, NCOMP                                       
               VALUE = DMZ(IRHS) - F(JJ)                                
               RHS(IRHS) = - VALUE                                      
C#             RNORM = RNORM + VALUE**2                                 
               IRHS = IRHS + 1                                          
  195        CONTINUE                                                   
             GO TO 220                                                  
C                                                                       
C...         the linear case                                            
C                                                                       
  200        CALL FSUB (XCOL, ZVAL, RHS(IRHS))                          
             IRHS = IRHS + NCOMP                                        
C                                                                       
C...         fill in ncomp rows of  w and v                             
C                                                                       
  210        CALL VWBLOK (XCOL, HRHO, J, W(IW), V(IV), IPVTW(IDMZ),     
     1       KD, ZVAL, DF, ACOL(1,J), DMZO(IDMZO), NCOMP, DFSUB, MSING) 
             IF ( MSING .NE. 0 )                    RETURN              
  220      CONTINUE                                                     
C                                                                       
C...       build global bvp matrix  g                                   
C                                                                       
           IF ( MODE .EQ. 2 )                       GO TO 230           
            CALL GBLOCK (H, G(IG), NROW, IZETA, W(IW), V(IV), KD,       
     1                  DUMMY, DUMMY, IPVTW(IDMZ), 1 )                  
            IF ( NPAR .EQ. 0 )                      GO TO 230           
            IPV = IV + MSTAR * KD                                       
            IPH = IP                                                    
            DO 232 IPAR = 1, NPAR                                       
               CALL GBLOCK (H, G(IG), NROW, IZETA, W(IW), V(IV),        
     1                      KD, G(IPH), V(IPV), IPVTW(IDMZ), 2 )        
               IPV = IPV + KD                                           
               IPH = IPH + NEQ                                          
  232       CONTINUE                                                    
  230      IF ( I .LT. N )                          GO TO 280           
           IZSAVE = IZETA                                               
  240      IF ( IZETA .GT. MSIDE )                  GO TO 290           
C                                                                       
C...       build equation for a side condition.                         
C                                                                       
           IF ( MODE .GT. 1 )                       GO TO 246           
           IF ( MODE .EQ. 0 )                       GO TO 250           
           IF ( IGUESS .NE. 1 )                     GO TO 244           
C                                                                       
C...       case where user provided current approximation               
C                                                                       
           CALL GUESS (ARIGHT, ZVAL, DMVAL)                             
           IF ( ICON .GT. 0 )  ZVAL(JCON) = ZCON                        
           GO TO 250                                                    
C                                                                       
C...       other nonlinear case                                         
C                                                                       
  244      CALL APPROX (NOLD+1, ARIGHT, ZVAL, AT, COEF, XIOLD, NOLD,    
     1          Z, DMZ, K, NCOMP, MMAX, M, MSTAR, 1, DUMMY, 0)          
           GO TO 250                                                    
  246      CALL APPROX (N+1, ARIGHT, ZVAL, AT, COEF, XI, N,             
     1          Z, DMZ, K, NCOMP, MMAX, M, MSTAR, 1, DUMMY, 0)          
           IF ( MODE .EQ. 3 .OR. MODE .EQ. 5 )      GO TO 260           
C                                                                       
C...       find  rhs  boundary value.                                   
C                                                                       
  250      CALL GSUB (IZETA, ZVAL, GVAL)                                
           RHS(NDMZ+IZETA) = - GVAL                                     
C#         RNORM = RNORM + GVAL**2                                      
           IF ( MODE .EQ. 2 )                       GO TO 270           
C                                                                       
C...       build a row of  a  corresponding to a boundary point         
C                                                                       
  260      CALL GDERIV (G(IG), G(IP), NROW, IZETA+MSTAR, ZVAL,          
     1                 DGZ, 2, DGSUB)                                   
  270      IZETA = IZETA + 1                                            
           GO TO 240                                                    
C                                                                       
C...       update counters -- i-th block completed                      
C                                                                       
  280      IG = IG + NROW * NCOL                                        
           IV = IV + KD * MSTATE                                        
           IF ( NPAR .GT. 0 )  IP = IP + MSTAR                          
           IW = IW + KD * KD                                            
           IDMZ = IDMZ + KD                                             
           IF ( MODE .EQ. 1 )  IDMZO = IDMZO + KD                       
  290 CONTINUE                                                          
C                                                                       
C...       assembly process completed                                   
C                                                                       
      IF ( MODE .EQ. 0 .OR. MODE .EQ. 3                                 
     1                 .OR. MODE .EQ. 5 )           GO TO 300           
C#    RNORM = DSQRT(RNORM / DFLOAT(NEQ+NDMZ))                           
      L = NDMZ + MSIDE                                                  
      RNORM = DNRM2(L,RHS,1) / DSQRT(DFLOAT(NEQ+NDMZ))                  
      IF ( MODE .NE. 2 )                            GO TO 300           
      RETURN                                                            
C                                                                       
C...  solve the linear system.                                          
C                                                                       
C...  matrix decomposition                                              
C                                                                       
  300 CALL FCBLOK (G, G(LPAR), INTEGS, N, NEQ, NPAR, IPVTG, DF, MSING)  
C                                                                       
C...  check for singular matrix                                         
C                                                                       
      MSING = - MSING                                                   
      IF ( MSING .NE. 0 )                           RETURN              
      IF ( MODE .EQ. 5 )                            RETURN              
C                                                                       
C...  perform forward and backward substitution for mode=4 only.        
C                                                                       
  310 CONTINUE                                                          
      DO 311 L = 1, NDMZ                                                
        DELDMZ(L) = RHS(L)                                              
  311 CONTINUE                                                          
      IW = 1                                                            
      IZ = 1                                                            
      IDMZ = 1                                                          
      IZET = 1                                                          
      DO 320 I=1, N                                                     
         NROW = INTEGS(1,I)                                             
         IZETA = NROW + 1 - MSTAR                                       
         IF ( I .EQ. N ) IZETA = IZSAVE                                 
  322    IF ( IZET .EQ. IZETA )                     GO TO 324           
           DELZ(IZ-1+IZET) = RHS(NDMZ+IZET)                             
           IZET = IZET + 1                                              
         GO TO 322                                                      
  324    H = XI(I+1) - XI(I)                                            
         CALL GBLOCK (H, G(1), NROW, IZETA, W(IW), V(1), KD,            
     1                DELZ(IZ), DELDMZ(IDMZ), IPVTW(IDMZ), 2 )          
         IW = IW + KD * KD                                              
         IZ = IZ + MSTAR                                                
         IDMZ = IDMZ + KD                                               
         IF ( I .LT. N )                            GO TO 320           
  326    IF ( IZET .GT. MSIDE )                     GO TO 320           
           DELZ(IZ-1+IZET) = RHS(NDMZ+IZET)                             
           IZET = IZET + 1                                              
         GO TO 326                                                      
  320 CONTINUE                                                          
C                                                                       
C...  perform forward and backward substitution for mode=0,2, or 3.     
C                                                                       
      CALL SBBLOK (G, G(LPAR), INTEGS, N, NEQ, NPAR, IPVTG, DELZ)       
C                                                                       
C...  finaly find deldmz                                                
C                                                                       
      CALL DMZSOL (KD, MSTAR, N, V, DELZ, DELDMZ)                       
C                                                                       
C...  project solution approximation onto a new mesh                    
C                                                                       
      IF ( MODE .NE. 1 )                            RETURN              
      DO 321 L = 1, NDMZ                                                
        DMZ(L) = DMZO(L)                                                
  321 CONTINUE                                                          
      IW = 1                                                            
      IZ = 1                                                            
      IDMZ = 1                                                          
      IZET = 1                                                          
      DO 350 I=1, N                                                     
         NROW = INTEGS(1,I)                                             
         IZETA = NROW + 1 - MSTAR                                       
         IF ( I .EQ. N ) IZETA = IZSAVE                                 
  330    IF ( IZET .EQ. IZETA )                     GO TO 340           
           Z(IZ-1+IZET) = DGZ(IZET)                                     
           IZET = IZET + 1                                              
         GO TO 330                                                      
  340    H = XI(I+1) - XI(I)                                            
         CALL GBLOCK (H, G(1), NROW, IZETA, W(IW), DF, KD,              
     1                Z(IZ), DMZ(IDMZ), IPVTW(IDMZ), 2 )                
         IW = IW + KD * KD                                              
         IZ = IZ + MSTAR                                                
         IDMZ = IDMZ + KD                                               
         IF ( I .LT. N )                            GO TO 350           
  342    IF ( IZET .GT. MSIDE )                     GO TO 350           
            Z(IZ-1+IZET) = DGZ(IZET)                                    
            IZET = IZET + 1                                             
         GO TO 342                                                      
  350 CONTINUE                                                          
C                                                                       
C...  compute z with respect to new mesh                                
C...  for an underdetermined system set last component                  
C...  and restore continuation parameter                                
C                                                                       
      IF ( ICON .GT. 0 )  Z(NZPAR) = 0.D0                               
      CALL SBBLOK (G, G(LPAR), INTEGS, N, NEQ, NPAR, IPVTG, Z)          
      IF ( IPATH .NE. 1 .AND. ICON .GT. 0 )                             
     1   Z(NZ+ICON) = ZCON                                              
C                                                                       
C...  finaly find dmz                                                   
C                                                                       
      CALL DMZSOL (KD, MSTAR, N, V, Z, DMZ)                             
C                                                                       
      RETURN                                                            
      END                                                               
      SUBROUTINE GDERIV ( GI, PI, NROW, IROW, ZVAL, DGZ, MODE, DGSUB)   
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose:                                                            
C                                                                       
C      construct a collocation matrix row according to mode:            
C      mode = 1  -  a row corresponding to a initial condition          
C                   (i.e. at the left end of the subinterval).          
C      mode = 2  -  a row corresponding to a final condition.           
C                                                                       
C   variables:                                                          
C                                                                       
C      gi     - the sub-block of the global bvp matrix in               
C               which the equations are to be formed.                   
C      nrow   - no. of rows in gi.                                      
C      irow   - the row in gi to be used for equations.                 
C      zval   - z(xi)                                                   
C      dg     - the derivatives of the side condition.                  
C                                                                       
C********************************************************************** 
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION GI(NROW,1), PI(1), ZVAL(1), DGZ(1), DG(40)              
C                                                                       
      COMMON /COLAPR/ N, NOLD, NMAX, NZ, NZPAR, NDMZ                    
      COMMON /COLORD/ KDUM, NDUM, MSTAR, NPAR, MSTATE, KD, MMAX, M(20)  
      COMMON /COLSID/ ZETA(40), ALEFT, ARIGHT, MSIDE, IZETA, IDUM       
      COMMON /COLNLN/ NONLIN, ITER, LIMIT, ICARE, IGUESS, IREAD         
      COMMON /COLPTH/ IPATH, ICON, LPAR                                 
C                                                                       
C...  zero jacobian dg                                                  
C                                                                       
      DO 10 J=1,MSTATE                                                  
   10   DG(J) = 0.D0                                                    
C                                                                       
C...  evaluate jacobian dg                                              
C                                                                       
      CALL DGSUB (IZETA, ZVAL, DG)                                      
C                                                                       
C...  for continuation with fixed parameter                             
C...  zero corresponding derivatives                                    
C                                                                       
      JCON = MSTAR + ICON                                               
      IF ( IPATH .NE. 1 .AND. ICON .GT. 0 )  DG(JCON) = 0.D0            
C                                                                       
C...  evaluate  dgz = dg * zval  once for a new mesh                    
C                                                                       
      IF (NONLIN .EQ. 0 .OR. ITER .GT. 0)           GO TO 30            
      DOT = 0.D0                                                        
      DO 20 J = 1, MSTATE                                               
   20   DOT = DOT  +  DG(J) * ZVAL(J)                                   
      DGZ(IZETA) = DOT                                                  
C                                                                       
C...  branch according to  m o d e                                      
C                                                                       
   30 IF ( NPAR .EQ. 0 )                            GO TO 34            
      NEQ = N * MSTAR + MSIDE                                           
      DO 32 IPAR = 1, NPAR                                              
        PI(IROW + (IPAR-1)*NEQ) = - DG(MSTAR+IPAR)                      
   32 CONTINUE                                                          
   34 CONTINUE                                                          
      IF ( MODE .EQ. 2 )                            GO TO 50            
C                                                                       
C...  provide coefficients of the j-th linearized side condition.       
C...  specifically, at x=zeta(j) the j-th side condition reads          
C...  dg(1)*z(1) + ... +dg(mstar)*z(mstar) + g = 0                      
C                                                                       
C                                                                       
C...  handle an initial condition                                       
C                                                                       
      DO 40 J = 1, MSTAR                                                
        GI(IROW,J) =  DG(J)                                             
   40 GI(IROW,MSTAR+J) = 0.D0                                           
      RETURN                                                            
C                                                                       
C...  handle a final condition                                          
C                                                                       
   50 DO 60 J= 1, MSTAR                                                 
        GI(IROW,J) = 0.D0                                               
   60 GI(IROW,MSTAR+J) = DG(J)                                          
      RETURN                                                            
      END                                                               
      SUBROUTINE VWBLOK (XCOL, HRHO, JJ, WI, VI, IPVTW, KD, ZVAL,       
     1                   DF, ACOL, DMZO, NCOMP, DFSUB, MSING)           
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose:                                                            
C                                                                       
C      construct a group of  ncomp  rows of the matrices  wi  and  vi.  
C      corresponding to an interior collocation point.                  
C                                                                       
C                                                                       
C   variables:                                                          
C                                                                       
C      xcol   - the location of the collocation point.                  
C      jj     - xcol is the jj-th of k collocation points               
C               in the i-th subinterval.                                
C      wi,vi  - the i-th block of the collocation matrix                
C               before parameter condensation.                          
C      kd     - no. of rows in vi and wi .                              
C      zval   - z(xcol)                                                 
C      df     - the jacobian at xcol .                                  
C      jcomp  - counter for the component being dealt with.             
C                                                                       
C********************************************************************** 
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION WI(KD,1), VI(KD,1), ZVAL(1), DMZO(1), DF(NCOMP,1)       
      DIMENSION IPVTW(1),  HA(7,4), ACOL(7,4), BASM(5)                  
C                                                                       
      COMMON /COLORD/ K, NDUM, MSTAR, NPAR, MSTATE, KDUM, MMAX, M(20)   
      COMMON /COLNLN/ NONLIN, ITER, LIMIT, ICARE, IGUESS, IREAD         
      COMMON /COLPTH/ IPATH, ICON, LPAR                                 
C                                                                       
C...  if jj = 1 initialize  wi .                                        
C                                                                       
      IF ( JJ .GT. 1 )                              GO TO 30            
      DO 10 ID = 1, KD                                                  
        WI(ID,ID) = 1.D0                                                
   10 CONTINUE                                                          
C                                                                       
C...  calculate local basis                                             
C                                                                       
   30        FACT = 1.D0                                                
             DO 150 L=1,MMAX                                            
                FACT = FACT * HRHO / DFLOAT(L)                          
                BASM(L) = FACT                                          
                DO 150 J=1,K                                            
                   HA(J,L) = FACT * ACOL(J,L)                           
  150        CONTINUE                                                   
C                                                                       
C... zero jacobian                                                      
C                                                                       
      DO 40 JCOL = 1, MSTATE                                            
        DO 40 IR = 1, NCOMP                                             
   40 DF(IR,JCOL) = 0.D0                                                
C                                                                       
C...  build ncomp rows for interior collocation point x.                
C...  the linear expressions to be constructed are:                     
C...   (m(id))                                                          
C...  u     -  df(id,1)*z(1) - ... - df(id,mstar)*z(mstar)              
C...   id                                                               
C...  for id = 1 to ncomp.                                              
C                                                                       
      CALL DFSUB (XCOL, ZVAL, DF)                                       
      I0 = (JJ-1) * NCOMP                                               
      I1 = I0 + 1                                                       
      I2 = I0 + NCOMP                                                   
C                                                                       
C...  for continuation with fixed parameter                             
C...  zero corresponding derivatives                                    
C                                                                       
      IF ( IPATH .EQ. 1 .OR. ICON .EQ. 0 )          GO TO 44            
      JCON = MSTAR + ICON                                               
      DO 42 IR = 1, NCOMP                                               
        DF(IR,JCON) = 0.D0                                              
   42 CONTINUE                                                          
C                                                                       
C...  evaluate  dmzo = dmz - df * zval  once for a new mesh             
C                                                                       
   44 IF (NONLIN .EQ. 0 .OR. ITER .GT. 0)          GO TO 60             
      DO 50 J = 1, MSTATE                                               
        FACT = - ZVAL(J)                                                
        DO 50 ID = 1, NCOMP                                             
          DMZO(I0+ID) = DMZO(I0+ID)  +  FACT * DF(ID,J)                 
  50  CONTINUE                                                          
C                                                                       
C...  loop over the  ncomp  expressions to be set up for the            
C...  current collocation point.                                        
C                                                                       
   60 DO 70 J = 1, MSTATE                                               
        DO 70 ID = 1, NCOMP                                             
          VI(I0+ID,J) = DF(ID,J)                                        
   70 CONTINUE                                                          
      JN = 1                                                            
      DO 140 JCOMP = 1, NCOMP                                           
         MJ = M(JCOMP)                                                  
         JN = JN + MJ                                                   
         DO 130 L = 1, MJ                                               
            JV = JN - L                                                 
            JW = JCOMP                                                  
            DO 90 J = 1, K                                              
              AJL = - HA(J,L)                                           
              DO 80 IW = I1, I2                                         
                 WI(IW,JW) = WI(IW,JW)  +  AJL * VI(IW,JV)              
   80         CONTINUE                                                  
   90       JW = JW + NCOMP                                             
            LP1 = L + 1                                                 
            IF ( L .EQ. MJ )                        GO TO 130           
            DO 110 LL = LP1, MJ                                         
              JDF = JN - LL                                             
              BL = BASM(LL-L)                                           
              DO 100 IW = I1, I2                                        
                VI(IW,JV) = VI(IW,JV)  +  BL * VI(IW,JDF)               
  100         CONTINUE                                                  
  110       CONTINUE                                                    
  130    CONTINUE                                                       
  140 CONTINUE                                                          
      IF ( JJ .LT. K )                          RETURN                  
C                                                                       
C   ...decompose the wi block and solve for the mstar columns of vi     
C                                                                       
C                                                                       
C...  do parameter condensation                                         
C                                                                       
      MSING = 0                                                         
      CALL DGEFA  (WI, KD, KD, IPVTW, MSING)                            
C                                                                       
C...   check for singularity                                            
C                                                                       
      IF ( MSING .NE. 0 )                         RETURN                
      DO 250 J= 1,MSTAR                                                 
         CALL DGESL  (WI, KD, KD, IPVTW, VI(1,J), 0)                    
  250 CONTINUE                                                          
      RETURN                                                            
      END                                                               
      SUBROUTINE GBLOCK (H, GI, NROW, IROW, WI, VI, KD,                 
     1                   RHSZ, RHSDMZ, IPVTW, MODE)                     
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose:                                                            
C                                                                       
C      construct collocation matrix rows according to mode:             
C      mode = 1  -  a group of  mstar    rows corresponding             
C                   an interior mesh interval.                          
C           = 2  -  corresponding right hand side                       
C                                                                       
C   variables:                                                          
C                                                                       
C      h      - the  local stepsize.                                    
C      gi     - the sub-block of the collocation matrix in              
C               which the equations are to be formed.                   
C      wi     - the sub-block of noncondensed collocation equations,    
C               left-hand side part.                                    
C      vi     - the sub-block of noncondensed collocation equations,    
C               right-hand side part.                                   
C      rhsdmz - the inhomogenous term of the uncondensed collocation    
C               equations.                                              
C      rhsz   - the inhomogenous term of the condensed collocation      
C               equations.                                              
C      nrow   - no. of rows in gi.                                      
C      irow   - the first row in gi to be used for equations.           
C                                                                       
C********************************************************************** 
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION HB(7,4), BASM(5)                                        
      DIMENSION GI(NROW,1), WI(1), VI(KD,1)                             
      DIMENSION RHSZ(1), RHSDMZ(1), IPVTW(1)                            
C                                                                       
      COMMON /COLORD/ K, NCOMP, MSTAR, NPAR, MSTATE, KDUM, MMAX, M(20)  
      COMMON /COLBAS/ B(7,4), ACOL(28,7), ASAVE(28,4)                   
C                                                                       
C...  compute local basis                                               
C                                                                       
      FACT = 1.D0                                                       
      BASM(1) = 1.D0                                                    
      DO 30 L=1,MMAX                                                    
         FACT = FACT * H / DFLOAT(L)                                    
         BASM(L+1) = FACT                                               
         DO 20 J=1,K                                                    
   20       HB(J,L) = FACT * B(J,L)                                     
   30 CONTINUE                                                          
C                                                                       
C...  branch according to  m o d e                                      
C                                                                       
      GO TO (40, 110), MODE                                             
C                                                                       
C...  set right gi-block to identity                                    
C                                                                       
   40 CONTINUE                                                          
      DO 60 J = 1, MSTAR                                                
        DO 50 IR = 1, MSTAR                                             
          GI(IROW-1+IR,J) = 0.D0                                        
   50   GI(IROW-1+IR,MSTAR+J) = 0.D0                                    
   60 GI(IROW-1+J,MSTAR+J) = 1.D0                                       
C                                                                       
C...  compute the block gi                                              
C                                                                       
      IR = IROW                                                         
      DO 100 ICOMP = 1, NCOMP                                           
         MJ = M(ICOMP)                                                  
         IR = IR + MJ                                                   
         DO 90 L = 1, MJ                                                
            ID = IR - L                                                 
            DO 80 JCOL = 1, MSTAR                                       
               IND = ICOMP                                              
               RSUM = 0.D0                                              
               DO 70 J = 1, K                                           
                  RSUM = RSUM  -  HB(J,L) * VI(IND,JCOL)                
   70          IND = IND + NCOMP                                        
               GI(ID,JCOL) = RSUM                                       
   80       CONTINUE                                                    
            JD = ID - IROW                                              
            DO 85 LL = 1, L                                             
               GI(ID,JD+LL) = GI(ID,JD+LL) - BASM(LL)                   
   85       CONTINUE                                                    
   90    CONTINUE                                                       
  100 CONTINUE                                                          
      RETURN                                                            
C                                                                       
C...  compute the appropriate piece of  rhsz                            
C                                                                       
  110 CONTINUE                                                          
      CALL DGESL  (WI, KD, KD, IPVTW, RHSDMZ, 0)                        
      IR = IROW                                                         
      DO 140 JCOMP = 1, NCOMP                                           
         MJ = M(JCOMP)                                                  
         IR = IR + MJ                                                   
         DO 130 L = 1, MJ                                               
            IND = JCOMP                                                 
            RSUM = 0.D0                                                 
            DO 120 J = 1, K                                             
               RSUM = RSUM  +  HB(J,L) * RHSDMZ(IND)                    
  120       IND = IND + NCOMP                                           
            RHSZ(IR-L) = RSUM                                           
  130    CONTINUE                                                       
  140 CONTINUE                                                          
      RETURN                                                            
      END                                                               
      SUBROUTINE TANGNT ( G, V, INTEGS, IPVTG, TGT, DMTGT )             
C                                                                       
C********************************************************************** 
C                                                                       
C     supervises the computation of a tangent approximation             
C                                                                       
C    parameters                                                         
C       g, v, integs, ipvtg    are as in routine lsyslv.                
C       tgt, dmtgt             contains tangent on output.              
C                                                                       
C*********************************************************************  
C                                                                       
      IMPLICIT REAL*8(A-H,O-Z)                                          
      INTEGER INTEGS(1),IPVTG(1)                                        
      DOUBLE PRECISION G(1),V(1),TGT(1),DMTGT(1)                        
C                                                                       
      COMMON /COLAPR/ N, NOLD, NMAX, NZ, NZPAR, NDMZ                    
      COMMON /COLORD/ K, NCOMP, MSTAR, NPAR, MSTATE, KD, MMAX, M(20)    
      COMMON /COLPTH/ IPATH, ICON, LPAR                                 
C                                                                       
C...  set homogeneous rhs for tangent calculation                       
C                                                                       
      DO 10 I = 1, NZPAR                                                
         TGT(I) = 0.D0                                                  
   10 CONTINUE                                                          
      TGT(NZPAR) = 1.D0                                                 
C                                                                       
      DO 20 I = 1, NDMZ                                                 
         DMTGT(I) = 0.D0                                                
   20 CONTINUE                                                          
C                                                                       
C...  solve for mesh values of tangent                                  
C                                                                       
      NEQ = NZPAR - 1                                                   
      CALL SBBLOK ( G, G(LPAR), INTEGS, N, NEQ, NPAR, IPVTG, TGT )      
C                                                                       
C...  solve for m-th order derivative values of tangent                 
C                                                                       
      CALL DMZSOL (KD, MSTAR, N, V, TGT, DMTGT)                         
C                                                                       
      RETURN                                                            
      END                                                               
C---------------------------------------------------------------------- 
C               p a r t  4  of COLCON and COLPAR                        
C               polynomial and service routines                         
C---------------------------------------------------------------------- 
C                                                                       
      SUBROUTINE APPSLN (X, Z, FSPACE, ISPACE)                          
C                                                                       
C*****************************************************************      
C                                                                       
C     purpose                                                           
C                                                                       
C           set up a standard call to  approx  to evaluate the          
C           approximate solution  z = z( u(x),par )  at a point x       
C           (it has been computed by a call to  colcon or colpar ).     
C           the parameters needed for  approx  are retrieved            
C           from the work arrays  ispace  and  fspace .                 
C                                                                       
C*****************************************************************      
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION Z(1), FSPACE(1), ISPACE(1), A(28), DUMMY(1)             
C                                                                       
      N = ISPACE(1)                                                     
      K = ISPACE(2)                                                     
      NCOMP = ISPACE(3)                                                 
      MSTAR = ISPACE(4)                                                 
      MMAX = ISPACE(5)                                                  
      NPAR = ISPACE(6)                                                  
C                                                                       
      LZ = N + 2                                                        
      LDMZ = LZ + ( N + 1 ) * MSTAR + NPAR                              
      LCOEF = LDMZ + K * NCOMP * N                                      
C                                                                       
      I = 1                                                             
      CALL APPROX (I, X, Z, A, FSPACE(LCOEF), FSPACE(1), N,             
     1             FSPACE(LZ), FSPACE(LDMZ), K, NCOMP,                  
     2             MMAX, ISPACE(8), MSTAR, 2, DUMMY, 0)                 
      IF ( NPAR .EQ. 0 )                            RETURN              
C                                                                       
C...   retrieve parameter values                                        
C                                                                       
      LP = LDMZ - NPAR - 1                                              
      DO 10 IPAR = 1, NPAR                                              
         Z(MSTAR+IPAR) = FSPACE(LP+IPAR)                                
   10 CONTINUE                                                          
      RETURN                                                            
      END                                                               
      SUBROUTINE USER( XI, Z, DMZ, TGT, DMTGT, ISTEP, IEMBED, MODE )    
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose                                                             
C            provide solution information during path following         
C                                                                       
C   variables                                                           
C                                                                       
C            xi     - the current mesh                                  
C            z      - the solution vector                               
C            dmz    - m-th order derivatives                            
C            tgt    - tangent vector                                    
C            dmtgt  - m-th order derivatives                            
C            istep  - number of current continuation step               
C            iembed - suggested embedding parameter with                
C                     respect to final values                           
C            mode   - type of provided continuation point               
C                  = 0  point where a new branch starts                 
C                  = 1  normal continuation point                       
C                  = 2  turning point                                   
C                  = 3  bifurcation point                               
C                                                                       
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8(A-H,O-Z)                                          
      DIMENSION XI(1), Z(1), DMZ(1), TGT(1), DMTGT(1)                   
C                                                                       
      COMMON /COLAPR/ N, NOLD, NMAX, NZ, NZPAR, NDMZ                    
      COMMON /COLORD/ K, NCOMP, MSTAR, NPAR, MSTATE, KD, MMAX, M(20)    
C                                                                       
      RETURN                                                            
C                                                                       
      END                                                               
      SUBROUTINE APPROX (I, X, ZVAL, A, COEF, XI, N, Z, DMZ, K,         
     1                   NCOMP, MMAX, M, MSTAR, MODE, DMVAL, MODM )     
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose                                                             
C                                    (1)       (m1-1)     (mncomp-1)    
C           evaluate z(u(x))=(u (x),u (x),...,u  (x),...,u  (x)      )  
C                              1     1         1          mncomp        
C           at one point x.                                             
C                                                                       
C   variables                                                           
C     a      - array of mesh independent rk-basis coefficients          
C     basm   - array of mesh dependent monomial coefficients            
C     xi     - the current mesh (having n subintervals)                 
C     z      - the current solution vector                              
C     dmz    - the array of mj-th derivatives of the current solution   
C     mode   - determines the amount of initialization needed           
C            = 4  forms z(u(x)) using z, dmz and ha                     
C            = 3  as in =4, but computes local rk-basis                 
C            = 2  as in =3, but determines i such that                  
C                       xi(i) .le. x .lt. xi(i+1) (unless x=xi(n+1))    
C            = 1  retrieve  z=z(u(x(i)))  directly                      
C                                                                       
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION ZVAL(1), DMVAL(1), XI(1), M(1), A(7,1), DM(7)           
      DIMENSION Z(1), DMZ(1), BM(4), COEF(1)                            
C                                                                       
      COMMON /COLOUT/ PRECIS, IOUT, IPRINT                              
C                                                                       
      GO TO (10, 30, 80, 90), MODE                                      
C                                                                       
C...  mode = 1 , retrieve  z( u(x) )  directly for x = xi(i).           
C                                                                       
   10 X  = XI(I)                                                        
      IZ = (I-1) * MSTAR                                                
      DO 20 J = 1, MSTAR                                                
        IZ = IZ + 1                                                     
        ZVAL(J) = Z(IZ)                                                 
   20 CONTINUE                                                          
      RETURN                                                            
C                                                                       
C...  mode = 2 ,  locate i so  xi(i) .le. x .lt. xi(i+1)                
C                                                                       
   30 CONTINUE                                                          
      IF ( X .GE. XI(1)-PRECIS .AND. X .LE. XI(N+1)+PRECIS )            
     1                                              GO TO 40            
      IF (IPRINT .LT. 1)  WRITE(IOUT,900) X, XI(1), XI(N+1)             
      IF ( X .LT. XI(1) )  X = XI(1)                                    
      IF ( X .GT. XI(N+1) )  X = XI(N+1)                                
   40 IF ( I .GT. N .OR. I .LT. 1 )  I = (N+1) / 2                      
      ILEFT = I                                                         
      IF ( X .LT. XI(ILEFT) )                       GO TO 60            
      DO 50 L = ILEFT, N                                                
           I = L                                                        
           IF ( X .LT. XI(L+1) )                    GO TO 80            
   50 CONTINUE                                                          
      GO TO 80                                                          
   60 IRIGHT = ILEFT - 1                                                
      DO 70 L = 1, IRIGHT                                               
           I = IRIGHT + 1 - L                                           
           IF ( X .GE. XI(I) )                      GO TO 80            
   70 CONTINUE                                                          
C                                                                       
C...  mode = 2 or 3 , compute mesh independent rk-basis.                
C                                                                       
   80 CONTINUE                                                          
      S = (X - XI(I)) / (XI(I+1) - XI(I))                               
      CALL RKBAS ( S, COEF, K, MMAX, A, DM, MODM )                      
C                                                                       
C...  mode = 2, 3, or 4 , compute mesh dependent rk-basis.              
C                                                                       
   90 CONTINUE                                                          
      BM(1) = X - XI(I)                                                 
      DO 95 L = 2, MMAX                                                 
         BM(L) = BM(1) / DFLOAT(L)                                      
   95 CONTINUE                                                          
C                                                                       
C...  evaluate  z( u(x) ).                                              
C                                                                       
  100 IR = 1                                                            
      IZ = (I-1) * MSTAR + 1                                            
      IDMZ = (I-1) * K * NCOMP                                          
      DO 140 JCOMP = 1, NCOMP                                           
          MJ = M(JCOMP)                                                 
          IR = IR + MJ                                                  
          IZ = IZ + MJ                                                  
          DO 130 L = 1, MJ                                              
             IND = IDMZ + JCOMP                                         
             ZSUM = 0.D0                                                
             DO 110 J = 1, K                                            
               ZSUM = ZSUM  +  A(J,L) * DMZ(IND)                        
  110        IND = IND + NCOMP                                          
             DO 120 LL = 1, L                                           
               LB = L + 1 - LL                                          
  120          ZSUM = ZSUM * BM(LB)  +  Z(IZ-LL)                        
  130     ZVAL(IR-L) = ZSUM                                             
  140 CONTINUE                                                          
      IF ( MODM .EQ. 0 )                            RETURN              
C                                                                       
C...  for modm = 1 evaluate  dmval(j) = mj-th derivative of uj.         
C                                                                       
      DO 150 JCOMP = 1, NCOMP                                           
  150 DMVAL(JCOMP) = 0.D0                                               
      IDMZ = IDMZ + 1                                                   
      DO 170 J = 1, K                                                   
         FACT = DM(J)                                                   
         DO 160 JCOMP = 1, NCOMP                                        
            DMVAL(JCOMP) = DMVAL(JCOMP)  +  FACT * DMZ(IDMZ)            
            IDMZ = IDMZ + 1                                             
  160    CONTINUE                                                       
  170 CONTINUE                                                          
      RETURN                                                            
C--------------------------------------------------------------------   
  900 FORMAT(37H ****** DOMAIN ERROR IN APPROX ******                   
     1       /4H X =,D20.10, 10H   ALEFT =,D20.10,                      
     2       11H   ARIGHT =,D20.10)                                     
      END                                                               
      SUBROUTINE RKBAS (S, COEF, K, M, RKB, DM, MODE)                   
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose                                                             
C           evaluate mesh independent runge-kutta basis for given s     
C                                                                       
C   variables                                                           
C     s      - argument, i.e. the relative position for which           
C              the basis is to be evaluated ( 0. .le. s .le. 1. ).      
C     coef   - precomputed derivatives of the basis                     
C     k      - number of collocatin points per subinterval              
C     m      - maximal order of the differential equation               
C     rkb    - the runge-kutta basis (0-th to (m-1)-th derivatives )    
C     dm     - basis elements for m-th derivative                       
C                                                                       
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION COEF(K,1), RKB(7,1), DM(1), T(10)                       
C                                                                       
      IF ( K .EQ. 1 )                            GO TO 70               
      KPM1 = K + M - 1                                                  
      DO 10 I = 1, KPM1                                                 
   10   T(I) = S / DFLOAT(I)                                            
      DO 40 L = 1, M                                                    
         LB = K + L + 1                                                 
         DO 30 I = 1, K                                                 
           P = COEF(1,I)                                                
           DO 20 J = 2, K                                               
             P = P * T(LB-J)  + COEF(J,I)                               
   20      CONTINUE                                                     
           RKB(I,L) = P                                                 
   30    CONTINUE                                                       
   40 CONTINUE                                                          
      IF ( MODE .EQ. 0 )                         RETURN                 
      DO 60 I = 1, K                                                    
         P = COEF(1,I)                                                  
         DO 50 J = 2, K                                                 
   50       P = P * T(K+1-J) + COEF(J,I)                                
         DM(I) = P                                                      
   60 CONTINUE                                                          
      RETURN                                                            
   70 RKB(1,1) = 1.0D0                                                  
      DM(1) = 1.0D0                                                     
      RETURN                                                            
      END                                                               
      SUBROUTINE VMONDE ( RHO, COEF, K )                                
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose                                                             
C          solve vandermonde system v * x = e                           
C          with  v(i,j) = rho(j)**(i-1)/(i-1)! .                        
C                                                                       
C********************************************************************** 
C                                                                       
      INTEGER K, I,IFAC,J,KM1,KMI                                       
      DOUBLE PRECISION RHO(K), COEF(K)                                  
C                                                                       
      IF ( K .EQ. 1 )                             RETURN                
      KM1 = K - 1                                                       
      DO 10 I = 1, KM1                                                  
         KMI = K - I                                                    
         DO 10 J = 1, KMI                                               
           COEF(J) = (COEF(J+1) - COEF(J)) / (RHO(J+I) - RHO(J))        
  10  CONTINUE                                                          
C                                                                       
      IFAC = 1                                                          
      DO 40 I = 1, KM1                                                  
         KMI = K + 1 - I                                                
         DO 30 J = 2, KMI                                               
  30        COEF(J) = COEF(J) - RHO(J+I-1) * COEF(J-1)                  
         COEF(KMI) = DFLOAT(IFAC) * COEF(KMI)                           
         IFAC = IFAC * I                                                
  40  CONTINUE                                                          
      COEF(1) = DFLOAT(IFAC) * COEF(1)                                  
      RETURN                                                            
      END                                                               
      SUBROUTINE HORDER (I, UHIGH, HI, DMZ)                             
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose                                                             
C           determine highest order (piecewise constant) derivatives    
C           of the current collocation solution                         
C                                                                       
C   variables                                                           
C     hi     - the stepsize, hi = xi(i+1) - xi(i)                       
C     dmz    - vector of mj-th derivative of the solution               
C     uhigh  - the array of highest order (piecewise constant)          
C              derivatives of the approximate solution on               
C              (xi(i),xi(i+1)), viz,                                    
C                          (k+mj-1)                                     
C              uhigh(j) = u   (x)    on (xi(i),xi(i+1))                 
C                          j                                            
C                                                                       
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION UHIGH(1), DMZ(1)                                        
C                                                                       
      COMMON /COLLOC/ RHO(7), COEF(49)                                  
      COMMON /COLORD/ K, NCOMP, MSTAR, NPAR, MSTATE, KD, MMAX, M(20)    
C                                                                       
      DN = 1.D0 / HI**(K-1)                                             
C                                                                       
C...  loop over the ncomp solution components                           
C                                                                       
      DO 10 ID = 1, NCOMP                                               
         UHIGH(ID) = 0.D0                                               
   10 CONTINUE                                                          
      KIN = 1                                                           
      IDMZ = (I-1) * K * NCOMP + 1                                      
      DO 30 J = 1, K                                                    
         FACT = DN * COEF(KIN)                                          
         DO 20 ID = 1, NCOMP                                            
            UHIGH(ID) = UHIGH(ID)  +  FACT * DMZ(IDMZ)                  
            IDMZ = IDMZ + 1                                             
   20    CONTINUE                                                       
         KIN = KIN + K                                                  
   30 CONTINUE                                                          
      RETURN                                                            
      END                                                               
      SUBROUTINE DMZSOL (KD, MSTAR, N, V, Z, DMZ)                       
C                                                                       
C********************************************************************** 
C                                                                       
C   purpose                                                             
C          compute dmz in a blockwise manner                            
C          dmz(i) = dmz(i)  +  v(i) * z(i), i = 1,...,n                 
C                                                                       
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION V(KD,1), DMZ(KD,1), Z(1)                                
C                                                                       
      COMMON /COLAPR/ NDUM, NOLD, NMAX, NZ, NZPAR, NDMZ                 
      COMMON /COLORD/ K, NCOMP, MDUM, NPAR, MSTATE, KDUM, MMAX, M(20)   
C                                                                       
      JZ = 1                                                            
      JV = 1                                                            
      DO 50 I = 1, N                                                    
         DO 20 J = 1, MSTAR                                             
            FACT = Z(JZ)                                                
            DO 10 L = 1, KD                                             
               DMZ(L,I) = DMZ(L,I)  +  FACT * V(L,JV)                   
   10       CONTINUE                                                    
            JZ = JZ + 1                                                 
            JV = JV + 1                                                 
   20    CONTINUE                                                       
         IF ( NPAR .EQ. 0 )                         GO TO 50            
         DO 40 IPAR = 1, NPAR                                           
            FACT = Z( NZ + IPAR )                                       
            DO 30 L = 1, KD                                             
               DMZ(L,I) = DMZ(L,I) + FACT * V(L,JV)                     
   30       CONTINUE                                                    
            JV = JV + 1                                                 
   40    CONTINUE                                                       
   50 CONTINUE                                                          
      RETURN                                                            
      END                                                               
      SUBROUTINE L2DOT (XI, Z1, DMZ1, Z2, DMZ2,                         
     1                  SCALE, DSCALE, DOTL2, MODE)                     
C                                                                       
C********************************************************************** 
C                                                                       
C      purpose                                                          
C               determine the minimal least-square solution of the      
C               bvp with respect to a discretized L2-norm:              
C      mode = 0 - calculate L2-norm (z1,z1).                            
C           = 1 - normalize z1 to z1/sqrt(z1,z1).                       
C           = 2 - calculate L2-scalar product (z1,z2).                  
C           = 3 - find minimal least square solution z1-(z1,z2)*z2.     
C                 (z2 is assumed to have unit norm.)                    
C                                                                       
C      variables                                                        
C        xi     - current mesh points                                   
C        z1     - first solution values at mesh points.                 
C        dmz1   - first solution values for m-th order derivatives.     
C        z2     - second solution values at mesh points.                
C        dmz2   - second solution values for m-th order derivatives.    
C        scale  - scaling vector for z.                                 
C        dscale - scaling vector for dmz.                               
C        dotl2  - scalar product.                                       
C                                                                       
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION Z1VAL(40), Z2VAL(40), DUMMY(1)                          
      DIMENSION XI(1), Z1(1), DMZ1(1), Z2(1), DMZ2(1)                   
      DIMENSION SCALE(1), DSCALE(1)                                     
C                                                                       
      COMMON /COLLOC/ RHO(7), COEF(49)                                  
      COMMON /COLAPR/ N, NOLD, NMAX, NZ, NZPAR, NDMZ                    
      COMMON /COLORD/ K, NCOMP, MSTAR, NPAR, MSTATE, KD, MMAX, M(20)    
      COMMON /COLBAS/ B(7,4), ACOL(28,7), ASAVE(28,4)                   
C                                                                       
      LSCALE = 1                                                        
      LDSCAL = 1                                                        
      DOTL2 = 0.D0                                                      
      DO 60 I = 1, N                                                    
        XMSH = XI(I)                                                    
        HI = XI(I+1) - XI(I)                                            
        DOTI = 0.D0                                                     
        DO 50 J = 1, K                                                  
           XCOL = XMSH + HI * RHO(J)                                    
           DOT = 0.D0                                                   
C                                                                       
C...       look over the collocation points to calculate the            
C...       l2-norm or the l2-scalar product.                            
C                                                                       
           CALL APPROX (I, XCOL, Z1VAL, ACOL(1,J), DUMMY, XI, N,        
     1           Z1, DMZ1, K, NCOMP, MMAX, M, MSTAR, 4, DUMMY, 0)       
           IF ( MODE .GE. 2 )                       GO TO 30            
C                                                                       
C...       compute pointwise norm <z1,z1>                               
C                                                                       
           DO 20 L = 1, MSTAR                                           
             DOT = DOT + (Z1VAL(L) * SCALE(LSCALE))**2                  
             LSCALE = LSCALE + 1                                        
   20      CONTINUE                                                     
           DO 25 L = 1, NCOMP                                           
             DOT = DOT + (DMZ1(LDSCAL) * DSCALE(LDSCAL))**2             
             LDSCAL = LDSCAL + 1                                        
   25      CONTINUE                                                     
           GO TO 50                                                     
C                                                                       
   30      CALL APPROX (I, XCOL, Z2VAL, ACOL(1,J), DUMMY, XI, N,        
     1           Z2, DMZ2, K, NCOMP, MMAX, M, MSTAR, 4, DUMMY, 0)       
C                                                                       
C...       compute pointwise scalar product <z1,z2>                     
C                                                                       
           DO 40 L = 1, MSTAR                                           
             DOT = DOT + Z1VAL(L) * Z2VAL(L) * SCALE(LSCALE)**2         
             LSCALE = LSCALE + 1                                        
   40      CONTINUE                                                     
           DO 45 L = 1, NCOMP                                           
             DOT = DOT + DMZ1(LDSCAL) * DMZ2(LDSCAL) * DSCALE(LDSCAL)**2
             LDSCAL = LDSCAL + 1                                        
   45      CONTINUE                                                     
C                                                                       
C...       update scalar product                                        
C                                                                       
   50    DOTI = DOTI + B(J,1) * DOT                                     
         DOTL2 = DOTL2 + HI * DOTI                                      
   60 CONTINUE                                                          
C                                                                       
C...   loop over parameter block                                        
C                                                                       
      IF ( MODE .GE. 2 )                            GO TO 70            
      IF ( NPAR .EQ. 0 )                            GO TO 64            
      DO 62 IPAR = 1, NPAR                                              
         DOTL2 = DOTL2 + (Z1(NZ+IPAR) * SCALE(LSCALE))**2               
         LSCALE = LSCALE + 1                                            
   62 CONTINUE                                                          
   64 CONTINUE                                                          
      IF ( MODE .EQ. 0 )                            RETURN              
      GO TO 100                                                         
   70 IF ( NPAR .EQ. 0 )                            GO TO 74            
      DO 72 IPAR = 1, NPAR                                              
         DOTL2 = DOTL2 + Z1(NZ+IPAR) * Z2(NZ+IPAR) * SCALE(LSCALE)**2   
         LSCALE = LSCALE + 1                                            
   72 CONTINUE                                                          
   74 CONTINUE                                                          
      IF ( MODE .EQ. 2 )                            RETURN              
C                                                                       
C...  calculate least-square solution  z = z - (z,tgt)*tgt              
C...  (the tangent is assumed to have unit norm).                       
C                                                                       
         ZN = - DOTL2                                                   
         DO 80 I = 1, NZPAR                                             
           Z1(I) = Z1(I) + ZN * Z2(I)                                   
   80    CONTINUE                                                       
         DO 90 I = 1, NDMZ                                              
           DMZ1(I) = DMZ1(I) + ZN * DMZ2(I)                             
   90    CONTINUE                                                       
      RETURN                                                            
C                                                                       
C...  normalize first solution                                          
C                                                                       
  100    ZN = 1.D0 / DSQRT(DOTL2)                                       
         DO 110 I = 1, NZPAR                                            
           Z1(I) = ZN * Z1(I)                                           
  110    CONTINUE                                                       
         DO 120 I = 1, NDMZ                                             
           DMZ1(I) = ZN * DMZ1(I)                                       
  120    CONTINUE                                                       
      RETURN                                                            
      END                                                               
      SUBROUTINE ZINT (N, NOLD, XI, XIOLD, Z, DMZ, ZOLD, DMZOLD)        
C                                                                       
C********************************************************************** 
C                                                                       
C      purpose                                                          
C               interpolate old solution on new mesh                    
C                                                                       
C      variables                                                        
C        xi     - current mesh points                                   
C        xiold  - old mesh points                                       
C        z      - solution values at mesh points of current mesh.       
C        dmz    - solution values for m-th order derivatives.           
C        zold   - solution values at mesh points of old mesh.           
C        dmzold - solution values for m-th order derivatives.           
C                                                                       
C********************************************************************** 
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      DIMENSION ZVAL(40), AT(49), DUMMY(1)                              
      DIMENSION XI(1), XIOLD(1), Z(1), DMZ(1), ZOLD(1), DMZOLD(1)       
C                                                                       
      COMMON /COLLOC/ RHO(7), COEF(49)                                  
      COMMON /COLORD/ K, NCOMP, MSTAR, NPAR, MSTATE, KD, MMAX, M(20)    
C                                                                       
      IZ = 1                                                            
      IDMZ = 1                                                          
      IOLD = 1                                                          
      DO 20 I = 1, N                                                    
        XMSH = XI(I)                                                    
        HI = XI(I+1) - XI(I)                                            
        CALL APPROX (IOLD, XMSH, Z(IZ), AT, COEF, XIOLD, NOLD,          
     1       ZOLD, DMZOLD, K, NCOMP, MMAX, M, MSTAR, 2, DUMMY, 0)       
        IZ = IZ + MSTAR                                                 
        DO 10 J = 1, K                                                  
           XCOL = XMSH + HI * RHO(J)                                    
C                                                                       
C...       loop over the collocation points to calculate the            
C...       solution with respect to new mesh xi.                        
C                                                                       
           CALL APPROX (IOLD, XCOL, ZVAL, AT, COEF, XIOLD, NOLD,        
     1          ZOLD, DMZOLD, K, NCOMP, MMAX, M, MSTAR, 2, DMZ(IDMZ), 1)
           IDMZ = IDMZ + NCOMP                                          
   10    CONTINUE                                                       
   20 CONTINUE                                                          
      IOLD = NOLD + 1                                                   
      CALL APPROX (IOLD, XMSH, Z(IZ), AT, COEF, XIOLD, NOLD,            
     1     ZOLD, DMZOLD, K, NCOMP, MMAX, M, MSTAR, 1, DUMMY, 0)         
C                                                                       
      IF ( NPAR .EQ. 0 )                            RETURN              
      NZ = (N + 1) * MSTAR                                              
      NZOLD = (NOLD + 1) * MSTAR                                        
      DO 30 IPAR = 1, NPAR                                              
        Z( NZ + IPAR ) = ZOLD( NZOLD + IPAR )                           
   30 CONTINUE                                                          
      RETURN                                                            
      END                                                               
C---------------------------------------------------------------------- 
C                p a r t  5  of COLCON and COLPAR                       
C          we list here a modified (column oriented, faster)            
C          version of the package solveblok of de boor - weiss [5].     
C          we have also included an option for solving parameter        
C          dependend problems.                                          
C          we also give a listing of the linpack                        
C          routines dgefa und dgesl used by colcon and colpar.          
C---------------------------------------------------------------------- 
C                                                                       
      SUBROUTINE FCBLOK (BLOKS, P, INTEGS, NBLOKS, NEQ, NPAR,           
     1                   IPIVOT, SCRTCH, INFO)                          
C                                                                       
C                                                                       
C     calls subroutines  factrb, factrc, subfor, and shiftb .           
C                                                                       
C     fcblok  supervises the lu factorization with pivoting of scaled   
C     rows or columns of the almost block diagonal matrix stored in the 
C     arrays  bloks  and  integs .                                      
C                                                                       
C     factrb = subprogram which carries out steps 1,...,last of gauss   
C            elimination (with pivoting) for an individual block.       
C     factrc = subprogram for column-oriented lu decomposition.         
C     subfor = subprogram which carries out the elimination of          
C            parameter columns.                                         
C     shiftb = subprogram which shifts the remaining rows to the top of 
C            the next block                                             
C                                                                       
C     parameters                                                        
C      bloks   an array that initially contains the almost block diago- 
C            nal matrix  a  to be factored, and on return contains the  
C            computed factorization of  a .                             
C      p     an array containing the parameter columns of the matrix .  
C      integs  an integer array describing the block structure of  a .  
C      nbloks  the number of blocks in  a .                             
C      neq   the number of equations .                                  
C      npar  the number of parameters .                                 
C      ipivot  an integer array of dimension   sum (integs(3,n) ; n=1,  
C            ...,nbloks) which, on return, contains the pivoting stra-  
C            tegy used.                                                 
C      scrtch  work area required, of length  max (integs(1,n) ; n=1,   
C            ...,nbloks).                                               
C      info    output parameter;                                        
C            = 0  in case matrix was found to be nonsingular.           
C            = n if the pivot element in the nth gauss step is zero.    
C                                                                       
C********************************************************************** 
C                                                                       
      INTEGER INTEGS(3,NBLOKS),IPIVOT(1),INFO,NEQ,NPAR, I,INDEX,INDEXN, 
     1        LAST,NCOL,NROW                                            
      DOUBLE PRECISION BLOKS(1),P(NEQ,1),SCRTCH(1)                      
C                                                                       
      INFO = 0                                                          
      INDEXX = 1                                                        
      INDEXN = 1                                                        
      I = 1                                                             
      IF ( NBLOKS .EQ. 1 )                          GO TO 40            
C                                                                       
C...  loop over the blocks.  i  is loop index                           
C                                                                       
   10      INDEX = INDEXN                                               
           NROW = INTEGS(1,I)                                           
           NCOL = INTEGS(2,I)                                           
           LAST = INTEGS(3,I)                                           
C                                                                       
C...       carry out elimination on the i-th block until next block     
C...       enters, i.e., for columns 1,...,last  of i-th block.         
C                                                                       
           CALL FACTRB ( BLOKS(INDEX), IPIVOT(INDEXX), SCRTCH, NROW,    
     1                   NCOL, LAST, INFO)                              
C                                                                       
C...       check for having reached a singular block                    
C                                                                       
           IF ( INFO .NE. 0 )                       GO TO 70            
C                                                                       
C...       eliminate parameter columns                                  
C                                                                       
           IF ( NPAR .EQ. 0 )                       GO TO 30            
           DO 20 IPAR = 1, NPAR                                         
             CALL SUBFOR ( BLOKS(INDEX), IPIVOT(INDEXX),                
     1                      NROW, LAST, P(INDEXX,IPAR) )                
   20      CONTINUE                                                     
   30      I = I + 1                                                    
           INDEXN = NROW * NCOL + INDEX                                 
           INDEXX = INDEXX + LAST                                       
C                                                                       
C...       put the rest of the i-th block onto the next block           
C                                                                       
           CALL SHIFTB ( BLOKS(INDEX), NROW, NCOL, LAST,                
     1                   BLOKS(INDEXN), INTEGS(1,I), INTEGS(2,I) )      
      IF ( I .LT. NBLOKS )                          GO TO 10            
C                                                                       
C...  eliminate last block                                              
C                                                                       
   40 INDEX = INDEXN                                                    
      NROW = INTEGS(1,NBLOKS)                                           
      NCOL = INTEGS(2,NBLOKS)                                           
      LAST = INTEGS(3,NBLOKS)                                           
      IF ( NPAR .GT. 0 )                            GO TO 50            
C                                                                       
C...  perform row elimination for problems without parameters           
C                                                                       
      CALL FACTRB ( BLOKS(INDEX), IPIVOT(INDEXX), SCRTCH, NROW,         
     1              NCOL, LAST, INFO)                                   
C                                                                       
C...  check for singularity                                             
C                                                                       
      IF ( INFO .NE. 0 )                            GO TO 70            
      RETURN                                                            
C                                                                       
C...  copy parameter columns into last block                            
C                                                                       
   50 INDEXP = INDEX + NROW * (NCOL - NPAR)                             
      INDEXR = INDEXX - 1                                               
      DO 60 IPAR = 1, NPAR                                              
        DO 60 J = 1, NROW                                               
          BLOKS(INDEXP) = - P(INDEXR+J,IPAR)                            
          INDEXP = INDEXP + 1                                           
   60 CONTINUE                                                          
C                                                                       
C...  perform column elimination for problems with parameters           
C                                                                       
      CALL FACTRC ( BLOKS(INDEX), IPIVOT(INDEXX),                       
     1              SCRTCH, NROW, NCOL, LAST, INFO )                    
C                                                                       
C...  check singularity                                                 
C                                                                       
      IF ( INFO .EQ. 0 )                            RETURN              
C                                                                       
   70 INFO = INFO + INDEXX - 1                                          
      RETURN                                                            
      END                                                               
      SUBROUTINE FACTRB ( W, IPIVOT, D, NROW, NCOL, LAST, INFO)         
C                                                                       
C********************************************************************   
C                                                                       
C     adapted from p.132 of  element.numer.analysis  by conte-de boor   
C                                                                       
C     constructs a partial plu factorization, corresponding to steps    
C      1,..., last   in gauss elimination, for the matrix  w  of        
C      order ( nrow ,  ncol ), using pivoting of scaled rows.           
C                                                                       
C     parameters                                                        
C       w       contains the (nrow,ncol) matrix to be partially factored
C               on input, and the partial factorization on output.      
C       ipivot  an integer array of length last containing a record of  
C               the pivoting strategy used; explicit interchanges       
C               are used for pivoting.                                  
C       d       a work array of length nrow used to store row sizes     
C               temporarily.                                            
C       nrow    number of rows of w.                                    
C       ncol    number of columns of w.                                 
C       last    number of elimination steps to be carried out.          
C       info    on output, zero if the matrix is found to be non-       
C               singular, in case a zero pivot was encountered in row   
C               n,  info = n on output.                                 
C                                                                       
C********************************************************************** 
C                                                                       
      INTEGER IPIVOT(NROW),NCOL,LAST,INFO, I,J,K,L,KP1                  
      DOUBLE PRECISION W(NROW,NCOL),D(NROW), COLMAX,T,S                 
      DOUBLE PRECISION DABS,DMAX1                                       
C                                                                       
C...  initialize  d                                                     
C                                                                       
      DO 10 I = 1, NROW                                                 
        D(I) = 0.D0                                                     
   10 CONTINUE                                                          
      DO 20 J = 1, NCOL                                                 
        DO 20 I = 1, NROW                                               
          D(I) = DMAX1( D(I) , DABS(W(I,J)))                            
   20 CONTINUE                                                          
C                                                                       
C...  gauss elimination with pivoting of scaled rows, loop over         
C...  k=1,.,last                                                        
C                                                                       
      K = 1                                                             
C                                                                       
C...  as pivot row for k-th step, pick among the rows not yet used,     
C...  i.e., from rows  k ,..., nrow , the one whose k-th entry          
C...  (compared to the row size) is largest. then, if this row          
C...  does not turn out to be row k, interchange row k with this        
C...  particular row and define ipivot(k).                              
C                                                                       
   30      CONTINUE                                                     
           IF (K .EQ. NROW)                         GO TO 80            
           L = K                                                        
           KP1 = K+1                                                    
           COLMAX = 0.D0                                                
C                                                                       
C...       find the (relatively) largest pivot                          
C                                                                       
           DO 40 I = K, NROW                                            
             IF ( DABS(W(I,K)) .LE. COLMAX * D(I) ) GO TO 40            
             COLMAX = DABS(W(I,K)) / D(I)                               
             L = I                                                      
   40      CONTINUE                                                     
           IPIVOT(K) = L                                                
           T = W(L,K)                                                   
           S = D(L)                                                     
           IF ( L .EQ. K )                          GO TO 50            
             W(L,K) = W(K,K)                                            
             W(K,K) = T                                                 
             D(L) = D(K)                                                
             D(K) = S                                                   
   50      CONTINUE                                                     
C                                                                       
C...       if pivot element is too small in absolute value, declare     
C...       matrix to be noninvertible and quit.                         
C                                                                       
           IF ( DABS(T)+D(K) .LE. D(K) )            GO TO 90            
C                                                                       
C...       otherwise, subtract the appropriate multiple of the pivot    
C...       row from remaining rows, i.e., the rows (k+1),..., (nrow)    
C...       to make k-th entry zero. save the multiplier in its place.   
C...       for high performance do this operations column oriented.     
C                                                                       
           T = -1.0D0/T                                                 
           DO 60 I = KP1, NROW                                          
   60        W(I,K) = W(I,K) * T                                        
           DO 70 J=KP1,NCOL                                             
             T = W(L,J)                                                 
             IF ( L .EQ. K )                        GO TO 62            
               W(L,J) = W(K,J)                                          
               W(K,J) = T                                               
   62        IF ( T .EQ. 0.D0 )                     GO TO 70            
             DO 64 I = KP1, NROW                                        
   64           W(I,J) = W(I,J) + W(I,K) * T                            
   70      CONTINUE                                                     
           K = KP1                                                      
C                                                                       
C...       check for having reached the next block.                     
C                                                                       
           IF ( K .LE. LAST )                       GO TO 30            
      RETURN                                                            
C                                                                       
C...  if  last  .eq. nrow , check now that pivot element in last row    
C...  is nonzero.                                                       
C                                                                       
   80 IF( DABS(W(NROW,NROW))+D(NROW) .GT. D(NROW) ) RETURN              
C                                                                       
C...  singularity flag set                                              
C                                                                       
   90 INFO = K                                                          
      RETURN                                                            
      END                                                               
      SUBROUTINE FACTRC ( W, IPIVOT, D, NROW, NCOL, LAST, INFO)         
C                                                                       
C********************************************************************   
C                                                                       
C     constructs a partial lup factorization, corresponding to steps    
C      1,..., last   in gauss elimination, for the matrix  w  of        
C      order ( nrow ,  ncol ), using pivoting of scaled columns .       
C                                                                       
C     parameters                                                        
C       w       contains the (nrow,ncol) matrix to be partially factored
C               on input, and the partial factorization on output.      
C       ipivot  an integer array of length last containing a record of  
C               the pivoting strategy used; explicit interchanges       
C               are used for pivoting.                                  
C       d       a work array of length ncol used to store column sizes  
C               temporarily.                                            
C       nrow    number of rows of w.                                    
C       ncol    number of columns of w.                                 
C       last    number of elimination steps to be carried out.          
C       info    on output, zero if the matrix is found to be non-       
C               singular, in case a zero pivot was encountered in       
C               column n,  info = n on output.                          
C                                                                       
C********************************************************************** 
C                                                                       
      INTEGER IPIVOT(NROW),NCOL,LAST,INFO, I,J,K,L,KP1                  
      DOUBLE PRECISION W(NROW,NCOL),D(NCOL), ROWMAX,T,S                 
      DOUBLE PRECISION DABS,DMAX1                                       
C                                                                       
C...  initialize  d                                                     
C                                                                       
      DO 20 J = 1, NCOL                                                 
        T = 0.D0                                                        
        DO 10 I = 1, NROW                                               
   10     T = DMAX1( T , DABS(W(I,J)))                                  
        IF ( T .EQ. 0.D0 )  T = 1.D0                                    
        D(J) = T                                                        
   20 CONTINUE                                                          
C                                                                       
C...  gauss elimination with pivoting of scaled columns, loop over      
C...  k=1,.,last                                                        
C                                                                       
      K = 1                                                             
C                                                                       
C...  as pivot column for k-th step, pick among the columns not yet     
C...  used, i.e., from columns  k ,..., ncol , the one whose kth entry  
C...  (compared to the column size) is largest. then, if this column    
C...  does not turn out to be column k, interchange column k with this  
C...  particular column and define ipivot(k).                           
C                                                                       
   30      CONTINUE                                                     
           L = K                                                        
           KP1 = K+1                                                    
           ROWMAX = DABS(W(K,K)) / D(K)                                 
C                                                                       
C...       find the (relatively) largest pivot                          
C                                                                       
           IF ( K .EQ. NCOL )                       GO TO 50            
           DO 40 J = KP1, NCOL                                          
             IF ( DABS(W(K,J)) .LE. ROWMAX * D(J) ) GO TO 40            
             ROWMAX = DABS(W(K,J)) / D(J)                               
             L = J                                                      
   40      CONTINUE                                                     
C                                                                       
C...       interchange columns k and l                                  
C                                                                       
           IF ( L .EQ. K )                          GO TO 50            
             DO 44 I = 1, NROW                                          
               S = W(I,L)                                               
               W(I,L) = W(I,K)                                          
               W(I,K) = S                                               
   44        CONTINUE                                                   
             S = D(L)                                                   
             D(L) = D(K)                                                
             D(K) = S                                                   
   50      CONTINUE                                                     
           IPIVOT(K) = L                                                
C                                                                       
C...       if pivot element is too small in absolute value, declare     
C...       matrix to be noninvertible and quit.                         
C                                                                       
           IF ( 1.D0 + ROWMAX .LE. 1.D0 )           GO TO 90            
           IF ( K .EQ. NCOL )                       RETURN              
C                                                                       
C...       otherwise, subtract the appropriate multiple of the pivot    
C...       row from remaining rows, i.e., the rows (k+1),..., (nrow)    
C...       to make k-th entry zero. save the multiplier in its place.   
C...       for high performance do this operations column oriented.     
C                                                                       
           T = -1.0D0/W(K,K)                                            
           DO 70 J=KP1,NCOL                                             
             S = W(K,J) * T                                             
             W(K,J) = S                                                 
             IF ( K .EQ. NROW )                     GO TO 70            
             IF ( S .EQ. 0.D0 )                     GO TO 70            
             DO 60 I = KP1, NROW                                        
   60           W(I,J) = W(I,J) + W(I,K) * S                            
   70      CONTINUE                                                     
           K = KP1                                                      
C                                                                       
C...       check for having reached the next block.                     
C                                                                       
           IF ( K .LE. LAST )                       GO TO 30            
      RETURN                                                            
C                                                                       
C...  singularity flag set                                              
C                                                                       
   90 INFO = K                                                          
      RETURN                                                            
      END                                                               
      SUBROUTINE SHIFTB (AI, NROWI, NCOLI, LAST, AI1, NROWI1, NCOLI1)   
C                                                                       
C*********************************************************************  
C                                                                       
C     shifts the rows in current block, ai, not used as pivot rows, if  
C     any, i.e., rows  (last+1),..., (nrowi), onto the first mmax =     
C      = nrow-last  rows of the next block, ai1, with column last+j of  
C      ai  going to column j , j=1,...,jmax=ncoli-last. the remaining   
C     columns of these rows of ai1 are zeroed out.                      
C                                                                       
C                                picture                                
C                                                                       
C          original situation after         results in a new block i+1  
C          last = 2 columns have been       created and ready to be     
C          done in factrb (assuming no      factored by next factrb     
C          interchanges of rows)            call.                       
C                      1                                                
C                 x  x 1x  x  x           x  x  x  x  x                 
C                      1                                                
C                 0  x 1x  x  x           0  x  x  x  x                 
C     block i          1                       ---------------          
C     nrowi = 4   0  0 1x  x  x           0  0 1x  x  x  0  01          
C     ncoli = 5        1                       1             1          
C     last = 2    0  0 1x  x  x           0  0 1x  x  x  0  01          
C     -------------------------------          1             1   new    
C                      1x  x  x  x  x          1x  x  x  x  x1  block   
C                      1                       1             1   i+1    
C     block i+1        1x  x  x  x  x          1x  x  x  x  x1          
C     nrowi1= 5        1                       1             1          
C     ncoli1= 5        1x  x  x  x  x          1x  x  x  x  x1          
C     -------------------------------          1-------------1          
C                      1                                                
C                                                                       
C*********************************************************************  
C                                                                       
      INTEGER LAST, J,JMAX,JMAXP1,M,MMAX                                
      DOUBLE PRECISION AI(NROWI,NCOLI),AI1(NROWI1,NCOLI1)               
      MMAX = NROWI - LAST                                               
      JMAX = NCOLI - LAST                                               
      IF (MMAX .LT. 1 .OR. JMAX .LT. 1)             RETURN              
C                                                                       
C...  put the remainder of block i into ai1                             
C                                                                       
      DO 10 J=1,JMAX                                                    
           DO 10 M=1,MMAX                                               
   10 AI1(M,J) = AI(LAST+M,LAST+J)                                      
      IF (JMAX .EQ. NCOLI1)                         RETURN              
C                                                                       
C...  zero out the upper right corner of ai1                            
C                                                                       
      JMAXP1 = JMAX + 1                                                 
      DO 20 J=JMAXP1,NCOLI1                                             
           DO 20 M=1,MMAX                                               
   20 AI1(M,J) = 0.D0                                                   
      RETURN                                                            
      END                                                               
      SUBROUTINE SBBLOK (BLOKS, P, INTEGS, NBLOKS, NEQ, NPAR, IPIVOT, X)
C                                                                       
C********************************************************************** 
C                                                                       
C     calls subroutines  subfor, subfrc, subbak, and subbkc.            
C                                                                       
C     supervises the solution (by forward and backward substitution) of 
C     the linear system  a*x = b  for x, with the lu factorization of   
C     a  already generated in  fcblok .  individual blocks of           
C     equations are solved via   subfor and subbak  or  subfrc and      
C     subbkc, respectively .                                            
C                                                                       
C    parameters                                                         
C       bloks, integs, nbloks, neq, npar, ipivot,   are as on return    
C               from fcblok.                                            
C       x       on input: the right hand side, in dense storage         
C               on output: the solution vector                          
C                                                                       
C*********************************************************************  
C                                                                       
      INTEGER INTEGS(3,NBLOKS),IPIVOT(1),NEQ,NPAR, I,INDEX,INDEXX,J,    
     1        LAST,NBP1,NCOL,NROW                                       
      DOUBLE PRECISION BLOKS(1),P(NEQ,1),X(1), T                        
C                                                                       
C...  forward substitution pass                                         
C                                                                       
      INDEX = 1                                                         
      INDEXX = 1                                                        
      NBLOK = NBLOKS                                                    
      IF ( NPAR .GT. 0 )  NBLOK = NBLOKS - 1                            
      IF ( NBLOK .EQ. 0 )                           GO TO 20            
      DO 10 I = 1, NBLOK                                                
           NROW = INTEGS(1,I)                                           
           LAST = INTEGS(3,I)                                           
           CALL SUBFOR ( BLOKS(INDEX), IPIVOT(INDEXX), NROW, LAST,      
     1                   X(INDEXX) )                                    
           INDEX = NROW * INTEGS(2,I) + INDEX                           
   10 INDEXX = INDEXX + LAST                                            
      IF ( NPAR .EQ. 0 )                            GO TO 50            
C                                                                       
   20 NROW = INTEGS(1,NBLOKS)                                           
      NCOL = INTEGS(2,NBLOKS)                                           
      LAST = INTEGS(3,NBLOKS)                                           
C                                                                       
C...  forward substitution pass for last block                          
C                                                                       
      CALL SUBFRC ( BLOKS(INDEX), NROW, LAST, X(INDEXX) )               
C                                                                       
C...  back substitution pass for last block                             
C                                                                       
      CALL SUBBKC ( BLOKS(INDEX), IPIVOT(INDEXX),                       
     1              NROW, NCOL, LAST, X(INDEXX) )                       
      IF ( NBLOK .EQ. 0 )                           RETURN              
C                                                                       
C...  back substitution for parameter columns                           
C                                                                       
      INDEXR = INDEXX - 1                                               
      INDEXP = INDEXR + NCOL - NPAR                                     
      DO 40 IPAR = 1, NPAR                                              
        T = X(INDEXP+IPAR)                                              
        DO 30 J = 1, INDEXR                                             
          X(J) = X(J) + P(J,IPAR) * T                                   
   30   CONTINUE                                                        
   40 CONTINUE                                                          
C                                                                       
C...  back substitution pass for remaining blocks                       
C                                                                       
   50 NBP1 = NBLOK + 1                                                  
      DO 60 J = 1, NBLOK                                                
           I = NBP1 - J                                                 
           NROW = INTEGS(1,I)                                           
           NCOL = INTEGS(2,I)                                           
           LAST = INTEGS(3,I)                                           
           INDEX = INDEX - NROW * NCOL                                  
           INDEXX = INDEXX - LAST                                       
   60 CALL SUBBAK ( BLOKS(INDEX), NROW, NCOL, LAST, X(INDEXX) )         
      RETURN                                                            
      END                                                               
      SUBROUTINE SUBFOR ( W, IPIVOT, NROW, LAST, X )                    
C                                                                       
C********************************************************************** 
C                                                                       
C     carries out the forward pass of substitution for the current      
C     block, i.e., the action on the right side corresponding to the    
C     elimination carried out in  factrb  for this block.               
C                                                                       
C    parameters                                                         
C       w, ipivot, nrow, last  are as on return from factrb.            
C       x(j)  is expected to contain, on input, the right side of j-th  
C             equation for this block, j=1,...,nrow.                    
C       x(j)  contains, on output, the appropriately modified right     
C             side of equation (j) in this block, j=1,...,last and      
C             for j=last+1,...,nrow.                                    
C                                                                       
C*********************************************************************  
C                                                                       
      INTEGER IPIVOT(LAST), IP,K,KP1,LSTEP                              
      DOUBLE PRECISION W(NROW,LAST), X(NROW), T                         
C                                                                       
      IF ( NROW .EQ. 1 )                            RETURN              
      LSTEP = MIN0( NROW-1 , LAST )                                     
      DO 20 K = 1, LSTEP                                                
           KP1 = K + 1                                                  
           IP = IPIVOT(K)                                               
           T = X(IP)                                                    
           X(IP) = X(K)                                                 
           X(K) = T                                                     
           IF ( T .EQ. 0.D0 )                       GO TO 20            
           DO 10 I = KP1, NROW                                          
   10         X(I) = X(I) + W(I,K) * T                                  
   20 CONTINUE                                                          
   30 RETURN                                                            
      END                                                               
      SUBROUTINE SUBBAK ( W, NROW, NCOL, LAST, X )                      
C                                                                       
C*********************************************************************  
C                                                                       
C     carries out backsubstitution for current block.                   
C                                                                       
C    parameters                                                         
C       w, ipivot, nrow, ncol, last  are as on return from factrb.      
C       x(1),...,x(ncol)  contains, on input, the right side for the    
C               equations in this block after backsubstitution has been 
C               carried up to but not including equation (last).        
C               means that x(j) contains the right side of equation (j) 
C               as modified during elimination, j=1,...,last, while     
C               for j .gt. last, x(j) is already a component of the     
C               solution vector.                                        
C       x(1),...,x(ncol) contains, on output, the components of the     
C               solution corresponding to the present block.            
C                                                                       
C*********************************************************************  
C                                                                       
      INTEGER  LAST,  I,J,K,KM1,LM1,LP1                                 
      DOUBLE PRECISION W(NROW,NCOL),X(NCOL), T                          
C                                                                       
      LP1 = LAST + 1                                                    
      IF ( LP1 .GT. NCOL )                          GO TO 30            
      DO 20 J = LP1, NCOL                                               
         T = - X(J)                                                     
         IF ( T .EQ. 0.D0 )                         GO TO 20            
         DO 10 I = 1, LAST                                              
   10       X(I) = X(I) + W(I,J) * T                                    
   20 CONTINUE                                                          
   30 IF ( LAST .EQ. 1 )                            GO TO 60            
      LM1 = LAST - 1                                                    
      DO 50 KB = 1, LM1                                                 
        KM1 = LAST - KB                                                 
        K = KM1 + 1                                                     
        X(K) = X(K)/W(K,K)                                              
        T = - X(K)                                                      
        IF ( T .EQ. 0.D0 )                          GO TO 50            
        DO 40 I = 1, KM1                                                
   40     X(I) = X(I) + W(I,K) * T                                      
   50 CONTINUE                                                          
   60 X(1) = X(1)/W(1,1)                                                
      RETURN                                                            
      END                                                               
      SUBROUTINE SUBFRC ( W, NROW, LAST, X )                            
C                                                                       
C********************************************************************** 
C                                                                       
C     carries out the forward pass of substitution for the current      
C     block, i.e., the action on the right side corresponding to the    
C     elimination carried out in  factrc  for this block.               
C                                                                       
C    parameters                                                         
C       w, nrow, last  are as on return from factrc.                    
C       x(j)  is expected to contain, on input, the right side of j-th  
C             equation for this block, j=1,...,nrow.                    
C       x(j)  contains, on output, the appropriately modified right     
C             side of equation (j) in this block, j=1,...,last and      
C             for j=last+1,...,nrow.                                    
C                                                                       
C*********************************************************************  
C                                                                       
      INTEGER K,KP1,LSTEP                                               
      DOUBLE PRECISION W(NROW,LAST), X(NROW), T                         
C                                                                       
      IF ( LAST .EQ. 1 )                            GO TO 30            
      LSTEP = LAST - 1                                                  
      DO 20 K = 1, LSTEP                                                
           X(K) = X(K)/W(K,K)                                           
           T = - X(K)                                                   
           IF ( T .EQ. 0.D0 )                       GO TO 20            
           KP1 = K + 1                                                  
           DO 10 I = KP1, NROW                                          
   10         X(I) = X(I) + W(I,K) * T                                  
   20 CONTINUE                                                          
   30 X(LAST) = X(LAST)/W(LAST,LAST)                                    
      RETURN                                                            
      END                                                               
      SUBROUTINE SUBBKC ( W, IPIVOT, NROW, NCOL, LAST, X )              
C                                                                       
C*********************************************************************  
C                                                                       
C     carries out backsubstitution for current block.                   
C                                                                       
C    parameters                                                         
C       w, ipivot, nrow, ncol, last  are as on return from factrc.      
C       x(1),...,x(ncol)  contains, on input, the right side for the    
C               equations in this block after backsubstitution has been 
C               carried up to but not including equation (last).        
C               means that x(j) contains the right side of equation (j) 
C               as modified during elimination, j=1,...,last, while     
C               for j .gt. last, x(j) is already a component of the     
C               solution vector.                                        
C       x(1),...,x(ncol) contains, on output, the components of the     
C               solution corresponding to the present block.            
C                                                                       
C*********************************************************************  
C                                                                       
      INTEGER  IPIVOT(NROW),LAST,  I,J,K,KM1,LM1,LP1                    
      DOUBLE PRECISION W(NROW,NCOL),X(NCOL), T                          
C                                                                       
      LP1 = LAST + 1                                                    
      IF ( LP1 .GT. NCOL )                          GO TO 30            
      DO 20 J = LP1, NCOL                                               
         T = X(J)                                                       
         IF ( T .EQ. 0.D0 )                         GO TO 20            
         DO 10 I = 1, LAST                                              
   10       X(I) = X(I) + W(I,J) * T                                    
   20 CONTINUE                                                          
C                                                                       
   30 IF ( LAST .EQ. 1 )                            GO TO 70            
      LM1 = LAST - 1                                                    
      DO 60 KB = 1, LM1                                                 
         KM1 = LAST - KB                                                
         K = KM1 + 1                                                    
         T = X(K)                                                       
         IF ( T .EQ. 0.D0 )                         GO TO 50            
         DO 40 I = 1, KM1                                               
   40       X(I) = X(I) + W(I,K) * T                                    
   50    IP = IPIVOT(K)                                                 
         X(K) = X(IP)                                                   
         X(IP) = T                                                      
   60 CONTINUE                                                          
   70 IP = IPIVOT(1)                                                    
      T = X(1)                                                          
      X(1) = X(IP)                                                      
      X(IP) = T                                                         
      RETURN                                                            
      END                                                               
      SUBROUTINE BLKDET( BLOKS, INTEGS, NBLOKS, IPIVOT, MSTAR, IZSAVE,  
     1           INDPAR, INDNEW, INDOLD, DETPAR, DETNEW, DETOLD )       
C                                                                       
C********************************************************************** 
C                                                                       
C     calls subroutines  bdet1, bdet2, and getcol .                     
C                                                                       
C     supervises the calculation of the determinands det1 and det2      
C     of a linear blockdiagonal system, with the plu factorization      
C     already generated in  fcblok .  individual sub-blocks of          
C     equations are treated via  bdet1  and  bdet2 .                    
C                                                                       
C    parameters                                                         
C       bloks, integs, nbloks, ipivot    are as on return from fcblok.  
C       indpar : column of embedding parameter specified by user.       
C       indold : column of embedding parameter chosen by pivoting       
C                in the former step.                                    
C       indnew : column of embedding parameter chosen by pivoting       
C                in the current step.                                   
C       detpar : determinant of jacobian skipping column indpar.        
C       detold : determinant of jacobian skipping column indold.        
C       detnew : determinant of jacobian skipping column indnew.        
C                                                                       
C*********************************************************************  
C                                                                       
      INTEGER INTEGS(3,NBLOKS),IPIVOT(1),MSTAR,IZSAVE,INDPAR,INDOLD,    
     1       INDNEW, I,INDEX,INDEXX,ISKNEW,ISKOLD,ISKPAR,               
     2       NCOL,NROW,LAST                                             
      DOUBLE PRECISION BLOKS(1),DETPAR(2),DETNEW(2),DETOLD(2), DETFIX(2)
C                                                                       
C...  calculate determinant in blockwise manner                         
C                                                                       
      DETFIX(1) = 1.D0                                                  
      DETFIX(2) = 0.D0                                                  
      ISIGI = MSTAR * NBLOKS                                            
      INDEX = 1                                                         
      INDEXX = 1                                                        
      DO 10 I = 1, NBLOKS                                               
           NROW = INTEGS(1,I)                                           
           LAST = INTEGS(3,I)                                           
           LSIDE = NROW - MSTAR                                         
           IF ( I .EQ. NBLOKS )  LSIDE = IZSAVE - 1                     
           ISIGI = ISIGI + LSIDE * MSTAR                                
           CALL BDET1 ( BLOKS(INDEX), IPIVOT(INDEXX), NROW, LAST,       
     1                  DETFIX )                                        
           INDEX = NROW * INTEGS(2,I) + INDEX                           
   10 INDEXX = INDEXX + LAST                                            
C                                                                       
C...  correct sign of determinant with respect to                       
C...  locations of boundary conditions                                  
C                                                                       
      ISIGI = MOD ( ISIGI, 2 )                                          
      IF ( ISIGI .EQ. 1 )  DETFIX(1) = - DETFIX(1)                      
C                                                                       
C...  treat last subblock                                               
C                                                                       
      NCOL = INTEGS(2,NBLOKS)                                           
      INDEX = INDEX - NROW * NCOL                                       
      INDEXX = INDEXX - LAST                                            
C                                                                       
C...  determine columns of jacobian to be skipped                       
C                                                                       
      CALL GETCOL ( IPIVOT(INDEXX), LAST, NCOL, MSTAR,                  
     1              INDNEW, INDOLD, INDPAR,                             
     2              ISKNEW, ISKOLD, ISKPAR,                             
     3              ISGNEW, ISGOLD, ISGPAR )                            
C                                                                       
C...  complete determinant calculations wrt. embedding parameters       
C                                                                       
      DETPAR(1) = DFLOAT(ISGPAR) * DETFIX(1)                            
      DETPAR(2) = DETFIX(2)                                             
      CALL BDET2 ( BLOKS(INDEX), NROW, NCOL, ISKPAR, DETPAR )           
C                                                                       
      IF ( INDOLD .EQ. INDPAR )                     GO TO 20            
      DETOLD(1) = DFLOAT(ISGOLD) * DETFIX(1)                            
      DETOLD(2) = DETFIX(2)                                             
      CALL BDET2 ( BLOKS(INDEX), NROW, NCOL, ISKOLD, DETOLD )           
      GO TO 30                                                          
   20 DETOLD(1) = DETPAR(1)                                             
      DETOLD(2) = DETPAR(2)                                             
   30 CONTINUE                                                          
C                                                                       
      IF ( INDNEW .EQ. INDPAR )                     GO TO 40            
      DETNEW(1) = DFLOAT(ISGNEW) * DETFIX(1)                            
      DETNEW(2) = DETFIX(2)                                             
      CALL BDET2 ( BLOKS(INDEX), NROW, NCOL, ISKNEW, DETNEW )           
      GO TO 50                                                          
   40 DETNEW(1) = DETPAR(1)                                             
      DETNEW(2) = DETPAR(2)                                             
   50 CONTINUE                                                          
C                                                                       
      RETURN                                                            
      END                                                               
      SUBROUTINE BDET1 ( W, IPIVOT, NROW, LAST, DET )                   
C                                                                       
C********************************************************************** 
C                                                                       
C     calculates the determinant of a decomposed matrix.                
C                                                                       
C    parameters                                                         
C       w, ipivot, nrow, last  are as on return from factrb.            
C       det(1) the mantissa of the determinant.                         
C       det(2) the exponent of the determinant.                         
C                                                                       
C*********************************************************************  
C                                                                       
      INTEGER IPIVOT(LAST), I                                           
      DOUBLE PRECISION W(NROW,LAST),DET(2), TEN                         
C                                                                       
C...  calculate determinant and sign of permutation                     
C                                                                       
      TEN = 10.D0                                                       
      DO 40 I = 1, LAST                                                 
           DET(1) = W(I,I) * DET(1)                                     
           IF ( IPIVOT(I) .NE. I )  DET(1) = - DET(1)                   
           IF ( DET(1) .EQ. 0.D0 )                  RETURN              
   10      IF ( DABS(DET(1)) .GE. 1.D0 )            GO TO 20            
                DET(1) = TEN * DET(1)                                   
                DET(2) = DET(2) - 1.D0                                  
           GO TO 10                                                     
   20      IF ( DABS(DET(1)) .LT. TEN )             GO TO 30            
                DET(1) = DET(1) / TEN                                   
                DET(2) = DET(2) + 1.D0                                  
           GO TO 20                                                     
   30      CONTINUE                                                     
   40 CONTINUE                                                          
      RETURN                                                            
      END                                                               
      SUBROUTINE BDET2 ( W, NROW, NCOL, ISKIP, DET )                    
C                                                                       
C*********************************************************************  
C                                                                       
C     calculates determinant of a hessenberg matrix.                    
C                                                                       
C    parameters                                                         
C       w, nrow, ncol  are as on return from factrb.                    
C       iskip  this column of  w  is to be skipped.                     
C                                                                       
C*********************************************************************  
C                                                                       
      INTEGER NROW,NCOL,ISKIP,  I,ISKIP1,J,JB,JM1                       
      DOUBLE PRECISION W(NROW,NCOL),DET(2), DIAG(80),C,S,TEN,X          
C                                                                       
C...   downdate hessenberg matrix by plane rotations                    
C                                                                       
      IF ( ISKIP .EQ. NCOL )                        RETURN              
      DO 10 I = ISKIP, NROW                                             
         DIAG(I) = - W(I,NCOL)                                          
   10 CONTINUE                                                          
      IF ( ISKIP .EQ. NROW )                        GO TO 40            
      ISKIP1 = ISKIP + 1                                                
      DO 30 JB = ISKIP1, NROW                                           
         J = NCOL + ISKIP - JB                                          
         JM1 = J - 1                                                    
         X = DIAG(J)                                                    
         S = 1.D0 / DSQRT(X*X + 1.0D0)                                  
         C = X * S                                                      
         DIAG(J) = C * X + S                                            
         DO 20 I = ISKIP, JM1                                           
            DIAG(I) = - C * W(I,J) - S * DIAG(I)                        
   20    CONTINUE                                                       
   30 CONTINUE                                                          
C                                                                       
C...  calculate determinant                                             
C                                                                       
   40 TEN = 10.D0                                                       
      DO 80 I = ISKIP, NROW                                             
           DET(1) = DIAG(I) * DET(1)                                    
           IF ( DET(1) .EQ. 0.D0 )                  RETURN              
   50      IF ( DABS(DET(1)) .GE. 1.D0 )            GO TO 60            
                DET(1) = TEN * DET(1)                                   
                DET(2) = DET(2) - 1.D0                                  
           GO TO 50                                                     
   60      IF ( DABS(DET(1)) .LT. TEN )             GO TO 70            
                DET(1) = DET(1) / TEN                                   
                DET(2) = DET(2) + 1.D0                                  
           GO TO 60                                                     
   70      CONTINUE                                                     
   80 CONTINUE                                                          
      RETURN                                                            
      END                                                               
      SUBROUTINE GETCOL ( IPIVOT, LAST, NCOL, MSTAR,                    
     1                    INDNEW, INDOLD, INDPAR,                       
     2                    ISKNEW, ISKOLD, ISKPAR,                       
     3                    ISGNEW, ISGOLD, ISGPAR )                      
C                                                                       
C*********************************************************************  
C                                                                       
C     find location of valid embedding parameters.                      
C                                                                       
C    parameters                                                         
C       ipivot, last  are as on return from factrc.                     
C       mstar  number of first order differential equations.            
C       indnew, indold  index of new and old embedding parameter.       
C       isknew, iskold, iskpar  columns of new, old, and real           
C              embedding parameter, respectively.                       
C                                                                       
C*********************************************************************  
C                                                                       
      INTEGER IPIVOT(LAST),MSTAR,NCOL,INDNEW,INDOLD,INDPAR,             
     1        ISKNEW,ISKOLD,ISKPAR, IPIV(80),I,IB,IPJ,J                 
C                                                                       
C...  compute inverse of column permutation                             
C                                                                       
      DO 10 I = 1, NCOL                                                 
   10 IPIV(I) = I                                                       
      DO 20 I = 1, LAST                                                 
         IB = NCOL - I                                                  
         J = IPIVOT(IB)                                                 
         IPJ = IPIV(J)                                                  
         IPIV(J) = IPIV(IB)                                             
         IPIV(IB) = IPJ                                                 
   20 CONTINUE                                                          
C                                                                       
C...  determine column of embedding parameters                          
C                                                                       
      ISKPAR = IPIV(INDPAR)                                             
      ISKOLD = IPIV(INDOLD)                                             
      ISKNEW = IPIV(INDNEW)                                             
C                                                                       
C...  compute column permutation                                        
C                                                                       
      DO 30 I = 1, NCOL                                                 
   30 IPIV(I) = I                                                       
      DO 40 I = 1, LAST                                                 
         J = IPIVOT(I)                                                  
         IPJ = IPIV(J)                                                  
         IPIV(J) = IPIV(I)                                              
         IPIV(I) = IPJ                                                  
   40 CONTINUE                                                          
C                                                                       
C...  correct sign of determinants                                      
C                                                                       
      ISGPAR = 1                                                        
      ISGOLD = 1                                                        
      ISGNEW = 1                                                        
      DO 50 I = 1, NCOL                                                 
        IPJ = IPIV(I)                                                   
        IF ( (INDPAR-IPJ)*(ISKPAR-I) .LT. 0 )  ISGPAR = - ISGPAR        
        IF ( (INDOLD-IPJ)*(ISKOLD-I) .LT. 0 )  ISGOLD = - ISGOLD        
        IF ( (INDNEW-IPJ)*(ISKNEW-I) .LT. 0 )  ISGNEW = - ISGNEW        
   50 CONTINUE                                                          
      RETURN                                                            
      END                                                               
